Package: pcaMethods
Maintainer: Henning Redestig <henning.red@gmail.com>
License: GPL (>= 3)
Title: A collection of PCA methods
LinkingTo: Rcpp
LazyLoad: Yes
Author: Wolfram Stacklies, Henning Redestig, Kevin Wright
Authors@R: 
  c(person(given="Wolfram",
           family="Stacklies",
           role=c("aut")),
    person(given="Henning",
           family="Redestig",
           email="henning.red@gmail.com",
           comment=c(ORCID="0000-0003-2130-9288"),
           role=c("aut","cre")),
    person(given="Kevin",
           family="Wright",
           comment=c(ORCID="0000-0002-0617-8673"),
           role="aut"))
SystemRequirements: Rcpp
Description: Provides Bayesian PCA, Probabilistic PCA, Nipals PCA,
    Inverse Non-Linear PCA and the conventional SVD PCA. A cluster
    based method for missing value estimation is included for
    comparison. BPCA, PPCA and NipalsPCA may be used to perform PCA
    on incomplete data as well as for accurate missing value
    estimation. A set of methods for printing and plotting the
    results is also provided. All PCA methods make use of the same
    data structure (pcaRes) to provide a common interface to the
    PCA results. Initiated at the Max-Planck Institute for
    Molecular Plant Physiology, Golm, Germany.
Version: 2.4.0
URL: https://github.com/hredestig/pcamethods
BugReports: https://github.com/hredestig/pcamethods/issues
Encoding: UTF-8
Depends: Biobase, methods
Imports: BiocGenerics, Rcpp (>= 0.11.3), MASS
Suggests: matrixStats, lattice, ggplot2
Collate: 'derrorHierarchic.R' 'errorHierarchic.R' 'AllClasses.R'
        'AllGenerics.R' 'BPCA_dostep.R' 'BPCA_initmodel.R' 'bpca.R'
        'checkData.R' 'forkNlpcaNet.R' 'kEstimate.R' 'kEstimateFast.R'
        'lineSearch.R' 'llsImpute.R' 'methods-ExpressionSet.R'
        'methods-nniRes.R' 'methods-pcaRes.R' 'nipalsPca.R' 'nlpca.R'
        'optiAlgCgd.R' 'orth.R' 'pca.R' 'pcaMethods-package.R' 'ppca.R'
        'prep.R' 'repmat.R' 'robustPca.R' 'sortFeatures.R'
        'svdImpute.R' 'vector2matrices.R' 'xval.R'
Packaged: 2026-04-28 23:50:20 UTC; biocbuild
biocViews: Bayesian
RoxygenNote: 6.1.1
git_url: https://git.bioconductor.org/packages/pcaMethods
git_branch: RELEASE_3_23
git_last_commit: 4b5f83b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes

# Generated by roxygen2: do not edit by hand

S3method(biplot,pcaRes)
S3method(dim,pcaRes)
S3method(fitted,pcaRes)
S3method(loadings,pcaRes)
S3method(plot,pcaRes)
S3method(predict,pcaRes)
S3method(residuals,pcaRes)
S3method(scores,pcaRes)
S3method(summary,pcaRes)
export(Q2)
export(RnipalsPca)
export(asExprSet)
export(bpca)
export(checkData)
export(cvseg)
export(kEstimate)
export(kEstimateFast)
export(listPcaMethods)
export(llsImpute)
export(nipalsPca)
export(nlpca)
export(nni)
export(pca)
export(plotPcs)
export(ppca)
export(prep)
export(robustPca)
export(robustSvd)
export(showNniRes)
export(showPcaRes)
export(svdImpute)
export(svdPca)
exportClasses(nlpcaNet)
exportClasses(nniRes)
exportClasses(pcaRes)
exportMethods(DModX)
exportMethods(R2VX)
exportMethods(R2cum)
exportMethods(center)
exportMethods(centered)
exportMethods(completeObs)
exportMethods(cvstat)
exportMethods(leverage)
exportMethods(loadings)
exportMethods(method)
exportMethods(nObs)
exportMethods(nP)
exportMethods(nPcs)
exportMethods(nVar)
exportMethods(nmissing)
exportMethods(sDev)
exportMethods(scaled)
exportMethods(scl)
exportMethods(scores)
exportMethods(slplot)
exportMethods(wasna)
import(Biobase)
import(BiocGenerics)
import(methods)
importFrom(Rcpp,evalCpp)
useDynLib(pcaMethods)

# pcaMethods

R package for performing
[principal component analysis PCA](https://en.wikipedia.org/wiki/Principal_component_analysis)
with applications to missing value imputation. Provides a single
interface to performing PCA using

- **SVD:** a fast method which is also the standard method in R but
  which is not applicable for data with missing values.
- **NIPALS:** an iterative fast method which is applicable also to
  data with missing values.
- **PPCA:** Probabilistic PCA which is applicable also on data with
  missing values. Missing value estimation is typically better than
  NIPALS but also slower to compute and uses more memory. A port to R
  of the
  [implementation by Jakob Verbeek](http://lear.inrialpes.fr/~verbeek/software.php).
- **BPCA:** Bayesian PCA which performs very well in the presence of
  missing values but is slower than PPCA. A port of the
  [matlab implementation by Shigeyuki Oba](http://ishiilab.jp/member/oba/tools/BPCAFill.html).
- **NLPCA:** Non-linear PCA which can find curves in data and in
  presence of such can perform accurate missing value
  estimation. [Matlab port of the implementation by Mathias Scholz](http://www.nlpca.org/).


[pcaMethods is a Bioconductor package](http://www.bioconductor.org/packages/release/bioc/html/pcaMethods.html)
and you can install it by

```R
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("pcaMethods")
```

## Documentation

```R
browseVignettes("pcaMethods")
?<function_name>
```

### R code from vignette source 'outliers.Rnw'

###################################################
### code chunk number 1: outliers.Rnw:59-60
###################################################
library(pcaMethods)


###################################################
### code chunk number 2: outliers.Rnw:62-67
###################################################
data(metaboliteDataComplete)
mdc          <- scale(metaboliteDataComplete, center=TRUE, scale=FALSE)
cond         <- runif(length(mdc)) < 0.05
mdcOut       <- mdc
mdcOut[cond] <- 10


###################################################
### code chunk number 3: outliers.Rnw:70-73
###################################################
resSvd       <- pca(mdc, method="svd", nPcs=5, center=FALSE)
resSvdOut    <- pca(mdcOut, method="svd", nPcs=5, center=FALSE)
resRobSvd    <- pca(mdcOut, method="robustPca", nPcs=5, center=FALSE)


###################################################
### code chunk number 4: outliers.Rnw:77-80
###################################################
mdcNa        <- mdc
mdcNa[cond]  <- NA
resPPCA      <- pca(mdcNa, method="ppca", nPcs=5, center=FALSE)


###################################################
### code chunk number 5: outliers.Rnw:88-97
###################################################
par(mfrow=c(2,2))
plot(loadings(resSvd)[,1], loadings(resSvdOut)[,1], 
xlab="Loading 1 SVD", ylab="Loading 1 SVD with outliers")
plot(loadings(resSvd)[,1], loadings(resRobSvd)[,1],
xlab="Loading 1 SVD", ylab="Loading 1 robustSVD with outliers")
plot(loadings(resSvd)[,1], loadings(resPPCA)[,1],
xlab="Loading 1 SVD", ylab="Loading 1 PPCA with outliers=NA")
plot(loadings(resRobSvd)[,1], loadings(resPPCA)[,1],
xlab="Loading 1 robust SVD with outliers", ylab="Loading 1 svdImpute with outliers=NA")

##' Vector with current valid PCA methods
##' @title List PCA methods
##' @param which the type of methods to get. E.g. only  get the PCA
##' methods based on the classical model where the fitted data is a
##' direct multiplication of scores and loadings. 
##' @return A character vector with the current methods for doing PCA
##' @export 
##' @author Henning Redestig
listPcaMethods <- function(which=c("all", "linear", "nonlinear")) {
  switch(match.arg(which),
         all={
           return(c("svd", "nipals", "rnipals", "bpca", "ppca",
                    "svdImpute", "robustPca", "nlpca", "llsImpute",
                    "llsImputeAll"))
         },
         linear={
           return(c("svd", "nipals", "rnipals", "bpca", "ppca",
                    "svdImpute", "robustPca"))
         },
         nonlinear={
           return("nlpca")
         })
}

##' Perform PCA on a numeric matrix for visualisation, information
##' extraction and missing value imputation.
##'
##' This method is wrapper function for the following set of pca
##' methods:
##'
##' \describe{\item{svd:}{Uses classical \code{prcomp}. See
##' documentation for \code{\link{svdPca}}.}
##' 
##' \item{nipals:}{An iterative method capable of handling small
##' amounts of missing values. See documentation for
##' \code{\link{nipalsPca}}.}
##' 
##' \item{rnipals:}{Same as nipals but implemented in R.}
##' 
##' \item{bpca:}{An iterative method using a Bayesian model to handle
##' missing values. See documentation for \code{\link{bpca}}.}
##' 
##' \item{ppca:}{An iterative method using a probabilistic model to
##' handle missing values. See documentation for \code{\link{ppca}}.}
##' 
##' \item{svdImpute:}{Uses expectation maximation to perform SVD PCA
##' on incomplete data. See documentation for
##' \code{\link{svdImpute}}.}}
##' 
##' Scaling and centering is part of the PCA model and handled by
##' \code{\link{prep}}.
##' @title Perform principal component analysis
##' @param object Numerical matrix with (or an object coercible to
##' such) with samples in rows and variables as columns. Also takes
##' \code{ExpressionSet} in which case the transposed expression
##' matrix is used. Can also be a data frame in which case all
##' numberic variables are used to fit the PCA.
##' @param method One of the methods reported by
##' \code{listPcaMethods()}. Can be left missing in which case the
##' \code{svd} PCA is chosen for data wihout missing values and
##' \code{nipalsPca} for data with missing values
##' @param nPcs Number of principal components to calculate.
##' @param scale Scaling, see \code{\link{prep}}.
##' @param center Centering, see \code{\link{prep}}.
##' @param completeObs Sets the \code{completeObs} slot on the
##' resulting \code{pcaRes} object containing the original data with
##' but with all NAs replaced with the estimates.
##' @param subset A subset of variables to use for calculating the
##' model. Can be column names or indices.
##' @param cv character naming a the type of cross-validation
##' to be performed. 
##' @param ... Arguments to \code{\link{prep}}, the chosen pca
##' method and \code{\link{Q2}}.
##' @return A \code{pcaRes} object.
##' @references
##' Wold, H. (1966) Estimation of principal components and
##' related models by iterative least squares. In Multivariate
##' Analysis (Ed., P.R. Krishnaiah), Academic Press, NY, 391-420.
##'   
##' Shigeyuki Oba, Masa-aki Sato, Ichiro Takemasa, Morito Monden,
##' Ken-ichi Matsubara and Shin Ishii.  A Bayesian missing value
##' estimation method for gene expression profile
##' data. \emph{Bioinformatics, 19(16):2088-2096, Nov 2003}.
##' 
##' Troyanskaya O. and Cantor M. and Sherlock G. and Brown P. and
##' Hastie T. and Tibshirani R. and Botstein D. and Altman RB.  -
##' Missing value estimation methods for DNA microarrays.
##' \emph{Bioinformatics. 2001 Jun;17(6):520-5}.
##' @seealso \code{\link{prcomp}}, \code{\link{princomp}},
##' \code{\link{nipalsPca}}, \code{\link{svdPca}}
##' @examples
##' data(iris)
##' ##  Usually some kind of scaling is appropriate
##' pcIr <- pca(iris, method="svd", nPcs=2)
##' pcIr <- pca(iris, method="nipals", nPcs=3, cv="q2")
##' ## Get a short summary on the calculated model
##' summary(pcIr)
##' plot(pcIr)
##' ## Scores and loadings plot
##' slplot(pcIr, sl=as.character(iris[,5]))
##' 
##' ## use an expressionset and ggplot
##' data(sample.ExpressionSet)
##' pc <- pca(sample.ExpressionSet)
##' df <- merge(scores(pc), pData(sample.ExpressionSet), by=0)
##' library(ggplot2)
##' ggplot(df, aes(PC1, PC2, shape=sex, color=type)) +
##'   geom_point() +
##'   xlab(paste("PC1", pc@R2[1] * 100, "% of the variance")) +
##'   ylab(paste("PC2", pc@R2[2] * 100, "% of the variance"))
##' @export
##' @keywords multivariate
##' @author Wolfram Stacklies, Henning Redestig
pca <- function(object, method, nPcs=2, 
                scale=c("none", "pareto", "vector", "uv"),
                center=TRUE, completeObs=TRUE, subset=NULL,
                cv=c("none","q2"), ...) {
  if(inherits(object, 'data.frame')) {
    num <- vapply(object, is.numeric, logical(1))
    if(sum(num) < 2)
      stop('no numeric data in supplied data.frame')
    Matrix <- as.matrix(object[,num])
  }
  else if(inherits(object, "ExpressionSet")) {
    Matrix <- t(exprs(object))
  } else
  Matrix <- as.matrix(object, rownames.force=TRUE)

  if(!is.null(subset)) 
    Matrix <- Matrix[,subset]
  
  cv <- match.arg(cv)
  scale <- match.arg(scale)

  if (nPcs > ncol(Matrix)) {
    warning("more components than matrix columns requested")
    nPcs <- min(dim(Matrix))
  }
  if (nPcs > nrow(Matrix)) {
    warning("more components than matrix rows requested")
    nPcs <- min(dim(Matrix))
  }

  if (!checkData(Matrix, verbose=interactive()))
    stop("Invalid data format.",
         "Run checkData(data, verbose=TRUE) for details")

  missing <- is.na(Matrix)

  if(missing(method)) {
    if(any(missing))
      method <- 'nipals'
    else
      method <- 'svd'
  }
  if(any(missing) & method == 'svd') {
    warning('data has missing values using nipals instead of user requested svd')
    method <- 'nipals'
  }
  method <- match.arg(method, choices=listPcaMethods())
  
  prepres <- prep(Matrix, scale=scale, center=center, simple=FALSE, ...)

  switch(method,
         svd={
           res <- svdPca(prepres$data, nPcs=nPcs,...)   
         },
         nipals={
           res <- nipalsPca(prepres$data, nPcs=nPcs, ...) 
         },
         rnipals={
           res <- RnipalsPca(prepres$data,  nPcs=nPcs, ...) 
         },
         bpca={
           res <- bpca(prepres$data, nPcs=nPcs, ...) 
         },
         ppca={
           res <- ppca(prepres$data, nPcs=nPcs, ...) 
         },
         svdImpute={
           res <- svdImpute(prepres$data, nPcs=nPcs, ...) 
         },
         robustPca={
           res <- robustPca(prepres$data,  nPcs=nPcs, ...) 
         },
         nlpca={
           res <- nlpca(prepres$data, nPcs=nPcs, ...)
         })

  nPcs <- ncol(res@scores)
  if(is.null(scores(res)) | is.null(loadings(res)) |
     is.null(R2cum(res)) | is.null(method(res)))
    stop(paste("bad result from pca method", method))

  colnames(res@scores) <- paste("PC", 1:nPcs, sep="")
  rownames(res@scores) <- rownames(Matrix)
  if(all(dim(loadings(res)) == c(ncol(Matrix), nPcs))) {
    colnames(res@loadings) <- paste("PC", 1:nPcs, sep="")
    rownames(res@loadings) <- colnames(Matrix)
  }
  if(!is.null(subset))
    res@subset <- subset
  res@missing <- missing
  res@nPcs <- nPcs
  res@nObs <- nrow(Matrix)
  res@nVar <- ncol(Matrix)
  res@sDev <- apply(scores(res), 2, sd)
  res@center <- prepres$center
  res@centered <- center
  res@scale <- prepres$scale
  res@scaled <- scale 
  res@R2 <- res@R2cum[1]
  if(length(res@R2cum) > 1)
    res@R2 <- c(res@R2, diff(res@R2cum))

  if (completeObs) {
    cObs <- Matrix
    if(method %in% listPcaMethods("nonlinear"))
      cObs[missing] <- fitted(res, Matrix, pre=TRUE, post=TRUE)[missing]
    else
      cObs[missing] <- fitted(res, post=TRUE)[missing]
    res@completeObs <- cObs
  }
  if(cv == "q2")
    res@cvstat <- Q2(res, Matrix, nruncv=1, ...)

  return(res)
}

##' Wrapper function for imputation methods based on nearest neighbour
##' clustering. Currently llsImpute only.
##'
##' This method is wrapper function to llsImpute, See documentation
##' for \code{link{llsImpute}}.
##' @title Nearest neighbour imputation
##' @param object Numerical matrix with (or an object coercible to
##' such) with samples in rows and variables as columns. Also takes
##' \code{ExpressionSet} in which case the transposed expression
##' matrix is used.
##' @param method For convenience one can pass a large matrix but only
##' use the variable specified as subset. Can be colnames or indices.
##' @param subset Currently "llsImpute" only.
##' @param ... Further arguments to the chosen method.
##' @return A \code{clusterRes} object. Or a list containing a
##' clusterRes object as first and an ExpressionSet object as second
##' entry if the input was of type ExpressionSet.
##' @export
##' @seealso \code{\link{llsImpute}}, \code{\link{pca}}
##' @keywords multivariate
##' @examples
##' data(metaboliteData)
##' llsRes <- nni(metaboliteData, k=6, method="llsImpute", allGenes=TRUE)
##' @author Wolfram Stacklies
nni <- function(object, method=c("llsImpute"), subset=numeric(), ...) {
  isExprSet <- FALSE
  if(inherits(object, "ExpressionSet")) {
    set <- object
    isExprSet <- TRUE
    object <- t(exprs(object))
  }

  method <- match.arg(method)

  if ( !checkData(as.matrix(object), verbose=interactive()) )
    stop("Invalid data format, exiting...\n",
         "Run checkData(data, verbose=TRUE) for details\n")

  missing <- sum(is.na(object))
  if(length(subset) > 0)
    object <- object[,subset]

  res <- llsImpute(object, ...) 

  return(res)
}

##' A function that can be used to visualise many PCs plotted against
##' each other
##'
##' Uses \code{\link{pairs}} to provide side-by-side plots. Note that
##' this function only plots scores or loadings but not both in the
##' same plot.
##' @title Plot many side by side scores XOR loadings plots
##' @param object \code{pcaRes} a pcaRes object
##' @param pcs \code{numeric} which pcs to plot
##' @param type \code{character} Either "scores" or "loadings" for
##' scores or loadings plot respectively
##' @param sl \code{character} Text labels to plot instead of a point,
##' if NULL points are plotted instead of text
##' @param hotelling \code{numeric} Significance level for the
##' confidence ellipse. NULL means that no ellipse is drawn.
##' @param ... Further arguments to \code{\link{pairs}} on which this
##' function is based. 
##' @return None, used for side effect.
##' @seealso \code{prcomp}, \code{pca}, \code{princomp}, \code{slplot}
##' @export
##' @examples
##' data(iris)
##' pcIr <- pca(iris[,1:4], nPcs=3,  method="svd")
##' plotPcs(pcIr, col=as.integer(iris[,4]) + 1)
##' @keywords multivariate
##' @author Henning Redestig
plotPcs <- function(object,
                    pcs=1:nP(object), type=c("scores", "loadings"), sl=NULL,
                    hotelling=0.95,  ...) {
  type <- match.arg(type)

  panel <- function(x,y, ...) {
    abline(h=0, v=0, col="black")
    if(!is.null(hotelling)) {
      A <- length(pcs)
      el <- simpleEllipse(x, y, alfa=hotelling)
      lines(el)
    }
    if(is.null(sl))
      points(x, y, ...)
    else
      text(x, y, labels=sl,...)
  }

  switch(type,
         scores={
           labels <- paste("PC", pcs, "\n", "R^2 =", round(object@R2[pcs], 2))
           pairs(scores(object)[,pcs], labels=labels,
                 panel=panel, upper.panel=NULL,...)
         },
         loadings={
           if(method(object) == "nlpca")
             stop("Loadings plot not applicable for non-linear PCA")
           labels <- paste("PC", pcs, "\n", "R^2 =", round(object@R2[pcs], 2))
           pairs(loadings(object)[,pcs], labels=labels, panel=panel,
                 upper.panel=NULL, ...)
         })
}


##' A wrapper function for \code{prcomp} to deliver the result as a
##' \code{pcaRes} method. Supplied for  compatibility with  the rest
##' of the pcaMethods package.  It is not recommended to use this
##' function directely but rather to use  the \code{pca()} wrapper
##' function. 
##' @title Perform principal component analysis using singular value
##' decomposition
##' @param Matrix Pre-processed (centered and possibly scaled)
##' numerical matrix samples in rows and variables as columns. No
##' missing values allowed.
##' @param nPcs Number of components that should be extracted.
##' @param varLimit Optionally the ratio of variance that should be
##' explained. \code{nPcs} is ignored if varLimit < 1
##' @param verbose Verbose complaints to matrix structure
##' @param ... Only used for passing through arguments.
##' @return A \code{pcaRes} object.
##' @seealso \code{prcomp}, \code{princomp}, \code{pca}
##' @examples
##' data(metaboliteDataComplete)
##' mat <- prep(t(metaboliteDataComplete))
##' pc <- svdPca(mat, nPcs=2)
##' ## better use pca()
##' pc <- pca(t(metaboliteDataComplete), method="svd", nPcs=2)
##' \dontshow{stopifnot(sum((fitted(pc) - t(metaboliteDataComplete))^2, na.rm=TRUE) < 200)}
##' @export
##' @keywords multivariate
##' @author Henning Redestig
svdPca <- function(Matrix, nPcs=2, 
                   varLimit=1, verbose=interactive(), ...) {

  pcs <- prcomp(Matrix, center=FALSE, scale.=FALSE)
  imp <- summary(pcs)$importance
  if(varLimit < 1)
    nPcs <- sum(imp[3,] < varLimit) + 1

  res <- new("pcaRes")
  res@scores <- cbind(pcs$x[,1:nPcs])
  res@loadings <- cbind(pcs$rotation[,1:nPcs])
  res@R2cum <- imp[3,1:nPcs]
  res@varLimit <- varLimit
  res@method <- "svd"
  return(res)
}

##' Get a confidence ellipse for uncorrelated bivariate data
##'
##' As described in 'Introduction to multi and megavariate data analysis
##' using PCA and
##' PLS' by Eriksson et al. This produces very similar ellipse as
##' compared to the ellipse function the ellipse package except that
##' this function assumes that and y are uncorrelated (which they of
##' are if they are scores or loadings from a PCA). 
##' @title Hotelling's T^2 Ellipse
##' @param x first variable
##' @param y second variable
##' @param alfa confidence level of the circle
##' @param len Number of points in the circle
##' @seealso ellipse
##' @author Henning Redestig
##' @return A matrix with X and Y coordinates for the circle
simpleEllipse <- function(x, y, alfa=0.95, len=200) {
  N <- length(x)
  A <- 2
  mypi <- seq(0, 2 * pi, length=len)
  r1 <- sqrt(var(x) * qf(alfa, 2, N - 2) * (2*(N^2 - 1)/(N * (N - 2))))
  r2 <- sqrt(var(y) * qf(alfa, 2, N - 2) * (2*(N^2 - 1)/(N * (N - 2))))
  cbind(r1 * cos(mypi) + mean(x), r2 * sin(mypi) + mean(y))
}


# .onLoad <- function(libname, pkgname) {
#   require("methods")
# }

##' This is a simple estimator for the optimal number of componets
##' when applying PCA or LLSimpute for missing value estimation.  No
##' cross validation is performed, instead the estimation quality is
##' defined as Matrix[!missing] - Estimate[!missing]. This will give a
##' relatively rough estimate, but the number of iterations equals the
##' length of the parameter evalPcs.\cr Does not work with LLSimpute!!
##' As error measure the NRMSEP (see Feten et. al, 2005) or the Q2
##' distance is used.  The NRMSEP basically normalises the RMSD
##' between original data and estimate by the variable-wise
##' variance. The reason for this is that a higher variance will
##' generally lead to a higher estimation error.  If the number of
##' samples is small, the gene - wise variance may become an unstable
##' criterion and the Q2 distance should be used instead. Also if
##' variance normalisation was applied previously.
##' @title Estimate best number of Components for missing value estimation
##' @param Matrix \code{matrix} -- numeric matrix containing
##' observations in rows and variables in columns
##' @param method \code{character} -- a valid pca method (see
##' \code{\link{pca}}).
##' @param evalPcs \code{numeric} -- The principal components to use
##' for cross validation or cluster sizes if used with
##' llsImpute. Should be an array containing integer values,
##' eg. evalPcs = 1:10 or evalPcs = C(2,5,8).The NRMSEP is calculated
##' for each component.
##' @param em \code{character} -- The error measure. This can be
##' nrmsep or q2
##' @param allVariables \code{boolean} -- If TRUE, the NRMSEP is
##' calculated for all variables, If FALSE, only the incomplete ones
##' are included. You maybe want to do this to compare several methods
##' on a  complete data set.
##' @param verbose \code{boolean} -- If TRUE, the NRMSEP and the
##' variance are printed to the console each iteration.
##' @param ... Further arguments to \code{pca}
##' @return   \item{list}{Returns a list with the elements:
##' \itemize{
##' \item minNPcs - number of PCs for which the minimal average NRMSEP
##' was obtained
##' \item eError - an array of of size length(evalPcs). Contains the
##' estimation error for each number of
##' components.
##' \item evalPcs - The evaluated numbers of components or
##' cluster sizes  (the same as the evalPcs input parameter). }}
##' @seealso \code{\link{kEstimate}}.
##' @export
##' @examples
##' data(metaboliteData)
##' # Estimate best number of PCs with ppca for component 2:4
##' esti <- kEstimateFast(t(metaboliteData), method = "ppca", evalPcs = 2:4, em="nrmsep")
##' barplot(drop(esti$eError), xlab = "Components",ylab = "NRMSEP (1 iterations)")
##' # The best k value is:
##' print(esti$minNPcs)
##' @keywords multivariate
##' @author Wolfram Stacklies
kEstimateFast <- function(Matrix, method = "ppca", evalPcs = 1:3,
                          em = "nrmsep", allVariables = FALSE,
                          verbose = interactive(), ...) {

    method <-
      match.arg(method, c("ppca", "bpca", "svdImpute", "nipals", "nlpca"))
    em <- match.arg(em, c("nrmsep", "q2"))
    maxPcs <- max(evalPcs)
    lengthPcs <- length(evalPcs)
    missing <- is.na(Matrix)
    error <- array(0, lengthPcs)

     ## If the data is a data frame, convert it into a matrix
    Matrix <- as.matrix(Matrix, rownames.force=TRUE)
    if(maxPcs > (ncol(Matrix) - 1))
        stop("maxPcs exceeds matrix size, choose a lower value!")

     ## And now check if everything is right...
    if( !checkData(Matrix, verbose=interactive()) )
        stop("Invalid data format! Use checkData(Matrix, verbose = TRUE) for details.\n")

    if( (sum(is.na(Matrix)) == 0) && (allVariables == FALSE) )
        stop("No missing values. Maybe you want to set allVariables = TRUE. Exiting\n")

    iteration = 0
    for(nPcs in evalPcs) {
       iteration = iteration + 1
       if (method == "nlpca") {
           estimate <- fitted(pca(Matrix, nPcs = nPcs, verbose = FALSE,
                                  method = method, center = TRUE,...), Matrix, nPcs = nPcs)
       } else {
           estimate <- fitted(pca(Matrix, nPcs = nPcs, verbose = FALSE,
                                  method = method, center = TRUE,...), nPcs = nPcs)
       }

        if (em == "q2") {
            # The Q2 distance
            q2 <- 1 - sum((Matrix[!missing] - estimate[!missing])^2) / sum(Matrix[!missing]^2)
            error[iteration] <- q2
        } else {
            nrmsep <- 0
            for(i in 1:ncol(Matrix)) {
                nrmsep <- nrmsep + (
                    sum((Matrix[!missing[,i], i] - estimate[!missing[,i], i])^2) /
                    (sum(!missing[,i]) * var(Matrix[,i], na.rm = TRUE))
                )
            }
            nrmsep <- nrmsep / sum(apply(missing, 2, sum) > 0)
            error[iteration] <- nrmsep
        }
        if(verbose)
           cat("The", em, "for", evalPcs[iteration], "components is:", error[iteration], "\n")
    }
            
    ret <- list()
    if (em == "nrmsep") ret$bestNPcs <- evalPcs[which(error == min(error))]
    else ret$bestNPcs <- evalPcs[which(error == max(error))]
    ret$eError <- error
    ret$evalPcs <- evalPcs

    return(ret)
}

##' Later
##' @param nlnet the nlnet
##' @param trainIn training data
##' @param trainOut  fitted data
##' @return derror
##' @author Henning Redestig, Matthias Scholz
derrorHierarchic <- function(nlnet, trainIn, trainOut) {

  weights <- nlnet@weights$current()
  netDim <- dim(nlnet@net) 
  if(nlnet@inverse) {
    numElements <- nlnet@net[1] * dim(trainOut)[2]
    trainIn <- matrix(weights[1:numElements], nrow=nlnet@net[1], ncol=dim(trainOut)[2])
    wTrainIn <- weights[1:numElements,drop=FALSE]
    weights <- weights[(numElements + 1):length(weights), ,drop=FALSE]
  }

  weightMats <- vector2matrices(weights, nlnet@net)
  trainDim <- dim(trainIn)
  subnetNum <- length(nlnet@hierarchic$var)

  ## ******************************

  Epattern <- array(0, dim=c(dim(trainOut), subnetNum))
  nOut <- array(0, dim=c(sum(nlnet@net), trainDim[2], subnetNum))

  for(subnet in 1:subnetNum)
    nOut[1:trainDim[1],,subnet] <- eval(parse(text=paste(nlnet@fct[1], "(trainIn)")))

  if(nlnet@inverse)
    for(subnet in 1:subnetNum)
      nOut[nlnet@hierarchic$idx[,subnet]==0,,subnet] <- 0

  ## forward propagation
  for(subnet in 1:subnetNum) {
    if(nlnet@hierarchic$var[subnet] != 0) {
      sBias <- array(1, dim=c(1, trainDim[2]))
      for(i in 1:(netDim[2] - 1)) {
        if(i == 1)
          nBegin <- 1
        else
          nBegin <- sum(nlnet@net[1:(i-1)])+1
        sIn <- rbind(sBias, nOut[nBegin:sum(nlnet@net[1:i]),,subnet])

        sOut <- eval(parse(text=paste(nlnet@fct[i+1], "(weightMats[[i]] %*% sIn)")))

        if(i == (nlnet@hierarchic$layer - 1)) 
          sOut[nlnet@hierarchic$idx[,subnet]==0,] <- 0

        nOut[(sum(nlnet@net[1:i])+1):sum(nlnet@net[1:(i+1)]),,subnet] <- sOut
      }
      output <- nOut[(sum(nlnet@net[1:(length(nlnet@net)-1)])+1):dim(nOut)[1],,subnet]
      Epattern[,,subnet] <- output - trainOut
    }
  }

  ## error function
  Epattern <- Epattern^2
  Epattern[is.na(Epattern)] <- 0        #set the missing values to zero

  if(!is.null(nlnet@dataDist))
    for(subnet in 1:subnetNum)
      Epattern[,,subnet] <- Epattern[,,subnet] * nlnet@dataDist

  Eitemize <- apply(Epattern, 3, sum) * 0.5
  Etotal <- sum(nlnet@hierarchic$var * Eitemize)

  if(!is.null(nlnet@weightDecay))
    Etotal <- Etotal + nlnet@weightDecay * 0.5 * sum(weights^2)

  if(nlnet@inverse)
    Etotal <- Etotal + 0.01 * nlnet@weightDecay * 0.5 * sum(wTrainIn^2)

  ## back propagation
  nError <- array(0, dim=c(sum(nlnet@net), trainDim[2], subnet))
  dWeight <- vector(length=netDim[2] - 1, mode="list")
  wBp <- vector(length=netDim[2] - 1, mode="list")

  ## wBp is weights for back propagation
  for(u in 1:(netDim[2] - 1)) 
    wBp[[u]] <- weightMats[[u]][,2:(nlnet@net[u] + 1)] # cats the weights which belong to bias

  dw <- array(0, dim=c(length(weights), subnet))
  for(subnet in 1:subnetNum) {
    if(nlnet@hierarchic$var[subnet] != 0) {
      ## last layer
      sTmp <- nOut[(dim(nOut)[1]-nlnet@net[length(nlnet@net)]+1):dim(nOut)[1],,subnet]
      if(nlnet@fct[length(nlnet@fct)] == "tanh")
        eTmp <- (1 - sTmp^2) * (sTmp - trainOut) #prev trainOut - sTmp (fixed to get rid of sign change)
      else if(nlnet@fct[length(nlnet@fct)] == "linr")
        eTmp <- sTmp - trainOut #prev trainOut - sTmp (fixed to get rid of sign change)
      eTmp[is.na(eTmp)] <- 0
      if(!is.null(nlnet@dataDist))
        eTmp <- eTmp * nlnet@dataDist
      
      nError[(dim(nError)[1]-nlnet@net[length(nlnet@net)]+1):dim(nError)[1],,subnet] <- eTmp

      ## all other layers
      for(n in 1:(netDim[2] - 1)){
        i <- netDim[2]-n
        
        ## the if clause is to avoid 1:0 difference in R
        ## Matlab (1:0 => Empty matrix), R (1:0 => [1,0])
        if(i > 1)
          sTmp <- nOut[(sum(nlnet@net[1:(i-1)])+1):sum(nlnet@net[1:i]),,subnet]
        else
          sTmp <- nOut[1:sum(nlnet@net[1:i]),,subnet]

        if(i==(nlnet@hierarchic$layer-1))
          eTmp[nlnet@hierarchic$idx[,subnet]==0,] <- 0

        dWeight[[i]] <- tcrossprod(eTmp, rbind(sBias, sTmp)) #gradient

        if (nlnet@fct[i] == "tanh")
          eTmp <- (1 - sTmp^2) * crossprod(wBp[[i]],eTmp)
        else if (nlnet@fct[i] == "linr")
          eTmp <- crossprod(wBp[[i]], eTmp)

        ## the if clause is to avoid 1:0 difference in R
        if(i > 1) 
          nError[(sum(nlnet@net[1:(i - 1)]) + 1):sum(nlnet@net[1:i]), ,subnet] <- eTmp
        else 
          nError[1:sum(nlnet@net[1:i]), ,subnet] <- eTmp
      }
      dw[,subnet] <- unlist(dWeight)    #fixed sign change
      
    }
  }

  if(nlnet@inverse) {
    dw <- rbind(array(0, dim=c(numElements, subnetNum)), dw)
    for(subnet in 1:subnetNum) {
      eTmp <- array(nError[1:nlnet@net[1],,subnet], dim=c(nlnet@net[1], dim(nError)[2]))
      
      eTmp[nlnet@hierarchic$idx[,subnet] == 0,] <- 0

      dim(eTmp) <- NULL                 #a bit unsure if this is correct but seems to work
      dw[1:numElements,subnet] <- unlist(eTmp) #fixed sign change
    }
    ## weights <- rbind(cbind(rep(0, numElements)), cbind(weights)) #old: only weight decay for real weights
    weights <- rbind(cbind(0.01 * wTrainIn), cbind(weights)) #new 
  }

  dwTotal <- array(0, dim=dim(weights))
  for (subnet in 1:subnetNum)  {
    dwTotal <- dwTotal + nlnet@hierarchic$var[subnet] * dw[, subnet]
  }

  if(!is.null(nlnet@weightDecay))
    dwTotal <- dwTotal + nlnet@weightDecay * weights
  return(list(dwTotal=dwTotal, Etotal=Etotal, nError=nError, nOut=nOut))
}

##' PCA by non-linear iterative partial least squares
##'
##' Can be used for computing PCA on a numeric matrix using either the
##' NIPALS algorithm which is an iterative approach for estimating the
##' principal components extracting them one at a time. NIPALS can
##' handle a small amount of missing values. It is not recommended to
##' use this function directely but rather to use the pca() wrapper
##' function. There is a C++ implementation given as \code{nipalsPca}
##' which is faster.
##' @title NIPALS PCA implemented in R
##' @param Matrix Pre-processed (centered, scaled) numerical matrix
##' samples in rows and variables as columns. 
##' @param nPcs Number of components that should be extracted.
##' @param varLimit Optionally the ratio of variance that should be
##' explained. \code{nPcs} is ignored if varLimit < 1
##' @param maxSteps Defines how many iterations can be done before
##' algorithm should abort (happens almost exclusively when there were
##' some wrong in the input data).
##' @param threshold The limit condition for judging if the algorithm
##' has converged or not, specifically if a new iteration is done if
##' \eqn{(T_{old} - T)^T(T_{old} - T) > \code{limit}}.
##' @param verbose Show simple progress information.
##' @param ... Only used for passing through arguments.
##' @return A \code{pcaRes} object.
##' @references Wold, H. (1966) Estimation of principal components and
##' related models by iterative least squares. In Multivariate
##' Analysis (Ed., P.R. Krishnaiah), Academic Press, NY, 391-420.
##' @author Henning Redestig
##' @seealso \code{prcomp}, \code{princomp}, \code{pca}
##' @examples
##' data(metaboliteData)
##' mat <- prep(t(metaboliteData))
##' ## c++ version is faster
##' system.time(pc <- RnipalsPca(mat, method="rnipals", nPcs=2))
##' system.time(pc <- nipalsPca(mat, nPcs=2))
##' ## better use pca()
##' pc <- pca(t(metaboliteData), method="rnipals", nPcs=2)
##' \dontshow{stopifnot(sum((fitted(pc) - t(metaboliteData))^2, na.rm=TRUE) < 200)}
##' @keywords multivariate
##' @export
RnipalsPca <- function(Matrix, nPcs=2, 
                       varLimit=1,
                       maxSteps=5000, 
                       threshold=1e-6, verbose=interactive(), ...) {

  nVar <- ncol(Matrix)

  ##Find a good? starting column -- better way?
  startingColumn <- 1

  ## sum(c(NA, NA), na.rm=TRUE) is 0, but we want NA
  sum.na <- function(x){ ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}

  TotalSS <- sum(Matrix*Matrix, na.rm=TRUE)

  ph <- rep(0, nVar)
  R2cum <- rep(NA, nPcs)
  scores <- NULL
  loadings <- NULL
  anotherPc <- TRUE
  l <- 1
  
  while(anotherPc) {
    count <- 0                 #number of iterations done
    th <- Matrix[,startingColumn]   #first column is starting vector for th
    continue <- TRUE
    if(verbose) cat(paste("Calculating PC", l, ": ", sep=""))
    
    while(continue) {
      count <- count+1
      ph <- rep(0, nVar)

      ##Calculate loadings through LS regression
      ##Note: Matrix*th is column-wise multiplication
      tsize <- sum(th * th, na.rm=TRUE)
      ph <- apply(Matrix * (th / tsize), 2, sum.na)
      ##normalize ph based on the available values.
      psize <- sum(ph*ph, na.rm=TRUE)
      ph <- ph / sqrt(psize)
      
      ##Calculate scores through LS regression
      ##Trick: To get row-wise multiplication, use t(Matrix)*ph, then
      ##be sure to use apply(,2,) and NOT apply(,1,)!
      th.old <- th
      th <- apply(t(Matrix) * ph, 2, sum.na)
      
      ##Round up by calculating if convergence condition is met and
      ##checking if it seems to be an neverending loop.
      if (count > maxSteps) {
        stop("Too many iterations, quitting")
      }
      if (t(na.omit(th.old - th)) %*% (na.omit(th.old - th)) <= threshold) {
        continue = FALSE
      }
      if (verbose)cat("*")
    }
    if (verbose) cat(" Done\n")
    Matrix <- Matrix - (th %*% t(ph))
    scores <- cbind(scores, th)
    loadings <- cbind(loadings, ph)
    
    ##cumulative proportion of variance
    R2cum[l] <- 1 - (sum(Matrix*Matrix,na.rm=TRUE) / TotalSS)
    l <- l + 1
    if((!abs(varLimit - 1) < 1e-4 & R2cum[l - 1] >= varLimit) | l > nPcs) {
      anotherPc <- FALSE
      nPcs <- l - 1
    }
  }

  r <- new("pcaRes")
  r@scores <- scores
  r@loadings <- loadings
  r@R2cum <- R2cum
  r@varLimit <- varLimit
  r@method <- "rnipals"
  return(r)
}

##' PCA by non-linear iterative partial least squares
##'
##' Can be used for computing PCA on a numeric matrix using either the
##' NIPALS algorithm which is an iterative approach for estimating the
##' principal components extracting them one at a time. NIPALS can
##' handle a small amount of missing values. It is not recommended to
##' use this function directely but rather to use the pca() wrapper
##' function.
##' @title NIPALS PCA
##' @param Matrix Pre-processed (centered, scaled) numerical matrix
##' samples in rows and variables as columns.
##' @param nPcs Number of components that should be extracted.
##' @param varLimit Optionally the ratio of variance that should be
##' explained. \code{nPcs} is ignored if varLimit < 1
##' @param maxSteps Defines how many iterations can be done before
##' algorithm should abort (happens almost exclusively when there were
##' some wrong in the input data).
##' @param threshold The limit condition for judging if the algorithm
##' has converged or not, specifically if a new iteration is done if
##' \eqn{(T_{old} - T)^T(T_{old} - T) > \code{limit}}.
##' @param ... Only used for passing through arguments.
##' @return A \code{pcaRes} object.
##' @references Wold, H. (1966) Estimation of principal components and
##' related models by iterative least squares. In Multivariate
##' Analysis (Ed., P.R. Krishnaiah), Academic Press, NY, 391-420.
##' @author Henning Redestig
##' @seealso \code{prcomp}, \code{princomp}, \code{pca}
##' @examples
##' data(metaboliteData)
##' mat <- prep(t(metaboliteData))
##' pc <- nipalsPca(mat, nPcs=2)
##' ## better use pca()
##' pc <- pca(t(metaboliteData), method="nipals", nPcs=2)
##' \dontshow{stopifnot(sum((fitted(pc) - t(metaboliteData))^2, na.rm=TRUE) < 200)}
##' @keywords multivariate
##' @export
nipalsPca <- function(Matrix, nPcs=2, varLimit=1, maxSteps=5000, 
                      threshold=1e-6, ...) {

  nipRes <- .Call("pcaMethods_Nipals", Matrix, params=list(nPcs=nPcs,
                                      varLimit=varLimit,
                                      threshold=threshold,
                                      maxSteps=maxSteps),
                  PACKAGE="pcaMethods")

  r <- new("pcaRes")
  r@scores <- nipRes$scores
  r@loadings <- nipRes$loadings
  r@R2cum <- nipRes$R2cum
  r@varLimit <- varLimit
  r@method <- "nipals"
  return(r)
}

##' Sort the features of NLPCA object
##' @param nlnet The nlnet
##' @param trainIn Training data in
##' @param trainOut  Training data after it passed through the net
##' @return ...
##' @author Henning Redestig
sortFeatures <- function(nlnet, trainIn, trainOut) {
  weightsAll <- nlnet@weights$current()
  weights <- weightsAll
  if(nlnet@inverse) {
    numElements <- nlnet@net[1] * dim(trainOut)[2]
    trainIn <- array(unlist(weightsAll), dim=c(nlnet@net[1], dim(trainOut)[2]))
    weights <- weightsAll[(numElements + 1):length(weightsAll),,drop=FALSE]
  }
  
  netDim <- dim(nlnet@net)
  trainDim <- dim(trainIn)
  bneckNum <- nlnet@net[nlnet@componentLayer]
  weightMats <- vector2matrices(weights, nlnet@net)

  bneckNum <- nlnet@net[nlnet@componentLayer]

  ## ******************************
  nOut <- array(0, dim=c(sum(nlnet@net), trainDim[2], 2))
  for(subnet in 1:2)
    nOut[1:trainDim[1],,subnet] <- trainIn
  
  ## forward propagation
  for(n in 0:(bneckNum - 2)) {
    E <- c(0,0)
    for(choice in 1:2) {
      sBias <- rep(1, trainDim[2])
      for(i in 1:(netDim[2] - 1)) {
        if(i == 1)
          nBegin <- 1
        else
          nBegin <- sum(nlnet@net[1:(i - 1)]) + 1
        sIn <- rbind(sBias, nOut[nBegin:sum(nlnet@net[1:i]),, choice])
        sOut <- eval(parse(text=paste(nlnet@fkt[i], "(weightMats[[i]] %*% sIn)")))
        
        if(i == nlnet@componentLayer - 1) {
          idx <- rep(0, bneckNum)
          idx[1:(n + choice)] <- 1
          
          if(choice == 2)
            idx[n+choice-1] <- 0
          sOut[idx == 0,] <- 0
        }
        
        nOut[(sum(nlnet@net[1:i]) + 1):sum(nlnet@net[1:(i+1)]),,choice] <- sOut
      }
      output <-
        nOut[(sum(nlnet@net[1:(dim(nlnet@net)[2]-1)])+1):dim(nOut)[1], ,choice]
      
      Epattern <- (output - trainOut)^2
      Epattern[is.na(Epattern)] <- 0
      if(!is.null(nlnet@dataDist))
        Epattern <- Epattern * nlnet@dataDist
      E <- mean(Epattern)
      E[choice] <- E
    }

    if(E[1]>E[2]) {                     #change features
      changeIdx <- 1:bneckNum
      changeIdx[(n+1):(n+2)] <- c(n+2, n+1)
      weightMats[[nlnet@componentLayer - 1]] <-
        weightMats[[nlnet@componentLayer - 1]][changeIdx,]
      weightMats[[nlnet@componentLayer]] <-
        weightMats[[nlnet@componentLayer]][,c(1,changeIdx+1)]
      switching <- c(n+1, n+2)
      nlnet@fCount <- as.integer(nlnet@fCount + 1)
    }
  }
  weights <- cbind(unlist(weightMats))

  if(nlnet@inverse)
    nlnet@weights$set(rbind(matrix(trainIn, nrow=numElements, ncol=1), weights))
}

##' This is a PCA implementation robust to outliers in a data set. It
##' can also handle missing values, it is however NOT intended to be
##' used for missing value estimation.  As it is based on robustSVD we
##' will get an accurate estimation for the loadings also for
##' incomplete data or for data with outliers.  The returned scores
##' are, however, affected by the outliers as they are calculated
##' inputData X loadings. This also implies that you should look at
##' the returned R2/R2cum values with caution.  If the data show
##' missing values, scores are caluclated by just setting all NA -
##' values to zero. This is not expected to produce accurate results.
##' Please have also a look at the manual page for \code{robustSvd}.
##' Thus this method should mainly be seen as an attempt to integrate
##' \code{robustSvd()} into the framework of this package.  Use one of
##' the other methods coming with this package (like PPCA or BPCA) if
##' you want to do missing value estimation.  It is not recommended to
##' use this function directely but rather to use the pca() wrapper
##' function.
##' 
##' The method is very similar to the standard \code{prcomp()}
##' function.  The main difference is that \code{robustSvd()} is used
##' instead of the conventional \code{svd()} method.
##' @title PCA implementation based on robustSvd
##' @param Matrix \code{matrix} -- Data containing the variables in
##' columns and observations in rows. The data may contain missing
##' values, denoted as \code{NA}.
##' @param nPcs \code{numeric} -- Number of components to
##' estimate. The preciseness of the missing value estimation depends
##' on the number of components, which should resemble the internal
##' structure of the data.
##' @param verbose \code{boolean} Print some output to the command
##' line if TRUE
##' @param ...  Reserved for future use. Currently no further
##' parameters are used
##' @return Standard PCA result object used by all PCA-based methods
##' of this package. Contains scores, loadings, data mean and
##' more. See \code{\link{pcaRes}} for details.  are used.
##' @seealso \code{\link{robustSvd}, \link{svd}, \link{prcomp},
##' \link{pcaRes}}.
##' @examples
##' ## Load a complete sample metabolite data set and mean center the data
##' data(metaboliteDataComplete)
##' mdc <- scale(metaboliteDataComplete, center=TRUE, scale=FALSE)
##' ## Now create 5\% of outliers.
##' cond   <- runif(length(mdc)) < 0.05;
##' mdcOut <- mdc
##' mdcOut[cond] <- 10
##' ## Now we do a conventional PCA and robustPca on the original and the data
##' ## with outliers.
##' ## We use center=FALSE here because the large artificial outliers would
##' ## affect the means and not allow to objectively compare the results.
##' resSvd    <- pca(mdc, method="svd", nPcs=10, center=FALSE)
##' resSvdOut <- pca(mdcOut, method="svd", nPcs=10, center=FALSE)
##' resRobPca <- pca(mdcOut, method="robustPca", nPcs=10, center=FALSE)
##' ## Now we plot the results for the original data against those with outliers
##' ## We can see that robustPca is hardly effected by the outliers.
##' plot(loadings(resSvd)[,1], loadings(resSvdOut)[,1])
##' plot(loadings(resSvd)[,1], loadings(resRobPca)[,1])
##' @keywords multivariate
##' @export
##' @author Wolfram Stacklies
robustPca <- function(Matrix, nPcs=2, verbose=interactive(), ... ) {

  nas <- is.na(Matrix)

  if (sum(nas) != 0)
    warning("Data is incomplete, it is not recommended to use robustPca for missing value estimation")
  svdSol <- robustSvd(Matrix)
  
  ## Sort the eigenvalues and eigenvectors
  loadings <- svdSol$v[, 1:nPcs, drop=FALSE]
  sDev     <- svdSol$d[1:nPcs] / sqrt(max(1, nrow(Matrix) - 1))

  ## We estimate the scores by just setting all NA values to 0 This is
  ## a bad approximation, I know... Use ppca / bpca or other missing
  ## value estimation methods included in this package
  compMat <- Matrix
  compMat[is.na(compMat)] <- 0
  scores   <- compMat %*% loadings

  ## Calculate R2cum (on the complete observations only)
  R2cum <- rep(NA, nPcs)
  TSS <- sum(Matrix^2, na.rm=TRUE)
  for (i in 1:nPcs) {
    difference <- Matrix -
      (scores[,1:i, drop=FALSE] %*% t(loadings[,1:i, drop=FALSE]))
    R2cum[i] <- 1 - (sum(difference^2) / TSS)
  }

  result <- new("pcaRes")
  result@loadings <- loadings
  result@scores <- scores
  result@R2cum <- R2cum
  result@method <- "robustPca"
  return(result)
}

##' A robust approximation to the singular value decomposition of a
##' rectangular matrix is computed using an alternating L1 norm
##' (instead of the more usual least squares L2 norm). As the SVD is
##' a least-squares procedure, it is highly susceptible to outliers
##' and in the extreme case, an individual cell (if sufficiently
##' outlying) can draw even the leading principal component toward
##' itself.
##'
##' See Hawkins et al (2001) for details on the robust SVD algorithm.
##' Briefly, the idea is to sequentially estimate the left and right
##' eigenvectors using an L1 (absolute value) norm minimization.
##'
##' Note that the robust SVD is able to accomodate missing values in
##' the matrix \code{x}, unlike the usual \code{svd} function.
##' 
##' Also note that the eigenvectors returned by the robust SVD
##' algorithm are NOT (in general) orthogonal and the eigenvalues need
##' not be descending in order.
##' @title Alternating L1 Singular Value Decomposition
##' @param x A matrix whose SVD decomposition is to be
##' computed. Missing values are allowed.
##' @return The robust SVD of the matrix is x=u d v'. \item{d}{A
##' vector containing the singular values of \code{x}.} \item{u}{A
##' matrix whose columns are the left singular vectors of \code{x}.}
##' \item{v}{A matrix whose columns are the right singular vectors of
##' \code{x}.}
##' @note Two differences from the usual SVD may be noted. One relates
##' to orthogonality. In the conventional SVD, all the eigenvectors
##' are orthogonal even if not explicitly imposed.  Those returned by
##' the AL1 algorithm (used here) are (in general) not orthogonal.
##' Another difference is that, in the L2 analysis of the conventional
##' SVD, the successive eigen triples (eigenvalue, left eigenvector,
##' right eigenvector) are found in descending order of
##' eigenvalue. This is not necessarily the case with the AL1
##' algorithm.  Hawkins et al (2001) note that a larger eigen value
##' may follow a smaller one.
##' @references Hawkins, Douglas M, Li Liu, and S Stanley Young (2001)
##' Robust Singular Value Decomposition, National Institute of
##' Statistical Sciences, Technical Report Number
##' 122. \url{http://www.niss.org/technicalreports/tr122.pdf}
##' @author Kevin Wright, modifications by Wolfram Stacklies
##' @seealso \code{\link{svd}}, \code{\link[ade4:nipals]{nipals}} for
##' an alternating L2 norm method that also accommodates missing data.
##' @examples
##' ## Load a complete sample metabolite data set and mean center the data
##' data(metaboliteDataComplete)
##' mdc <- prep(metaboliteDataComplete, center=TRUE, scale="none")
##' ## Now create 5% of outliers.
##' cond   <- runif(length(mdc)) < 0.05;
##' mdcOut <- mdc
##' mdcOut[cond] <- 10
##' ## Now we do a conventional SVD and a robustSvd on both, the original and the 
##' ## data with outliers.
##' resSvd <- svd(mdc)
##' resSvdOut <- svd(mdcOut)
##' resRobSvd <- robustSvd(mdc)
##' resRobSvdOut <- robustSvd(mdcOut)
##' ## Now we plot the results for the original data against those with outliers
##' ## We can see that robustSvd is hardly affected by the outliers.
##' plot(resSvd$v[,1], resSvdOut$v[,1])
##' plot(resRobSvd$v[,1], resRobSvdOut$v[,1])
##' @keywords algebra
##' @export
robustSvd <- function(x) {
  ## We need the weightedMedian function provided by the aroma.light
  ## package. However we do not want to make the whole package dependant
  ## on aroma.light
  if (!requireNamespace("matrixStats", quietly=TRUE))
    stop("package matrixStats required but not available")

  L1RegCoef <- function(x, a){
    keep <- (abs(a) > .Machine$double.eps) & (!is.na(x))
    if(!any(keep)) {
      warning("No non-missing data for l1 regression, unstable results")
      return(0.)
    }
    a <- a[keep]
    return(matrixStats::weightedMedian(x[keep] / a, abs(a),
                                       na.rm=TRUE, interpolate=FALSE))
  }

  L1Eigen <- function(x, a, b){
    x <- as.vector(x) # Convert from matrix to vector
    ab <- as.vector(outer(a, b))
    keep <- (abs(ab) > .Machine$double.eps) & (!is.na(x))
    ab <- ab[keep]
    return(matrixStats::weightedMedian(x[keep] / ab, abs(ab),
                                       na.rm=TRUE, interpolate=FALSE))
  }

  ## Initialize outputs
  svdu <- matrix(NA, nrow=nrow(x), ncol=ncol(x))
  svdv <- matrix(NA, nrow=ncol(x), ncol=ncol(x))
  svdd <- rep(NA, ncol(x))

  for(k in 1:ncol(x)) {
    ak <- apply(abs(x), 1, median, na.rm=TRUE)
    converged <- FALSE
    while(!converged) {
      akprev <- ak
      c <- apply(x, 2, L1RegCoef, ak)
      bk <- c / sqrt(sum(c^2))
      d <- apply(x, 1, L1RegCoef, bk)
      ak <- d / sqrt(sum(d^2))
      if(sum((ak - akprev)^2) < 1e-10) {
        converged <- TRUE
      }
    }
    eigenk <- L1Eigen(x,ak,bk)
    ## Deflate the x matrix
    x <- x - eigenk * ak %*% t(bk)
    ## Store eigen triple for output
    svdu[,k] <- ak
    svdv[,k] <- bk 
    svdd[k] <- eigenk
  }

  ## Create the result object
  ret <- list()

  ret$d <- svdd
  ret$u <- svdu
  ret$v <- svdv
  return(ret)
}

##' Later
##' @param nlnet The nlnet
##' @param trainIn training data 
##' @param trainOut  fitted data
##' @return error
##' @author Henning Redestig, Matthias Scholz
errorHierarchic <- function(nlnet, trainIn, trainOut) {

  weights <- nlnet@weights$current()
  
  if(nlnet@inverse) {
    numElements <- nlnet@net[1] * dim(trainOut)[2]
    trainIn <- array(weights[1:numElements], dim=c(nlnet@net[1], dim(trainOut)[2]))
    wTrainIn <- weights[1:numElements, drop=FALSE]
    weights <- weights[(numElements + 1):length(weights),,drop=FALSE]
  }

  netDim <- dim(nlnet@net)
  trainDim <- dim(trainOut)
  weightMats <- vector2matrices(weights, nlnet@net)

  hierarchicIdx <- nlnet@hierarchic$idx[,nlnet@hierarchic$var != 0, drop=FALSE]
  hierarchicVar <- nlnet@hierarchic$var[,colSums(nlnet@hierarchic$var) != 0, drop=FALSE]

  subnetNum <- length(hierarchicVar)

  out <- array(0, dim=c(trainDim[1], trainDim[2], subnetNum))

  sBias <- array(1, dim=c(1, trainDim[2]))

  sExtract <- eval(parse(text=paste(nlnet@fct[1], "(trainIn)")))
  
  Eitemize <- NULL
  if(nlnet@hierarchic$layer > 1) {      #this should not be executed at all if sequence is 1:0
    for(layer in 1:(nlnet@hierarchic$layer - 1)) {
      sExtract <- rbind(sBias, sExtract)
      sExtract <- eval(parse(text=paste(nlnet@fct[layer + 1], "(weightMats[[layer]] %*% sExtract)")))
    }
  }

  for(subnet in 1:subnetNum) {
    sRecon <- sExtract
    sRecon[hierarchicIdx[,subnet]==0,] <- 0

    for(layer in nlnet@hierarchic$layer:(netDim[2] - 1)) {
      sRecon <- rbind(sBias, sRecon)
      sRecon <- eval(parse(text=paste(nlnet@fct[layer+1], "(weightMats[[layer]] %*% sRecon)")))
    }
    out[,,subnet] <- sRecon

    ## error function
    eTmp <- (sRecon - trainOut)^2
    eTmp[is.na(eTmp)] <- 0
    
    Eitemize[subnet] <- sum(eTmp) * 0.5

    if(!is.null(nlnet@dataDist)) 
      Eitemize[subnet] <- 0.5 * sum(nlnet@dataDist * eTmp)
    else
      Eitemize[subnet] <- 0.5 * sum(eTmp)
  }

  error <- tcrossprod(hierarchicVar, rbind(Eitemize))

  if(!is.null(nlnet@weightDecay))
    error <- error + nlnet@weightDecay * 0.5 * sum(weights^2)

  ## smooth (0.01) weight decay also for input values
  if(nlnet@inverse)
    error <- error + 0.01 * nlnet@weightDecay * 0.5 * sum(wTrainIn^2)
  
  return(list(error=error, out=out))
}
  

##' Tranform the vectors of weights to matrix structure
##' @param object  an nlpcaNet
##' @return weights in matrix structure
##' @author Henning Redestig
##' @aliases vector2matrices,nlpcaNet-method
setMethod("vector2matrices", "nlpcaNet", function(object) {
  netDim <- dim(object@net)
  posBegin <- 1
  posEnd <- 0
  result <- list()
  for(i in 1:(netDim[2] - 1)) {
    wSize <- c(object@net[i + 1], object@net[i] + 1)
    posEnd <- posEnd + prod(wSize)
    result[[i]] <-
      matrix(object@weights$current()[posBegin:posEnd], wSize[1], wSize[2])
    posBegin <- posEnd + 1
  }
  
  if(posEnd < length(object@weights$current()))
    stop("weight vector has too many elements\n")
  result
})

##' Tranform the vectors of weights to matrix structure
##' @param object an nlpcaNet
##' @param net the neural network
##' @return weights in matrix structure
##' @author Henning Redestig
##' @aliases vector2matrices,matrix-method
setMethod("vector2matrices", "matrix", function(object, net) {
  netDim <- dim(net)
  posBegin <- 1
  posEnd <- 0
  result <- list()
  for(i in 1:(netDim[2] - 1)) {
    wSize <- c(net[i + 1], net[i] + 1)
    posEnd <- posEnd + prod(wSize)
    result[[i]] <- matrix(object[posBegin:posEnd], wSize[1], wSize[2])
    posBegin <- posEnd + 1
  }
  if(posEnd < length(object))
    stop("weight vector has too many elements\n")
  result
})

##' Internal cross-validation can be used for estimating the level of
##' structure in a data set and to optimise the choice of number of
##' principal components.
##'
##' This method calculates \eqn{Q^2} for a PCA model. This is the
##' cross-validated version of \eqn{R^2} and can be interpreted as the
##' ratio of variance that can be predicted independently by the PCA
##' model. Poor (low) \eqn{Q^2} indicates that the PCA model only
##' describes noise and that the model is unrelated to the true data
##' structure. The definition of \eqn{Q^2} is: \deqn{Q^2=1 -
##' \frac{\sum_{i}^{k}\sum_{j}^{n}(x -
##' \hat{x})^2}{\sum_{i}^{k}\sum_{j}^{n}x^2}}{Q^2=1 - sum_i^k sum_j^n
##' (x - \hat{x})^2 / \sum_i^k \sum_j^n(x^2)} for the matrix \eqn{x}
##' which has \eqn{n} rows and \eqn{k} columns. For a given number of
##' PC's x is estimated as \eqn{\hat{x}=TP'} (T are scores and P are
##' loadings). Although this defines the leave-one-out
##' cross-validation this is not what is performed if fold is less
##' than the number of rows and/or columns.  In 'impute' type CV,
##' diagonal rows of elements in the matrix are deleted and the
##' re-estimated.  In 'krzanowski' type CV, rows are sequentially left
##' out to build fold PCA models which give the loadings. Then,
##' columns are sequentially left out to build fold models for
##' scores. By combining scores and loadings from different models, we
##' can estimate completely left out values.  The two types may seem
##' similar but can give very different results, krzanowski typically
##' yields more stable and reliable result for estimating data
##' structure whereas impute is better for evaluating missing value
##' imputation performance. Note that since Krzanowski CV operates on
##' a reduced matrix, it is not possible estimate Q2 for all
##' components and the result vector may therefore be shorter than
##' \code{nPcs(object)}.
##' @title Cross-validation for PCA
##' @param object A \code{pcaRes} object (result from previous PCA
##'   analysis.)
##' @param originalData The matrix (or ExpressionSet) that used to
##'   obtain the pcaRes object.
##' @param fold The number of groups to divide the data in.
##' @param nruncv The number of times to repeat the whole
##'   cross-validation. The deletion of diagnols in 'impute' is
##'   deterministic so result will alsways be the same but in
##'   krzanowski where cv-split is obtained by sampling it can be
##'   informative to examine the spread of the Q2 values over several
##'   CV runs.
##' @param type krzanowski or imputation type cross-validation
##' @param verbose \code{boolean} If TRUE Q2 outputs a primitive
##'   progress bar.
##' @param variables indices of the variables to use during
##'   cross-validation calculation. Other variables are kept as they
##'   are and do not contribute to the total sum-of-squares.
##' @param ... Further arguments passed to the \code{\link{pca}}
##'   function called within Q2.
##' @return A matrix or vector with \eqn{Q^2} estimates.
##' @export
##' @references Krzanowski, WJ. Cross-validation in principal
##'   component analysis. Biometrics. 1987(43):3,575-584
##' @examples
##' data(iris)
##' x <- iris[,1:4]
##' pcIr <- pca(x, nPcs=3)
##' q2 <- Q2(pcIr, x)
##' barplot(q2, main="Krzanowski CV", xlab="Number of PCs", ylab=expression(Q^2))
##' ## q2 for a single variable
##' Q2(pcIr, x, variables=2)
##' pcIr <- pca(x, nPcs=3, method="nipals")
##' q2 <- Q2(pcIr, x, type="impute")
##' barplot(q2, main="Imputation CV", xlab="Number of PCs", ylab=expression(Q^2))
##' @author Henning Redestig, Ondrej Mikula
##' @keywords multivariate
Q2 <- function (object, originalData=completeObs(object), fold=5, 
                nruncv=1, type=c("krzanowski", "impute"), verbose=interactive(),
                variables=1:nVar(object), ...) {
  type <- match.arg(type)
  if (inherits(originalData, "ExpressionSet")) {
    set <- originalData
    originalData <- t(exprs(originalData))
  }
  if (is.null(originalData)) 
    stop("missing data when estimating Q2")
  originalData <- as.matrix(originalData)
  originalData <- prep(originalData, scale=scl(object), center=center(object))
  nR <- nObs(object)
  nC <- nVar(object)
  if (nR != nrow(originalData) | nC != ncol(originalData)) 
    stop("data and model dimensions do not match")
  if (fold > max(nR, nC)) 
    stop("fold must be equal or less to max dimension of original data")
  if (method(object) %in% c("svd") & type != "krzanowski") 
    stop("Chosen PCA method must use krzanowski type cv")
  if (method(object) %in% c("llsImpute") & type != "impute") 
    stop("Chosen PCA method must use impute type cv")
  
  if (is.logical(variables))
    variables <- which(variables)

  ssx <- sum(originalData[, variables]^2, na.rm=TRUE)
  if(type == "impute")
    nP <- nPcs(object)
  if(type == "krzanowski") {
    rseg <- split(sample(1:nR), rep(1:fold, ceiling(nR/fold))[1:nR])
    cseg <- split(sample(1:nC), rep(1:fold, ceiling(nC/fold))[1:nC])
    foldC <- length(cseg)
    foldR <- length(rseg)
    nP <- min(nR - max(sapply(rseg, length)),
              nC - max(sapply(cseg, length)), nPcs(object))
  }
  q2 <- matrix(NA, nP, ncol=nruncv)
  for (nr in 1:nruncv) {
    press <- rep(0, nP)
    if (type == "impute") {
      seg <- list()
      nDiag <- max(nR, nC)
      diagPerFold <- floor(nDiag / fold)
      suppressWarnings(diags <- matrix(1:nDiag, nrow=diagPerFold, ncol=fold, byrow=TRUE))
      if (diagPerFold == 0 || diagPerFold > (nDiag/2)) 
        stop("Matrix could not be safely divided into ", 
             fold, " segments. Choose a different fold or provide the desired segments")
      if (nDiag%%fold > 0) 
        warning("Validation incomplete: ", (nDiag %% fold) * min(dim(originalData)),
                " values were left out of from cross validation, Q2 estimate will be biased.")
      for (i in 1:ncol(diags))
        seg[[i]] <- which(is.na(deletediagonals(originalData, diags[, i])))
      if (verbose) {
        pb <- txtProgressBar(0, length(seg), style=3, width=20)
      }
      j <- 0
      for (i in seg) {
        j <- j + 1
        if (verbose) 
          setTxtProgressBar(pb, j)
        test <- originalData
        test[i] <- NA
        test <- tempFixNas(test)
        if (method(object) != "llsImpute") {
          pc <- pca(test, nPcs=nP, method=method(object), 
					verbose=FALSE, center=centered(object), scale=object@scaled, ...)
        }
        for (np in 1:nP) {
          if (method(object) == "llsImpute") {
            fittedData <- completeObs(llsImpute(test, k=np, allVariables=TRUE, center=FALSE))
          }
          else {
            if (method(object) == "nlpca") 
              fittedData <- fitted(pc, data=test, nPcs=np)
            else fittedData <- fitted(pc, data=NULL, nPcs=np)
          }
          ii <- i[ceiling(i / nR) %in% variables]
          press[np] <- press[np] + sum((originalData[ii] - fittedData[ii])^2, na.rm=TRUE)
        }
      }
    }

    if (type == "krzanowski") {
      rseg <- split(sample(1:nR), rep(1:fold, ceiling(nR/fold))[1:nR])
      cseg <- split(sample(1:nC), rep(1:fold, ceiling(nC/fold))[1:nC])
      tcv <- array(0, dim=c(foldC, nR, nP))
      pcv <- array(0, dim=c(foldR, nC, nP))
      for (f in 1:foldC) {
        test <- tempFixNas(originalData[, -cseg[[f]]])
        tcv[f, , ] <- scores(pca(test, nPcs=nP, method=method(object), 
                                 verbose=FALSE, center=centered(object), scale=object@scaled, ...))
        for (p in 1:nP) {
          if (cor(tcv[f, , p], scores(object)[, p]) < 0) 
            tcv[f, , p] <- tcv[f, , p] * -1
        }
      }
      for (f in 1:foldR) {
        test <- tempFixNas(originalData[-rseg[[f]], ])
        pcv[f, , ] <- loadings(pca(test, nPcs=nP, method=method(object), 
                                   verbose=FALSE, center=centered(object), scale=object@scaled, ...))
        for (p in 1:nP) {
          if (cor(pcv[f, , p], loadings(object)[, p]) < 0) 
            pcv[f, , p] <- pcv[f, , p] * -1
        }
      }
      press <- rep(0, nP)
      for (p in 1:nP)
        for (fr in 1:foldR)
          for (fc in 1:foldC) 
            press[p] <- press[p] + sum((
              originalData[rseg[[fr]], cseg[[fc]]] - 
                (tcv[fc, , ][, 1:p, drop=FALSE] %*% t(pcv[fr, , ][,1:p, drop=FALSE]))
              [rseg[[fr]], intersect(cseg[[fc]], variables)])^2, na.rm=TRUE)
    }
    q2[, nr] <- 1 - press/ssx
  }

  if (verbose) 
    message("\n")
  rownames(q2) <- paste("PC", 1:nrow(q2))
  drop(q2)
}

##' Simply replace completely missing rows or cols with zeroes.
##' @title Temporary fix for missing values
##' @param mat a matrix
##' @return The original matrix with completely missing rows/cols
##' filled with zeroes.
##' @author Henning Redestig 
tempFixNas <- function(mat) {
  badRows <- apply(mat, 1, function(x) all(is.na(x)))
  badCols <- apply(mat, 2, function(x) all(is.na(x)))
  mat[ badRows,] <- 0
  mat[,badCols ] <- 0
  mat
}

##' Replace a diagonal of elements of a matrix with NA
##'
##' Used for creating artifical missing values in matrices without
##' causing any full row or column to be completely missing
##' @title Delete diagonals
##' @param x The matrix  
##' @param diagonals The diagonal to be replaced, i.e. the first,
##' second and so on when looking at the fat version of the matrix
##' (transposed or not) counting from the bottom.
##' Can be a vector to delete more than one diagonal.
##' @return The original matrix with some values missing
##' @author Henning Redestig 
deletediagonals <- function(x, diagonals=1) {
  wastransposed <- FALSE
  if (dim(x)[1] > dim(x)[2]) {          # matrix must be lying down
    x <- t(x)
    wastransposed <- TRUE
  }
  nr <- nrow(x)
  nc <- ncol(x)
  if (!all(diagonals <= nc)) {
    stop(paste("Order of diagonal number", max(diagonals),  "is out of bound"))
  }
  indexmatrix <- matrix(1 : (nr * nc), ncol=nc, nrow=nr)
  finalmatrix <- matrix(ncol=(nr - 1 + nc), nrow=nr)
  finalmatrix[,1 : (nr - 1)] <- indexmatrix[,rev((nc : 1)[1 : (nr - 1)])]
  finalmatrix[,nr : (nr - 1 + nc)] <- indexmatrix
  dia <- 1 + 0:(nr - 1) * (nr + 1)
  finalIndices <- NULL
  for (i in 1:length(diagonals)) {
    indicestodelete <- finalmatrix[dia + (diagonals[i] - 1) * nr]
    x[indicestodelete] <- NA
    finalIndices <- c(finalIndices, indicestodelete)
  }
  if (wastransposed) x <- t(x)
  return(x)
}

##' Get cross-validation segments that have (as far as possible) the
##' same ratio of all classes (if classes are present)
##' @title Get CV segments
##' @param x a factor, character or numeric vector that describes
##' class membership of a set of items, or, a numeric vector
##' indicating unique indices of items, or, a numeric of length 1 that
##' describes the number of items to segment (without any classes)
##' @param fold the desired number of segments
##' @param seed randomization seed for reproducibility
##' @return a list where each element is a set of indices that defines
##' the CV segment.
##' @examples
##' seg <- cvseg(iris$Species, 10)
##' sapply(seg, function(s) table(iris$Species[s]))
##' cvseg(20, 10)
##' @seealso the \code{cvsegments} function in the \code{pls} package
##' @export
##' @author Henning Redestig
cvseg <- function(x, fold=7, seed=NULL) {
  if(any(table(x) > 1)) {
    if(any(table(x) < fold)) {
      fold <- min(table(x))
    }
    if(fold < 2)
      stop("too few observations in the smallest class")
    res <-
      sapply(unique(x), function(z) {
        if(!is.null(seed))
          set.seed(seed)
        tmp <- sample(which(x == z))
        seg <- matrix(c(tmp,
                        rep(NA, ifelse(length(tmp) %% fold ==0, 0,
                                       fold - (length(tmp) %% fold)))),
                      nrow=fold)
      },simplify=FALSE)
    res <- do.call("cbind", res)
  } else {
    if(length(x) == 1) x <- 1:x
    res <- matrix(sample(c(x,
                           rep(NA, ifelse(length(x) %% fold ==0, 0,
                                          fold - (length(x) %% fold))))),
                  nrow=fold)
  }
  res <- res[!apply(is.na(res), 1, all),,drop=FALSE]
  res
  lapply(as.data.frame(t(res)), function(x) c(na.omit(x)))
}

##' Scaling and centering a matrix.
##'
##' Does basically the same as \code{\link{scale}} but adds some
##' alternative scaling options and functionality for treating
##' pre-processing as part of a model.
##' @title Pre-process a matrix for PCA
##' @param object Numerical matrix (or an object coercible to such)
##' with samples in rows and variables as columns. Also takes
##' \code{ExpressionSet} in which case the transposed expression
##' matrix is used. 
##' @param scale One of "UV" (unit variance \eqn{a=a/\sigma_{a}})
##' "vector" (vector normalisation \eqn{b=b/||b||}), "pareto" (sqrt
##' UV) or "none" to indicate which scaling should be used to scale
##' the matrix with \eqn{a} variables and \eqn{b} samples. Can also be
##' a vector of scales which should be used to scale the
##' matrix. \code{NULL} value is interpreted as \code{"none"}.
##' @param center Either a logical which indicates if the matrix
##' should be mean centred or not, or a vector with averages which
##' should be suntracted from the matrix. \code{NULL} value is
##' interpreted as \code{FALSE}
##' @param eps Minimum variance, variable with lower variance are not
##' scaled and warning is issued instead.
##' @param simple Logical indicating if only the data should be
##' returned or a list with the pre-processing statistics as well.
##' @param reverse Logical indicating  if matrix should be
##' 'post-processed' instead by multiplying each column with its scale
##' and adding the center. In this case, center and scale should be
##' vectors with the statistics (no warning is issued if not, instead
##' output becomes the same as input).
##' @param ... Only used for passing through arguments.
##' @return A pre-processed matrix or a list with
##' \item{center}{a vector with the estimated centers}
##' \item{scale}{a vector with the estimated scales}
##' \item{data}{the pre (or post) processed data}
##' @examples
##' object <- matrix(rnorm(50), nrow=10)
##' res <- prep(object, scale="uv", center=TRUE, simple=FALSE)
##' obj <- prep(object, scale=res$scale, center=res$center)
##' ## same as original
##' sum((object - prep(obj, scale=res$scale, center=res$center, rev=TRUE))^2)
##' @export 
##' @author Henning Redestig
prep <- function(object, scale=c("none", "pareto", "vector", "uv"),
                 center=TRUE, eps=1e-12, simple=TRUE, reverse=FALSE, ...) {
  if(inherits(object, "ExpressionSet"))
    obj <- t(exprs(object))
  else
    obj <- as.matrix(object)

  if(is.null(center))
    center <- FALSE
  if(is.null(scale))
    scale <- "none"

  if(is.logical(center[1])) {
    if(center[1]) 
      center <- colMeans(obj, na.rm=TRUE)
    else
      center <- rep(0, ncol(obj))
  }
  if(length(center) != ncol(obj))
    stop("center do not match matrix dimensions")
  if(!reverse)
    obj <- sweep(obj, 2, center, "-")
  
  if(is.character(scale[1])) {
    scale <- match.arg(scale)
    if(scale == "uv")
      scale <- apply(obj, 2, sd, na.rm=TRUE)
    else if(scale == "none")
      scale <- rep(1, ncol(obj))
    else if(scale == "pareto")
      scale <- sqrt(apply(obj, 2, sd, na.rm=TRUE))
    else if(scale == "vector")
      scale <- apply(obj, 2, function(x) sqrt(sum(x^2, na.rm=TRUE)))
  }
  if(length(scale) != ncol(obj))
    stop("scale vector do not match matrix dimensions")
  
  if (any(scale < eps)) 
    warning(paste("Variance is below eps for", sum(scale < eps),
                  "variables. Not scaling them."))

  scale[scale < eps] <- 1
  if(!reverse)
    obj <- sweep(obj, 2, scale, "/")

  if(reverse) {
    obj <- sweep(obj, 2, scale, "*")
    obj <- sweep(obj, 2, center, "+")
  }
	
  if(inherits(object, "ExpressionSet"))
    exprs(object) <- t(obj)
  else
    object <- obj

  if (simple)
    object
  else
    list(data=object, center=center, scale=scale)
}

##' Creates a large matrix B consisting of an M-by-N tiling of copies
##' of A
##' @title Replicate and tile an array.
##' @param mat numeric matrix
##' @param M number of copies in vertical direction
##' @param N number of copies in horizontal direction
##' @return Matrix consiting of M-by-N tiling copies of input matrix
##' @author Wolfram Stacklies
repmat <- function(mat, M, N) {

  ## Check if all input parameters are correct
  if( !all(M > 0, N > 0) ) {
    stop("M and N must be > 0")
  }    
  
  ## Convert array to matrix
  ma <- mat
  if(!is.matrix(mat)) {
    ma <- matrix(mat, nrow=1)
  }

  rows <- nrow(ma)
  cols <- ncol(ma)
  replicate <- matrix(0, rows * M, cols * N)

  for (i in 1:M) {
    for(j in 1:N) {
      start_row <- (i - 1) * rows + 1
      end_row <- i * rows
      start_col <- (j - 1) * cols + 1
      end_col <- j * cols
      replicate[start_row:end_row, start_col:end_col] <- ma
    }
  }

  return(replicate)
}

##' Neural network based non-linear PCA
##'
##' Artificial Neural Network (MLP) for performing non-linear
##' PCA. Non-linear PCA is conceptually similar to classical PCA but
##' theoretically quite different. Instead of simply decomposing our
##' matrix (X) to scores (T) loadings (P) and an error (E) we train a
##' neural network (our loadings) to find a curve through the
##' multidimensional space of X that describes a much variance as
##' possible. Classical ways of interpreting PCA results are thus not
##' applicable to NLPCA since the loadings are hidden in the network.
##' However, the scores of components that lead to low
##' cross-validation errors can still be interpreted via the score
##' plot.  Unfortunately this method depend on slow iterations which
##' currently are implemented in R only making this method extremely
##' slow. Furthermore, the algorithm does not by itself decide when it
##' has converged but simply does 'maxSteps' iterations.
##' @title Non-linear PCA
##' @param Matrix \code{matrix} --- Preprocessed data with the
##' variables in columns and observations in rows. The data may
##' contain missing values, denoted as \code{NA}
##' @param nPcs \code{numeric} -- Number of components to
##' estimate. The preciseness of the missing value estimation depends
##' on thenumber of components, which should resemble the internal
##' structure of the data.
##' @param maxSteps \code{numeric} -- Number of estimation
##' steps. Default is based on a generous rule of thumb.
##' @param unitsPerLayer The network units, example: c(2,4,6) for two
##' input units 2feature units (principal components), one hidden
##' layer fornon-linearity and three output units (original amount
##' ofvariables).
##' @param functionsPerLayer The function to apply at each layer
##' eg. c("linr", "tanh", "linr") 
##' @param weightDecay Value between 0 and 1.
##' @param weights Starting weights for the network. Defaults to
##' uniform random values but can be set specifically to make
##' algorithm deterministic.
##' @param verbose \code{boolean} -- nlpca prints the number of steps
##' and warning messages if set to TRUE. Default is interactive().
##' @param ...  Reserved for future use. Not passed on anywhere.
##' @return Standard PCA result object used by all PCA-basedmethods of
##' this package. Contains scores, loadings, data meanand more. See
##' \code{\link{pcaRes}} for details.
##' @author Based on a matlab script by Matthias Scholz and ported to
##' R by Henning Redestig
##' @references Matthias Scholz, Fatma Kaplan, Charles L Guy, Joachim
##' Kopkaand Joachim Selbig. Non-linear PCA: a missing
##' data approach. \emph{Bioinformatics, 21(20):3887-3895, Oct 2005}
##' @examples
##' ## Data set with three variables where data points constitute a helix
##' data(helix)
##' helixNA <- helix
##' ## not a single complete observation
##' helixNA <- t(apply(helix, 1, function(x) { x[sample(1:3, 1)] <- NA; x}))
##' ## 50 steps is not enough, for good estimation use 1000
##' helixNlPca <- pca(helixNA, nPcs=1, method="nlpca", maxSteps=50)
##' fittedData <- fitted(helixNlPca, helixNA)
##' plot(fittedData[which(is.na(helixNA))], helix[which(is.na(helixNA))])
##' ## compared to solution by Nipals PCA which cannot extract non-linear patterns
##' helixNipPca <- pca(helixNA, nPcs=2)
##' fittedData <- fitted(helixNipPca)
##' plot(fittedData[which(is.na(helixNA))], helix[which(is.na(helixNA))])
##' @export
nlpca <- function(Matrix, nPcs=2, 
                  maxSteps=2 * prod(dim(Matrix)),
                  unitsPerLayer=NULL, functionsPerLayer=NULL,
                  weightDecay=0.001, weights=NULL,
                  verbose=interactive(),...) {

  ## do some basic checks
  object <- Matrix
  trainIn <- NULL
  trainOut <- t(object)
  stds <- apply(trainOut, 2, sd, na.rm=TRUE)
  scalingFactor <- 0.1 / max(stds)
  trainOut <- trainOut * scalingFactor

  ## now setup the initial nlpcaNet object
  numNaN <- sum(is.na(object))
  
  ## always inverse in this version, bottleneck is not fully implemented 
  inverse <- TRUE

  ## DATADIST (nlnet@dataDist) is given by weightOut
  dataDist <- apply(!is.na(trainOut), 2, as.integer) #0 for NA, 1 for everything else
  if(!inverse)
    dataDist <- NULL

  ## setup the network architecture
  if(is.null(unitsPerLayer)) {
    ld <- dim(trainOut)[1]
    lh <- nPcs
    if(nPcs < 10)
      lh <- 2 + 2 * nPcs
    unitsPerLayer <- c(ld, lh, nPcs, lh, ld)
    if(inverse)
      unitsPerLayer <- c(nPcs, lh, ld)
  }
  featureLayer <- ceiling(length(unitsPerLayer) / 2)
  if(inverse)
    featureLayer <- 1

  if(is.null(functionsPerLayer)) {
    functionsPerLayer <- rep("tanh", length(unitsPerLayer))
    functionsPerLayer[1] <- "linr"
    functionsPerLayer[featureLayer] <- "linr"
    functionsPerLayer[length(unitsPerLayer)] <- "linr"
  }
  hierarchic <- list(layer=featureLayer,
                     var=rbind(c(rep(1, nPcs), 0.01)),
                     idx=getHierarchicIdx(unitsPerLayer[featureLayer]))
  
  ## set up the weights
  wNum <- sum(sapply(2:length(unitsPerLayer),
                     function(i) (1 + unitsPerLayer[i - 1]) * unitsPerLayer[i]))
  if(!is.null(weights) && length(weights) != wNum) {
    warning("Weight vector not expected length (", wNum, "), using random weights", sep="")
    weights <- NULL
  }
  if(is.null(weights)) 
    weights <- cbind(0.2 * (runif(wNum, 0, 1) - 0.1))
  
  if(inverse) {
    numPattern <- dim(trainOut)[2]
    tmpTrainIn <- cbind(rnorm(unitsPerLayer[1] * numPattern,0,1) * 0.1)
    weights <- rbind(tmpTrainIn, weights)
  }

  if(nPcs == 1)
      featureSorting <- FALSE
  if(nPcs > 1)
    featureSorting <- TRUE

  nlnet <- new("nlpcaNet")
  nlnet@net <- rbind(unitsPerLayer)
  nlnet@hierarchic <- hierarchic
  nlnet@fct <- functionsPerLayer
  nlnet@fkt <- functionsPerLayer[2:length(functionsPerLayer)]
  nlnet@weightDecay <- weightDecay
  nlnet@featureSorting <- featureSorting
  nlnet@dataDist <- dataDist
  nlnet@inverse <- inverse
  nlnet@fCount <- as.integer(0)
  nlnet@componentLayer <- as.integer(featureLayer)
  nlnet@error <- errorHierarchic
  nlnet@gradient <- derrorHierarchic
  nlnet@maxIter <- as.integer(maxSteps)
  nlnet@weights <- weightsAccount(weights)
  nlnet@scalingFactor <- scalingFactor
  ## ******************************

  if(verbose)
    cat("Training network with", nlnet@maxIter, "iterations...\n!:\tSquare error is NA -- accuracy in line-search might be too small\n<n>:\tComponents were sorted at iteration n\n^:\tToo many iterations while expanding\n")
  newnet <- optiAlgCgd(nlnet, trainIn, trainOut, verbose)
  if(verbose)
    cat("\nDone\n")

  if(inverse) {
    nObs <- unitsPerLayer[1] * dim(trainOut)[2]
    we <- newnet@weights$current()
    scores <- t(matrix(we[1:nObs], nrow=unitsPerLayer[1], dim(trainOut)[2]))
    newnet@weights$set(we[(nObs + 1):length(we),,drop=FALSE])
  }
  ## for further applications newnet must not be inverse anymore
  newnet@inverse <- FALSE               

  res <- new("pcaRes")
  res@scores <- scores
  res@loadings <- matrix()
  res@network <- newnet
  res@method <- "nlpca"

  R2cum <- rep(NA, nPcs)
  TSS <- sum(Matrix^2, na.rm=TRUE)
  for(i in 1:nPcs) 
    R2cum[i] <-
      1 - sum((Matrix - fitted(res, Matrix, nPcs=i))^2, na.rm=TRUE) / TSS

  res@R2cum <- R2cum
  res
}

##' Index in hiearchy
##' @param hierarchicNum  A number
##' @return ...
##' @author Henning Redestig, Matthias Scholz
getHierarchicIdx <- function(hierarchicNum) {
  res <- matrix(1, ncol=hierarchicNum, nrow=hierarchicNum)
  res[lower.tri(res)] <- 0
  cbind(res, c(0, rep(1, hierarchicNum - 1)))
}

##' @include errorHierarchic.R
##' @include derrorHierarchic.R
NULL

##' This is a class representation of a non-linear PCA neural
##' network. The \code{nlpcaNet} class is not meant for user-level
##' usage.
##'
##' Creating Objects
##' 
##' \code{new("nlpcaNet", net=[the network structure],
##' hierarchic=[hierarchic design],
##' fct=[the functions at each layer], fkt=[the functions used for
##' forward propagation], weightDecay=[incremental decrease of weight
##' changes over iterations (between 0 and 1)], featureSorting=[sort
##' features or not], dataDist=[represents the present values],
##' inverse=[net is inverse mode or not], fCount=[amount of times
##' features were sorted], componentLayer=[which layer is the
##' 'bottleneck' (principal components)],
##' erro=[the used error function], gradient=[the used gradient method],
##' weights=[the present weights],
##' maxIter=[the amount of iterations that was done], scalingFactor=[the
##' scale of the original matrix])}
##' 
##' Slots
##' 
##' \describe{
##' \item{net}{"matrix",  matrix showing the representation of the
##' neural network, e.g. (2,4,6) for a network with two features, a
##' hidden layer and six output neurons (original variables).}
##' \item{hierarchic}{"list",  the hierarchic design of the network,
##' holds 'idx' (), 'var' () and layer (which layer is the principal
##' component layer).}
##' \item{fct}{"character",  a vector naming the functions that will be
##' applied on each layer. "linr" is linear (i.e.) standard matrix
##' products and "tanh" means that the arcus tangens is applied on the
##' result of the matrix product (for non-linearity).}
##' \item{fkt}{"character",  same as fct but the functions used during
##' back propagation.}
##' \item{weightDecay}{"numeric",  the value that is used to
##' incrementally decrease the weight changes to ensure convergence.}
##' \item{featureSorting}{"logical", indicates if features will be
##' sorted or not. This is used to make the NLPCA assume properties
##' closer to those of standard PCA were the first component is more
##' important for reconstructing the data than the second component.}
##' \item{dataDist}{"matrix", a matrix of ones and zeroes indicating
##' which values will add to the errror.}
##' \item{inverse}{"logical", network is inverse mode (currently only
##' inverse is supported) or not. Eg. the case when we have truly
##' missing values and wish to impute them.}
##' \item{fCount}{"integer", Counter for the amount of times features
##' were really sorted.}
##' \item{componentLayer}{"numeric", the index of 'net' that is the
##' component layer.}
##' \item{error}{"function", the used error function. Currently only one
##' is provided \code{errorHierarchic}.}
##' \item{gradient}{"function", the used gradient function. Currently
##' only one is provided  \code{derrorHierarchic}}
##' \item{weights}{"list", A list holding managements of the
##' weights. The list has two functions, weights$current() and
##' weights$set() which access a matrix in the local environment of
##' this object.}
##' \item{maxIter}{"integer", the amount of iterations used to train
##' this network.}
##' \item{scalingFactor}{"numeric", training the network is best made
##' with 'small' values so the original data is scaled down to a
##' suitable range by division with this number.}}
##' 
##' Methods
##'
##' \describe{ \item{vector2matrices}{Returns the
##' weights in a matrix representation.} }   
##' @title Class representation of the NLPCA neural net
##' @docType class
##' @aliases nlpcaNet nlpcaNet-class
##' @seealso \code{\link{nlpca}}
##' @aliases nFit nFit-class
##' @exportClass nlpcaNet
##' @keywords classes
##' @name pcaNet
##' @author Henning Redestig 
setClass("nlpcaNet",
         representation(net="matrix",
                        hierarchic="list",
                        fct="character",
                        fkt="character",
                        weightDecay="numeric",
                        featureSorting="logical",
                        dataDist="matrix",
                        inverse="logical",
                        fCount="integer",
                        componentLayer="integer",
                        error="function",
                        gradient="function",
                        weights="list",
                        maxIter="integer",
                        scalingFactor="numeric"),
         prototype(net=rbind(c(4,6,2,6,4)),
                   hierarchic=list(var=rbind(c(1,1,0.01)),
                     layer=3, idx=rbind(c(1,1,0),c(0,1,1))),
                   fct=c("linr", "tanh", "linr", "tanh", "linr"),
                   fkt=c("tanh", "linr", "tanh", "linr"),
                   weightDecay=0.001,
                   featureSorting=TRUE,
                   inverse=FALSE,
                   dataDist=NULL,
                   fCount=as.integer(0),
                   componentLayer=as.integer(3),
                   error=errorHierarchic,
                   gradient=derrorHierarchic,
                   weights=NULL,
                   maxIter=as.integer(1200),
                   scalingFactor=NULL))
setAs("NULL", "nlpcaNet",
      function(from, to){
        new(to)
      })

##' This is a class representation of a PCA result
##' 
##' \bold{Creating Objects}\cr
##' \code{new("pcaRes", scores=[the scores], loadings=[the loadings],
##' nPcs=[amount of PCs], R2cum=[cumulative R2], nObs=[amount of
##' observations], nVar=[amount of variables], R2=[R2 for each
##' individual PC], sDev=[stdev for each individual PC],
##' centered=[was data centered], center=[original means],
##' varLimit=[what variance limit was exceeded], method=[method used to
##' calculate PCA], missing=[amount of NAs], 
##' completeObs=[estimated complete observations])}
##' 
##' \bold{Slots}\cr
##' \describe{
##'     \item{scores}{"matrix",  the calculated scores}
##'     \item{loadings}{"matrix",  the calculated loadings}
##'     \item{R2cum}{"numeric",  the cumulative R2 values}
##'     \item{sDev}{"numeric",  the individual standard
##'       deviations of the score vectors}
##'     \item{R2}{"numeric",  the individual R2 values}
##'     \item{cvstat}{"numeric",  cross-validation statistics}
##'     \item{nObs}{"numeric", number of observations}
##'     \item{nVar}{"numeric", number of variables}
##'     \item{centered}{"logical", data was centered or not}
##'     \item{center}{"numeric", the original variable centers}
##'     \item{scaled}{"logical", data was scaled or not}
##'     \item{scl}{"numeric", the original variable scales}
##'     \item{varLimit}{"numeric", the exceeded variance limit}
##'     \item{nPcs,nP}{"numeric", the number of calculated PCs}
##'     \item{method}{"character", the method used to perform PCA}
##'     \item{missing}{"numeric", the total amount of missing values in
##'       original data}
##'     \item{completeObs}{"matrix", the estimated complete observations}
##'     \item{network}{"nlpcaNet", the network used by non-linear PCA}
##'   }
##' 
##' \bold{Methods (not necessarily exhaustive)}\cr
##' \describe{
##'     \item{print}{Print function}
##'     \item{summary}{Extract information about PC relevance}
##'     \item{screeplot}{Plot a barplot of standard deviations for PCs}
##'     \item{slplot}{Make a side by side score and loadings plot}
##'     \item{nPcs}{Get the number of PCs}
##'     \item{nObs}{Get the number of observations}
##'     \item{cvstat}{Cross-validation statistics}
##'     \item{nVar}{Get the number of variables}
##'     \item{loadings}{Get the loadings}
##'     \item{scores}{Get the scores}
##'     \item{dim}{Get the dimensions (number of observations, number of
##'       features)}
##'     \item{centered}{Get a logical indicating if centering was done as
##'       part of the model}
##'     \item{center}{Get the averages of the original variables.}
##'     \item{completeObs}{Get the imputed data set}
##'     \item{method}{Get a string naming the used PCA method}
##'     \item{sDev}{Get the standard deviations of the PCs}
##'     \item{scaled}{Get a logical indicating if scaling was done as
##'       part of the model}
##'     \item{scl}{Get the scales of the original variablesb}
##'     \item{R2cum}{Get the cumulative R2}
##'   }
##' @title Class for representing a PCA result
##' @keywords classes
##' @exportClass pcaRes
##' @docType class
##' @name pcaRes
##' @aliases pcaRes pcaRes-class 
##' @author Henning Redestig
setClass("pcaRes",
         representation(completeObs="matrix",
			scores="matrix",
                        loadings="matrix",
                        R2cum="numeric",
                        R2="numeric",   # ditch, get from R2cum
                        cvstat="numeric",   # ditch, get from R2cum
                        sDev="numeric", # ditch, get from scores
                        nObs="numeric", # ditch, get from scores
                        nVar="numeric", 
                        centered="logical", 
                        center="numeric",
                        subset="numeric",
                        scaled="character", 
                        scale="numeric",
                        varLimit="numeric", # ditch, useless
                        nPcs="numeric",     # ditch, get from scores
                        method="character",
                        missing="matrix",
                        network="nlpcaNet"),
         prototype(completeObs=NULL,
		   scores=NULL,
                   loadings=NULL,
                   R2cum=NULL,
                   R2=NULL,
                   subset=NULL,
                   cvstat=NULL,
                   sDev=NULL,
                   nObs=NULL,
                   nVar=NULL,
                   centered=NULL,
                   center=NULL,
                   scaled=NULL,
                   scale=NULL,
                   varLimit=NULL,
                   nPcs=NULL,
                   method=NULL,
                   missing=NULL,
                   network=NULL))
setAs("NULL", "pcaRes",
      function(from, to){
        new(to)
      })

##' This is a class representation of nearest neighbour imputation
##' (nni) result
##'
##' \bold{Creating Objects}\cr
##' \code{new("nniRes", completeObs=[the estimated complete
##' observations], k=[cluster size], nObs=[amount of observations],
##' nVar=[amount of variables], centered=[was the data centered befor
##' running LLSimpute], center=[original means], method=[method used
##' to perform clustering], missing=[amount of NAs])}
##' 
##' \bold{Slots}\cr
##' \describe{
##'   \item{completeObs}{"matrix", the estimated complete observations}
##'   \item{nObs}{"numeric", amount of observations}
##'   \item{nVar}{"numeric", amount of variables}
##'   \item{correlation}{"character", the correlation method used
##'         (pearson, kendall or spearman)}
##'   \item{centered}{"logical", data was centered or not}
##'   \item{center}{"numeric", the original variable centers}
##'   \item{k}{"numeric", cluster size}
##'   \item{method}{"character", the method used to perform the clustering}
##'   \item{missing}{"numeric", the total amount of missing values in
##'     original data}
##' }
##' 
##' \bold{Methods}\cr
##' \describe{ \item{print}{Print function} }
##' @title Class for representing a nearest neighbour imputation result
##' @docType class
##' @exportClass nniRes
##' @name nniRes
##' @keywords classes
##' @aliases nniRes nniRes-class
##' @author Wolfram Stacklies
setClass("nniRes",
         representation(completeObs="matrix",
                        nObs="numeric",
                        nVar="numeric",
                        centered="logical",
                        center="numeric",
                        k="numeric",
                        method="character",
                        correlation="character",
                        missing="numeric"),
         prototype(completeObs=NULL,
                   nObs=NULL,
                   nVar=NULL,
                   centered=NULL,
                   center=NULL,
                   k=NULL,
                   method=NULL,
                   correlation=NULL,
                   missing=NULL))
setAs("NULL", "nniRes",
      function(from, to) {
          new(to)
      })

##' Create an object that holds the weights for nlpcaNet. Holds and
##' sets weights in using an environment object.
##' @param w  \code{matrix} -- New weights
##' @return A weightsAccound with \code{set} and \code{current}
##' functions.
##' @author Henning Redestig
weightsAccount <- function(w) {
  list(
       set = function(newWeights) {
         if(!inherits(newWeights, "matrix"))
           stop("The weights must inherit from matrix")
          w <<- newWeights
       },
       current = function() {
         w
       }
       )
}

##' Complete copy of nlpca net object
##' @param nlnet a nlnet
##' @return A copy of the input nlnet
##' @author Henning Redestig
forkNlpcaNet <- function(nlnet) {
  res <- new("nlpcaNet")
  res@net <- nlnet@net
  res@hierarchic <- nlnet@hierarchic
  res@fct <- nlnet@fct
  res@fkt <- nlnet@fkt
  res@weightDecay <- nlnet@weightDecay
  res@featureSorting <- nlnet@featureSorting
  res@dataDist <- nlnet@dataDist
  res@inverse <- nlnet@inverse
  res@fCount <- nlnet@fCount
  res@componentLayer <- nlnet@componentLayer
  res@error <- nlnet@error
  res@gradient <- nlnet@gradient
  res@weights <- weightsAccount(nlnet@weights$current())
  res@maxIter <- nlnet@maxIter
  res@scalingFactor <- nlnet@scalingFactor
  res
}

##' Missing value estimation using local least squares (LLS).  First,
##' k variables (for Microarrya data usually the genes)  are selected
##' by pearson, spearman or kendall correlation coefficients.  Then
##' missing values are imputed by a linear combination of the k
##' selected variables. The optimal combination is found by LLS
##' regression.  The method was first described by Kim et al,
##' Bioinformatics, 21(2),2005.
##'
##' Missing values are denoted as \code{NA}\cr It is not recommended
##' to use this function directely but rather to use the nni() wrapper
##' function. The methods provides two ways for missing value
##' estimation, selected by the \code{allVariables} option. The first
##' one is to use only complete variables for the  regression. This is
##' preferable when the number of incomplete variables is relatively
##' small.
##' 
##' The second way is to consider all variables as candidates for the
##' regression.  Hereby missing values are initially replaced by the
##' columns wise mean.  The method then iterates, using the current
##' estimate as input for the regression until the change between new
##' and old estimate falls below a threshold (0.001).
##' 
##' @title LLSimpute algorithm
##' @param Matrix \code{matrix} -- Data containing the variables
##' (genes) in columns and observations (samples) in rows. The data
##' may contain missing values, denoted as \code{NA}.
##' @param k \code{numeric} -- Cluster size, this is the number of
##' similar genes used for regression.
##' @param center \code{boolean} -- Mean center the data if TRUE
##' @param completeObs \code{boolean} -- Return the estimated complete
##' observations if  TRUE. This is the input data with NA values
##' replaced by the estimated values.
##' @param correlation \code{character} -- How to calculate the
##' distance between genes.  One out of pearson | kendall | spearman ,
##' see also help("cor").
##' @param allVariables \code{boolean} -- Use only complete genes to
##' do the regression if TRUE, all genes if FALSE.
##' @param maxSteps \code{numeric} -- Maximum number of iteration
##' steps if allGenes = TRUE.
##' @param xval \code{numeric} Use LLSimpute for cross
##' validation. xval is the index of the gene to estimate, all other
##' incomplete genes will be ignored if this parameter is set. We do
##' not consider them in the cross-validation.
##' @param verbose \code{boolean} -- Print step number and relative
##' change if TRUE and  allVariables = TRUE
##' @param ... Reserved for parameters used in future version of the
##' algorithm
##' @note Each step the generalized inverse of a \code{miss} x k
##' matrix is calculated. Where \code{miss} is the number of missing
##' values in  variable j and \code{k} the number of neighbours. This
##' may be slow for large values of k and / or many missing
##' values. See also help("ginv").
##' @return   \item{nniRes}{Standard nni (nearest neighbour
##' imputation) result object of this package. See
##' \code{\link{nniRes}} for details.}
##' @seealso \code{\link{pca}, \link{nniRes}, \link{nni}}.
##' @examples
##' ## Load a sample metabolite dataset (metaboliteData) with already 5\% of
##' ## data missing
##' data(metaboliteData)
##' ## Perform llsImpute using k = 10
##' ## Set allVariables TRUE because there are very few complete variables
##' result <- llsImpute(metaboliteData, k = 10, correlation="pearson", allVariables=TRUE)
##' ## Get the estimated complete observations
##' cObs <- completeObs(result)
##' @keywords multivariate
##' @export
##' @references Kim, H. and Golub, G.H. and Park, H.  - Missing value
##' estimation for DNA microarray gene expression data: local least
##' squares imputation.  \emph{Bioinformatics, 2005; 21(2):187-198.}
##' 
##' Troyanskaya O. and Cantor M. and Sherlock G. and Brown P. and
##' Hastie T. and Tibshirani R. and Botstein D. and Altman RB.  -
##' Missing value estimation methods for DNA microarrays.
##' \emph{Bioinformatics. 2001 Jun;17(6):520-525.}
##' @author Wolfram Stacklies
llsImpute <- function(Matrix, k=10, center=FALSE, completeObs=TRUE,
                      correlation="pearson", 
                      allVariables=FALSE, maxSteps=100, xval=NULL,
                      verbose=FALSE, ...) {

    threshold <- 0.001

    correlation <- match.arg(correlation, c("pearson", "kendall", "spearman"))

    ## If the data is a data frame, convert it into a matrix
    Matrix <- as.matrix(Matrix, rownames.force=TRUE)
    ## And now check if everything is right...
    if ( !checkData(Matrix, verbose = interactive()) ) {
        stop("Invalid data format! Use checkData(Matrix, verbose = TRUE) for details.\n")
    }

    ## Exit if number of neighbours exceeds number of columns
    if (k > ncol(Matrix))
        stop("Cluster size larger than the number of columns, choose a k < ncol(Matrix)!")
 
    ## Set allVariables TRUE if k exceeds number of complete genes
    ## Print warning messages in the first case and when less than 50% of all genes are complete
    ## and allVariables == FALSE
    cg <- sum( apply(is.na(Matrix), 2, sum) == 0)
    if ( (k > cg) && (!allVariables) ) {
        warning("Cluster size larger than number of complete genes, using allVariables = TRUE")
        allVariables <- TRUE
    } else if ( (cg < (ncol(Matrix) / 2)) && (!allVariables) ) {
        warning("Less than 50% of the genes are complete, consider using allVariables = TRUE")
    } else if (sum(is.na(Matrix)) == 0)
        stop("No missing values, no need for missing value imputation :))")

    ## Find all genes with missing values
    missing <- apply(is.na(Matrix), 2, sum) > 0
    missIx <- which(missing == TRUE)
    # For cross validation we want to only estimate one variable, the others
    # are not considered in the cross validation anyway
    if (!is.null(xval))
        missIx = xval
    obs <- Matrix    ## working copy of the data
    Ye <- Matrix     ## Estimated complete observations

    ## Center the data column wise
    if (center) {
        obs   <- scale(Matrix, center = TRUE, scale = FALSE)
        Ye    <- obs
        means <- attr(Ye, "scaled:center")
    }

    if (allVariables) {
        compIx <- 1:ncol(obs)
        ## Impute the row average
        rowMeans <- apply(obs, 1, mean, na.rm = TRUE)
        for (i in 1:nrow(obs)) {
            obs[i, is.na(Matrix[i,])] <- rowMeans[i]
        }
        ## distances between all genes, ignore the diagonal (correlation to itself)
        distance = abs(cor(obs, obs, method = correlation))
    } else {
        compIx <- which(missing == FALSE)
        ## missing genes are the rows, complete genes the columns
        distance = abs(cor(obs[,missIx, drop=FALSE], obs[,compIx, drop=FALSE], use="pairwise.complete.obs",
                       method = correlation))
    }

    change <- Inf
    step <- 0
    while ( (change > threshold) && (step < maxSteps) ) {
        step <- step + 1
        iteration <- 0
        
        ## Do the regression and imputation
        for (index in missIx) {
            iteration <- iteration + 1
	    if (allVariables) {
                similar <- sort(distance[iteration,], index.return = TRUE, decreasing = TRUE)
                simIx <- compIx[ similar$ix[similar$ix != iteration][1:k] ]
            } else {
                similar <- sort(distance[iteration,], index.return = TRUE, decreasing = TRUE)
                simIx <- compIx[ similar$ix[1:k] ]
            }

            ##
            ## Do a regression against the k most similar genes
            ## See Kim et. al 2005 for details
            ##
            target <- obs[, index, drop = FALSE]
            tMiss <- is.na(Matrix[, index, drop = FALSE])

            Apart <- obs[!tMiss, simIx, drop = FALSE]
            Bpart <- obs[tMiss, simIx, drop = FALSE]
            targetComplete <- target[!tMiss, , drop = FALSE]
            X <- MASS::ginv(Apart) %*% targetComplete
            estimate <- Bpart %*% X

            ## Impute the estimate
            Ye[tMiss, index] <- estimate
        }

        ## We do not want to iterate if allVariables == FALSE
        if (!allVariables || !is.null(xval)) {
            break
        } else {
            ## relative change in estimation
            change <- sqrt(sum( (obs - Ye)^2 ) / sum(obs^2))
            obs <- Ye
            if (verbose) {
                cat("Step number     : ", step, '\n')
                cat("Relative change : ", change, '\n')
                cat("---------------", '\n')
            }
        }
    }

    ## Add the original mean
    if (center) {
        for(i in 1:ncol(Ye)) {
            Ye[,i] <- Ye[,i] + means[i]
        }
    }

    ## Build the nniRes object
    ##
    result <- new("nniRes")

    if(completeObs) {
        Ye[!is.na(Matrix)] <- Matrix[!is.na(Matrix)]
        result@completeObs <- Ye
    }
    result@centered        <- center
    result@center          <- attr(scale(Matrix, center = TRUE, scale = FALSE), "scaled:center")
    result@nObs            <- nrow(Matrix)
    result@nVar            <- ncol(Matrix)
    result@method          <- "llsImpute"
    result@correlation     <- correlation
    result@k               <- k
    result@missing         <- sum(is.na(Matrix))

    return(result)        
} 

### R code from vignette source 'outliers.Rnw'

###################################################
### code chunk number 1: outliers.Rnw:59-60
###################################################
library(pcaMethods)


###################################################
### code chunk number 2: outliers.Rnw:62-67
###################################################
data(metaboliteDataComplete)
mdc          <- scale(metaboliteDataComplete, center=TRUE, scale=FALSE)
cond         <- runif(length(mdc)) < 0.05
mdcOut       <- mdc
mdcOut[cond] <- 10


###################################################
### code chunk number 3: outliers.Rnw:70-73
###################################################
resSvd       <- pca(mdc, method="svd", nPcs=5, center=FALSE)
resSvdOut    <- pca(mdcOut, method="svd", nPcs=5, center=FALSE)
resRobSvd    <- pca(mdcOut, method="robustPca", nPcs=5, center=FALSE)


###################################################
### code chunk number 4: outliers.Rnw:77-80
###################################################
mdcNa        <- mdc
mdcNa[cond]  <- NA
resPPCA      <- pca(mdcNa, method="ppca", nPcs=5, center=FALSE)


###################################################
### code chunk number 5: outliers.Rnw:88-97
###################################################
par(mfrow=c(2,2))
plot(loadings(resSvd)[,1], loadings(resSvdOut)[,1], 
xlab="Loading 1 SVD", ylab="Loading 1 SVD with outliers")
plot(loadings(resSvd)[,1], loadings(resRobSvd)[,1],
xlab="Loading 1 SVD", ylab="Loading 1 robustSVD with outliers")
plot(loadings(resSvd)[,1], loadings(resPPCA)[,1],
xlab="Loading 1 SVD", ylab="Loading 1 PPCA with outliers=NA")
plot(loadings(resRobSvd)[,1], loadings(resPPCA)[,1],
xlab="Loading 1 robust SVD with outliers", ylab="Loading 1 svdImpute with outliers=NA")

##' Implementation of probabilistic PCA (PPCA). PPCA allows to perform
##' PCA on incomplete data and may be used for missing value
##' estimation.  This script was implemented after the Matlab version
##' provided by Jakob Verbeek ( see
##' \url{http://lear.inrialpes.fr/~verbeek/}) and the draft \emph{``EM
##' Algorithms for PCA and Sensible PCA''} written by Sam Roweis.
##'
##' Probabilistic PCA combines an EM approach for PCA with a
##' probabilistic model. The EM approach is based on the assumption
##' that the latent variables as well as the noise are normal
##' distributed.
##'
##' In standard PCA data which is far from the training set but close
##' to the principal subspace may have the same reconstruction error.
##' PPCA defines a likelihood function such that the likelihood for
##' data far from the training set is much lower, even if they are
##' close to the principal subspace.  This allows to improve the
##' estimation accuracy.
##'
##' A method called \code{kEstimate} is provided to estimate the
##' optimal number of components via cross validation.  In general few
##' components are sufficient for reasonable estimation accuracy. See
##' also the package documentation for further discussion on what kind
##' of data PCA-based missing value estimation is advisable.
##'
##' \bold{Complexity:}\cr Runtime is linear in the number of data,
##' number of data dimensions and number of principal components.
##'
##' \bold{Convergence:}  The threshold indicating convergence was
##' changed from 1e-3 in 1.2.x to 1e-5 in the current version  leading
##' to more stable results.  For reproducability you can set the seed
##' (parameter seed) of the random number generator. If used for
##' missing value estimation, results may be checked by simply running
##' the algorithm several times with changing seed, if the estimated
##' values show little variance the algorithm converged well. 
##' @title Probabilistic PCA 
##' @param Matrix \code{matrix} -- Data containing the variables in
##' columns and observations in rows. The data may contain missing
##' values, denoted as \code{NA}.
##' @param nPcs \code{numeric} -- Number of components to
##' estimate. The preciseness of the missing value estimation depends
##' on the number of components, which should resemble the internal
##' structure of the data.
##' @param seed \code{numeric} Set the seed for the random number
##' generator. PPCA creates fills the initial loading matrix with
##' random numbers chosen from a normal distribution. Thus results may
##' vary slightly. Set the seed for exact reproduction of your
##' results.
##' @param threshold Convergence threshold.
##' @param maxIterations the maximum number of allowed iterations
##' @param ... Reserved for future use. Currently no further
##' parameters are used.
##' @note Requires \code{MASS}. It is not recommended to use this
##' function directely but rather to use the pca() wrapper function.
##' @return Standard PCA result object used by all PCA-based methods
##' of this package. Contains scores, loadings, data mean and
##' more. See \code{\link{pcaRes}} for details.
##' @seealso \code{\link{bpca}, \link{svdImpute}, \link{prcomp},
##' \link{nipalsPca}, \link{pca}, \link{pcaRes}}.
##' @examples
##' ## Load a sample metabolite dataset with 5\% missing values (metaboliteData)
##' data(metaboliteData)
##' ## Perform probabilistic PCA using the 3 largest components
##' result <- pca(t(metaboliteData), method="ppca", nPcs=3, seed=123)
##' ## Get the estimated complete observations
##' cObs <- completeObs(result)
##' ## Plot the scores
##' plotPcs(result, type = "scores")
##' \dontshow{
##'   stopifnot(sum((fitted(result) - t(metaboliteData))^2, na.rm=TRUE) < 200)
##' }
##' @keywords multivariate
##' @author Wolfram Stacklies
##' @export
ppca <- function(Matrix, nPcs=2, seed=NA, threshold=1e-5, maxIterations=1000, ...) {
  ## Set the seed to the user defined value. This affects the generation
  ## of random values for the initial setup of the loading matrix
  if (!is.na(seed)) 
    set.seed(seed)

  N <- nrow(Matrix)
  D <- ncol(Matrix)

  Obs <- !is.na(Matrix)
  hidden <- which(is.na(Matrix))
  missing <- length(hidden)

  if(missing) { Matrix[hidden] <- 0 } 

  ## ------- Initialization
  r <- sample(N)
  C <- t(Matrix[r[1:nPcs], ,drop = FALSE])
  ## Random matrix with the same dimnames as Matrix
  C <- matrix(rnorm(C), nrow(C), ncol(C), dimnames = labels(C) )
  CtC <- t(C) %*% C
  ## inv(C'C) C' X is the solution to the EM problem
  X  <- Matrix %*% C %*% solve(CtC)
  recon    <- X %*% t(C)
  recon[hidden] <- 0
  ss <- sum(sum((recon - Matrix)^2)) / (N * D - missing)

  count <- 1
  old <- Inf
  
  ## ------ EM iterations
  while (count > 0) {
    ## E-step, (co)variances
    Sx <- solve(diag(nPcs) + CtC/ss) 
    ss_old <- ss
    if(missing) {
      proj <- X %*% t(C)
      Matrix[hidden] <- proj[hidden]
    }
    
    ## E step: expected values
    X <- Matrix %*% C %*% Sx / ss
    
    ## M-step
    SumXtX <- t(X) %*% X

    ## Replace the right matrix division from matlab
    C <- (t(Matrix) %*% X) %*% solve( (SumXtX + N * Sx) )
    
    CtC <- t(C) %*% C
    ss <- ( sum(sum( (C %*% t(X) - t(Matrix))^2 )) + N * sum(sum(CtC %*% Sx)) +
           missing * ss_old ) / (N * D)

    objective <- N * (D * log(ss) + sum(diag(Sx)) - log(det(Sx)) ) +
      sum(diag(SumXtX)) - missing * log(ss_old)

    rel_ch <- abs( 1 - objective / old )
    old <- objective

    count <- count + 1
    if( rel_ch < threshold & count > 5 ) {
      count <- 0
    }
    else if (count > maxIterations) {
      count <- 0
      warning("stopped after max iterations, but rel_ch was > threshold")
    }
  } ## End EM iteration
  C <- orth(C)
  evs <- eigen( cov(Matrix %*% C) )
  vals <- evs[[1]]
  vecs <- evs[[2]]
  
  C <- C %*% vecs
  X <- Matrix %*% C

  ## Paramters in original Matlab implementation were:
  ## C (D by d)    - C has the approximate loadings (eigenvectors of
  ## the covariance matrix)
  ##          as columns.
  ## X        - The approximate scores 
  ## Matrix (N by D)    - Expected complete observations.
  ## M (D by 1)    - Column wise data mean
  ## ss (scalar)    - isotropic variance outside subspace

  R2cum <- rep(NA, nPcs)
  TSS <- sum(Matrix^2, na.rm=TRUE)
  for (i in 1:ncol(C)) {
    difference <- Matrix - (X[,1:i, drop=FALSE] %*% t(C[,1:i, drop=FALSE]))
    R2cum[i] <- 1 - (sum(difference^2, na.rm=TRUE) / TSS)
  }

  res <- new("pcaRes")
  res@scores <- X
  res@loadings <- C
  res@R2cum <- R2cum
  res@method <- "ppca"
  return(res)
}

##' This function can be used to conveniently replace the expression
##' matrix in an \code{ExpressionSet} with the completed data from a
##' \code{pcaRes} object.
##'
##' This is not a standard \code{as} function as \code{pcaRes}
##' object alone not can be converted to an \code{ExpressionSet} (the
##' \code{pcaRes} object does not hold any \code{phenoData} for
##' example).
##' @title Convert pcaRes object to an expression set
##' @param object \code{pcaRes} -- The object containing the completed
##' data.
##' @param exprSet \code{ExpressionSet} -- The object passed on to
##' \code{pca} for missing value estimation.  
##' @return An object without missing values of class \code{ExpressionSet}.
##' @export 
##' @author Wolfram Stacklies \cr CAS-MPG Partner Institute for
##' Computational Biology, Shanghai, China
##' @keywords multivariate
asExprSet <- function(object, exprSet) {
  if(!inherits(exprSet, "ExpressionSet"))
    stop("Parameter exprSet must be of type ExpressionSet")
  if(!inherits(object, "pcaRes") & !inherits(object, "nniRes"))
    stop("Parameter object must be either of type pcaRes or nniRes")
  if (is.null(completeObs(object)))
    stop("completeObs(object) is NULL, exiting")
  if(!all(dim(exprs(exprSet)) == dim(t(completeObs(object)))))
    stop("Dimensions of exprs(exprSet) and completeObs(object) do not match. 
Did you really do missing value estimation using this ExpressionSet object?")
  exprs(exprSet) <- t(completeObs(object))
  return(exprSet)
}

##' The leverages of PCA model indicate how much influence each
##' observation has on the PCA model. Observations with high leverage
##' has caused the principal components to rotate towards them. It can
##' be used to extract both "unimportant" observations as well as
##' picking potential outliers.
##'
##' Defined as \eqn{Tr(T(T'T)^{-1}T')}{Tr(T(T'T)^(-1)T')}
##' @title Extract leverages of a PCA model
##' @param object a \code{pcaRes} object
##' @return The observation leverages as a numeric vector
##' @references Introduction to Multi- and Megavariate Data Analysis
##' using Projection Methods (PCA and PLS), L. Eriksson, E. Johansson,
##' N. Kettaneh-Wold and S. Wold, Umetrics 1999, p. 466
##' @examples
##' data(iris)
##' pcIr <- pca(iris[,1:4])
##' ## versicolor has the lowest leverage
##' with(iris, plot(leverage(pcIr)~Species))
##' @keywords multivariate
##' @aliases leverage leverage,pcaRes-method
##' @author Henning Redestig
setMethod("leverage", "pcaRes",
          function(object) {
            diag(scores(object) %*%
                 solve(crossprod(scores(object))) %*% t(scores(object)))
          })

##' Distance to the model of X-space.
##'
##' Measures how well described the observations are, i.e. how well
##' they fit in the mode. High DModX indicate a poor fit. Defined as:
##'
##' \eqn{\frac{\sqrt{\frac{SSE_i}{K-A}}}{\sqrt{\frac{SSE}{(N-A-A_0)(K-A)}}}}
##'
##' For observation \eqn{i}, in a model with \eqn{A} components,
##' \eqn{K} variables and \eqn{N} obserations. SSE is the squared sum
##' of the residuals. \eqn{A_0} is 1 if model was centered and 0
##' otherwise. DModX is claimed to be approximately F-distributed and
##' can therefore be used to check if an observation is significantly
##' far away from the PCA model assuming normally distributed data.
##'
##' Pass original data as an argument if the model was calculated with
##' \code{completeObs=FALSE}.
##' @title DModX
##' @usage DModX(object, dat, newdata=FALSE, type=c("normalized","absolute"), ...)
##' @param object a pcaRes object
##' @param dat the original data, taken from \code{completeObs} if
##' left missing.
##' @param newdata logical indicating if this data was part of the
##' training data or not. If it was, it is adjusted by a near one factor
##' \eqn{v=(N/ (N-A-A0))^-1}
##' @param type if absolute or normalized values should be
##' given. Normalized values are adjusted to the the total RSD of the
##' model.
##' @param ... Not used 
##' @return A vector with distances from observations to the PCA model
##' @aliases DModX DModX,pcaRes-method
##' @examples
##' data(iris)
##' pcIr <- pca(iris[,1:4])
##' with(iris, plot(DModX(pcIr)~Species))
##' @references Introduction to Multi- and Megavariate Data Analysis
##' using Projection Methods (PCA and PLS), L. Eriksson, E. Johansson,
##' N. Kettaneh-Wold and S. Wold, Umetrics 1999, p. 468
##' @author Henning Redestig
setMethod("DModX", "pcaRes",
          function(object, dat, newdata=FALSE, type=c("normalized","absolute"), ...) {
            type <- match.arg(type)
            if(missing(dat)) {
              if(!is.null(completeObs(object)))
                dat <- completeObs(object)
              else stop("missing data when calculating DModX")
            }
            A0 <- as.integer(centered(object))
            ny <- ifelse(newdata, 1,
                         sqrt(nObs(object) /
                              (nObs(object) - nP(object) - A0)))
            E2 <- resid(object, dat)^2
            s <- sqrt(rowSums(E2) / (nVar(object) - nP(object))) * ny
            if(type == "absolute")
              return(s)
            s0 <- sqrt(sum(E2) / ((nObs(object) - nP(object) - A0) *
                                  (nVar(object) - nP(object))))
            s / s0
          })

##' Get number of PCs
##' @param object pcaRes object
##' @param ...  not used
##' @return Number of PCs
##' @aliases nP nP,pcaRes-method
##' @usage nP(object, ...)
##' @author Henning Redestig
setMethod("nP", "pcaRes", function(object, ...) {
  if(is.null(object@nPcs) & !is.null(scores(object)))
    return(ncol(scores(object)))
  object@nPcs
})


##' Get cross-validation statistics (e.g. \eqn{Q^2}).
##' @param object pcaRes object
##' @param ...  not used
##' @return vector CV statistics 
##' @aliases cvstat cvstat,pcaRes-method
##' @usage cvstat(object, ...)
##' @author Henning Redestig
setMethod("cvstat", "pcaRes", function(object, ...) {
  object@cvstat
})

##' Get number of PCs.
##' @param object pcaRes object
##' @param ...  not used
##' @note Try to use \code{link{nP}} instead since \code{nPcs} tend to
##' clash with argument names.
##' @return Number of PCs
##' @usage nPcs(object, ...)
##' @aliases nPcs nPcs,pcaRes-method
##' @author Henning Redestig
setMethod("nPcs", "pcaRes", function(object, ...) {
  nP(object)
})

##' Get the number of observations used to build the PCA model.
##' @param object pcaRes object
##' @param ... Not used
##' @usage nObs(object, ...)
##' @aliases nObs nObs,pcaRes-method
##' @return Number of observations
##' @author Henning Redestig
setMethod("nObs", "pcaRes", function(object, ...) {
  object@nObs
})

##' Get the number of variables used to build the PCA model.
##' @param object pcaRes object
##' @param ... Not used
##' @usage nVar(object, ...)
##' @aliases nVar nVar,pcaRes-method
##' @return Number of variables
##' @author Henning Redestig
setMethod("nVar", "pcaRes", function(object, ...) {
  object@nVar
})

##' Check centering was part of the model
##' @param object pcaRes object
##' @param ... Not used
##' @usage centered(object, ...)
##' @aliases centered centered,pcaRes-method
##' @return TRUE if model was centered
##' @author Henning Redestig
setMethod("centered", "pcaRes", function(object, ...) {
  if(is.null(object@centered))
    return(FALSE)
  object@centered  
})

##' Get the centers of the original variables
##' @param object pcaRes object
##' @param ... Not used
##' @usage center(object, ...)
##' @aliases center center,pcaRes-method
##' @return Vector with the centers
##' @author Henning Redestig
setMethod("center", "pcaRes", function(object, ...) {
  object@center
})

setMethod("completeObs", "pcaRes", function(object, ...) {
  object@completeObs
})
##' Get the original data with missing values replaced with predicted
##' values.
##' @param object object to fetch complete data from
##' @param ... Not used
##' @usage completeObs(object, ...)
##' @aliases completeObs completeObs,nniRes-method
##' completeObs,pcaRes-method
##' @return Completed data (matrix)
##' @author Henning Redestig
setMethod("completeObs", "nniRes", function(object, ...) {
  object@completeObs
})

##' Get the used PCA method
##' @param object pcaRes object
##' @param ... Not used
##' @usage method(object, ...)
##' @aliases method method,pcaRes-method
##' @return The used pca method
##' @author Henning Redestig
setMethod("method", "pcaRes", function(object, ...) {
  object@method
})

setMethod("nmissing", "nniRes", function(object, ...) {
  sum(object@missing)
})
##' Missing values
##' @param object pcaRes object
##' @param ... Not used
##' @usage nmissing(object, ...)
##' @aliases nmissing nmissing,pcaRes-method nmissing,nniRes-method
##' @return Get the number of missing values
##' @author Henning Redestig
setMethod("nmissing", "pcaRes", function(object, ...) {
  sum(object@missing)
})

##' Get a matrix with indicating the elements that were missing in the
##' input data. Convenient for estimating imputation performance.
##' @param object pcaRes object
##' @param ... Not used
##' @usage wasna(object, ...)
##' @aliases wasna wasna,pcaRes-method
##' @return A matrix with logicals
##' @examples
##' data(metaboliteData)
##' data(metaboliteDataComplete)
##' result <- pca(metaboliteData, nPcs=2)
##' plot(completeObs(result)[wasna(result)], metaboliteDataComplete[wasna(result)])
##' @author Henning Redestig
setMethod("wasna", "pcaRes", function(object, ...) {
  object@missing
})

##' Get the standard deviations of the scores (indicates their
##' relevance)
##' @param object pcaRes object
##' @param ... Not used
##' @usage sDev(object, ...)
##' @aliases sDev sDev,pcaRes-method
##' @return Standard devations of the scores
##' @author Henning Redestig
setMethod("sDev", "pcaRes", function(object, ...) {
  object@sDev
})

##' Check if scaling was part of the PCA model
##' @param object pcaRes object
##' @param ... Not used
##' @usage scaled(object, ...)
##' @aliases scaled scaled,pcaRes-method
##' @return TRUE if scaling was part of the PCA model
##' @author Henning Redestig
setMethod("scaled", "pcaRes", function(object, ...) {
  if(is.null(object@scaled))
    return(FALSE)
  object@scaled != "none"
})

##' Get the scales (e.g. standard deviations) of the original
##' variables
##' @param object pcaRes object
##' @param ... Not used
##' @usage scl(object, ...)
##' @aliases scl scl,pcaRes-method
##' @return Vector with the scales
##' @seealso \code{\link{prep}}
##' @author Henning Redestig
setMethod("scl", "pcaRes", function(object, ...) {
  object@scale 
})

##' Cumulative R2 is the total ratio of variance that is being
##' explained by the model
##' @param object a \code{pcaRes} model
##' @param ...  Not used
##' @return Get the cumulative R2
##' @aliases R2cum R2cum,pcaRes-method
##' @author Henning Redestig
setMethod("R2cum", "pcaRes", function(object, ...) {
  object@R2cum
})

##' Get scores from a pcaRes object
##' @param object a pcaRes object
##' @param ... not used
##' @return The scores as a matrix
##' @export
##' @author Henning Redestig
##' @method scores pcaRes
scores.pcaRes <- function(object, ...) object@scores
##' Get scores from a pcaRes object
##' @param object a pcaRes object
##' @param ... not used
##' @return The scores as a matrix
##' @seealso \code{\link{scores.pcaRes}}
##' @aliases scores scores,pcaRes-method
##' @author Henning Redestig
setMethod("scores", "pcaRes", scores.pcaRes)

##' Get loadings from a pcaRes object
##' @param object a pcaRes object
##' @param ... not used
##' @return The loadings as a matrix
##' @export
##' @author Henning Redestig
##' @method loadings pcaRes
loadings.pcaRes <- function(object, ...) object@loadings
##' Get loadings from a pcaRes object
##' @param object a pcaRes object
##' @param ...  not used
##' @return The loadings as a matrix
##' @seealso \code{\link{loadings.pcaRes}}
##' @author Henning Redestig
##' @aliases loadings,pcaRes-method
setMethod("loadings", "pcaRes", loadings.pcaRes)
##' Crude way to unmask the function with the same name from
##' \code{stats}
##' @param object any object
##' @param ...  not used
##' @return The loadings
##' @author Henning Redestig
##' @aliases loadings loadings,ANY-method
setMethod("loadings", "ANY", function(object,...) {
  stats::loadings(object)
})


##' Dimensions of a PCA model
##' @param x a pcaRes object
##' @return Get the dimensions of this PCA model
##' @method dim pcaRes
##' @export 
##' @author Henning Redestig
dim.pcaRes <- function(x)  {
  res <-  c(nObs(x), nVar(x), nP(x))
  names(res) <- c("nObs", "nVar", "nPcs")
  res
}

##' Print basic information about pcaRes object
##' @title Print/Show for pcaRes
##' @param x a pcaRes object
##' @param ...  not used
##' @return nothing, used for its side effect
##' @name show-methods
##' @export
##' @author Henning Redestig
showPcaRes <- function(x, ...) {
  summary(x)
  cat(nVar(x), "\tVariables\n")
  cat(nObs(x),"\tSamples\n")
  cat(nmissing(x), "\tNAs (",
      round(100 * nmissing(x) / (nObs(x) * nVar(x)),
            getOption("str")$digits.d), "%)\n")
  cat(nP(x), "\tCalculated component(s)\n")
  if(centered(x))
    cat("Data was mean centered before running PCA \n")
  else
    cat("Data was NOT mean centered before running PCA \n")
  if(scaled(x))
    cat("Data was scaled before running PCA \n")
  else
    cat("Data was NOT scaled before running PCA \n")
  cat("Scores structure:\n")
  print(dim(scores(x)))
  cat("Loadings structure:\n")
  if(method(x) == "nlpca") {
    cat("Inverse hierarchical neural network architecture\n")
    cat(drop(x@network@net), "\n")
    cat("Functions in layers\n")
    cat(x@network@fct, "\n")
    cat("hierarchic layer:", x@network@hierarchic$layer, "\n")
    cat("hierarchic coefficients:", x@network@hierarchic$var, "\n")
    cat("scaling factor:", x@network@scalingFactor, "\n")
  }
  else{
    print(dim(loadings(x)))
  }
}
##' @aliases print,pcaRes-method print,nniRes-method
##' @name show-methods
setMethod("print", "pcaRes", showPcaRes)
## @importFrom methods show
##' @aliases show,pcaRes-method show,nniRes-method
##' @param object the object to print information about
##' @name show-methods
setMethod("show", "pcaRes", function(object) showPcaRes(object))

##' Visualize two-components simultaneously
##'
##' This is a method for the generic function 'biplot'.  There is
##' considerable confusion over the precise definitions: those of the
##' original paper, Gabriel (1971), are followed here.  Gabriel and
##' Odoroff (1990) use the same definitions, but their plots actually
##' correspond to \code{pc.biplot = TRUE}. 
##' @title Plot a overlaid scores and loadings plot
##' @param x a pcaRes object
##' @param choices which two pcs to plot 
##' @param scale The variables are scaled by
##' \eqn{\lambda^{scale}}{lambda^scale} and the observations are
##' scaled    by \eqn{\lambda^{scale}}{lambda ^ (1-scale)} where
##' \code{lambda} are  the singular values as computed by
##' \code{princomp}.  Normally  \eqn{0\le{}scale\le{}1}{0 <= scale <=
##' 1}, and a warning will be issued if the specified 'scale' is
##' outside this range.
##' @param pc.biplot If true, use what Gabriel (1971) refers to as a
##' "principal component biplot", with \eqn{\lambda=1}{lambda = 1} and
##' observations scaled up by sqrt(n) and variables scaled down by
##' sqrt(n). Then the inner products between variables approximate
##' covariances and distances between observations approximate
##' Mahalanobis distance. 
##' @param ... optional arguments to be passed to
##' \code{biplot.default}. 
##' @return a plot is produced on the current graphics device.
##' @method biplot pcaRes
##' @export
##' @examples
##' data(iris)
##' pcIr <- pca(iris[,1:4])
##' biplot(pcIr)
##' @seealso \code{prcomp}, \code{pca}, \code{princomp}
##' @author Kevin Wright, Adapted from \code{biplot.prcomp}
##' @keywords multivariate
##' @name biplot-methods
biplot.pcaRes <- function(x, choices=1:2, scale=1, pc.biplot=FALSE, ...) {
  if(length(choices)!=2)
    stop("length of choices must be 2")
  scores <- scores(x)
  n <- nrow(scores)
  lam <- sDev(x)[choices] * sqrt(n)
  if(scale < 0 || scale > 1)
    warning("'scale' is outside [0,1]")
  if(scale != 0) lam <- lam^scale
  else lam <- 1
  if(pc.biplot) lam <- lam/sqrt(n)
  biplot(t(t(scores[,choices])/lam),
         t(t(loadings(x)[, choices]) * lam), , ...)
  invisible()
}
##' @aliases biplot,pcaRes-method
## @importFrom stats biplot
##' @name biplot-methods
setMethod("biplot", "pcaRes", biplot.pcaRes)

##' Flexible calculation of R2 goodness of fit.
##' @title R2 goodness of fit
##' @param object a PCA model object
##' @param direction choose between calculating R2 per variable, per
##' observation or for the entire data with 'variables',
##' 'observations' or 'complete'.
##' @param data the data used to fit the model
##' @param pcs the number of PCs to use to calculate R2
##' @aliases R2VX R2VX,pcaRes-method
##' @examples
##' R2VX(pca(iris))
##' @return A vector with R2 values
##' @author Henning Redestig
setMethod('R2VX', 'pcaRes',
          function(object,
                   direction=c('variables', 'observations', 'complete'),
                   data=completeObs(object), pcs=nP(object)) {
  direction <- match.arg(direction)
  if(is.null(data)) 
    stop('missing input when calculating R2')
  if(any(is.na(data))) 
    stop('missing values not allowed for calculating R2')
  dat <- prep(data, scale=scl(object), center=center(object))
  xhat <- resid(object, pcs=pcs, data=dat, pre=FALSE, post=FALSE)
  switch(direction, variables={
    1 - colSums(xhat^2) / colSums(dat^2)
  }, observations={
    1 - rowSums(xhat^2) / rowSums(dat^2)
  }, complete={
    1 - sum(xhat^2) / sum(dat^2)
  })
})

setAs('pcaRes', 'data.frame', function(from) {
  tt <- scores(from)
  pp <- loadings(from)
  if(is.null(rownames(tt)))
    rownames(tt) <- 1:nrow(tt)
  if(is.null(rownames(pp)))
    rownames(pp) <- 1:nrow(pp)
  dfs <- as.data.frame(tt)
  dfs$names <- rownames(tt)
  dfs$type <- 'scores'
  dfl <- as.data.frame(pp)
  dfl$names <- rownames(pp)
  dfl$type <- 'loadings'
  rownames(dfl) <- rownames(dfs) <- NULL
  rbind(dfl, dfs)
})

##' Print a brief description of the PCA model
##' @title Summary of PCA model
##' @param object a pcaRes object
##' @param ... Not used
##' @return Nothing, used for side-effect
##' @aliases summary summary.pcaRes summary,pcaRes-method
##' @author Henning Redestig
##' @export
##' @name summary
##' @method summary pcaRes
summary.pcaRes <- function(object, ...){
  cat(method(object), "calculated PCA\n")
  cat("Importance of component(s):\n")
  prop <- vector(length=length(R2cum(object)), mode="numeric")
  prop[1] <- R2cum(object)[1]
  if (length(R2cum(object)) > 1) {
    for (i in 2:length(prop)) {
      prop[i] <- R2cum(object)[i] - R2cum(object)[i-1]
    }
  }
  r <- rbind(prop, R2cum(object))
  rownames(r) <- c("R2", "Cumulative R2")
  colnames(r) <- paste("PC", 1:nP(object), sep="")
  print(r, digits=4)
  invisible(r)
}
setMethod("summary", "pcaRes", summary.pcaRes)

##' Predict data using PCA model
##'
##' This function extracts the predict values from a pcaRes object for
##' the PCA methods SVD, Nipals, PPCA and BPCA.  Newdata is first
##' centered if the PCA model was and then scores (\eqn{T}) and data
##' (\eqn{X}) is 'predicted' according to :
##' \eqn{\hat{T}=X_{new}P}{That=XnewP}
##' \eqn{\hat{X}_{new}=\hat{T}P'}{Xhat=ThatP'}.  Missing values are
##' set to zero before matrix multiplication to achieve NIPALS like
##' treatment of missing values. 
##' @title Predict values from PCA.
##' @param object \code{pcaRes} the \code{pcaRes} object of interest.
##' @param newdata \code{matrix} new data with same number of columns
##' as the used to compute \code{object}.
##' @param pcs \code{numeric} The number of PC's to consider
##' @param pre pre-process \code{newdata} based on the pre-processing
##' chosen for the PCA model
##' @param post unpre-process the final data (add the center back etc)
##' @param ... Not passed on anywhere, included for S3 consistency.
##' @return A list with the following components: \item{scores}{The
##' predicted scores} \item{x}{The predicted data}
##' @method predict pcaRes 
##' @keywords multivariate
##' @examples
##' data(iris)
##' hidden <- sample(nrow(iris), 50)
##' pcIr <- pca(iris[-hidden,1:4])
##' pcFull <- pca(iris[,1:4])
##' irisHat <- predict(pcIr, iris[hidden,1:4])
##' cor(irisHat$scores[,1], scores(pcFull)[hidden,1])
##' @export
##' @name predict-methods
##' @author Henning Redestig
predict.pcaRes <- function(object, newdata, pcs=nP(object),
                           pre=TRUE, post=TRUE, ...) {
  if(!method(object) %in% listPcaMethods("linear"))
    stop("predict method not implemented for that type of PCA")
  if(pre)
    newdata <- prep(newdata, scl(object), center(object))
  ## set na's to zero to achieve NIPALS like prediction
  newdata[is.na(newdata)] <- 0
  tnew <- newdata %*% loadings(object)[,1:pcs,drop=FALSE]
  xhat <- tcrossprod(tnew,  loadings(object)[,1:pcs,drop=FALSE])
  if(post)
    xhat <- prep(xhat, scl(object), center(object), reverse=TRUE)
  list(scores=tnew, x=xhat)
}
## @importFrom stats predict
##' @name predict-methods
##' @aliases predict,pcaRes-method
setMethod("predict", "pcaRes", predict.pcaRes)

##' This function extracts the residuals values from a pcaRes object
##' for the PCA methods SVD, Nipals, PPCA and BPCA
##' @title Residuals values from a PCA model.
##' @param object \code{pcaRes} the \code{pcaRes} object of interest.
##' @param data \code{matrix} The data that was used to calculate the
##' PCA model (or a different dataset to e.g. adress its proximity to
##' the model). 
##' @param ... Passed on to \code{\link{predict.pcaRes}}. E.g. setting
##' the number of used components.
##' @return A \code{matrix} with the residuals
##' @method residuals pcaRes
##' @keywords multivariate
##' @export
##' @name rediduals-methods
##' @examples
##' data(iris)
##' pcIr <- pca(iris[,1:4])
##' head(residuals(pcIr, iris[,1:4]))
##' @author Henning Redestig
residuals.pcaRes <- function(object, data=completeObs(object), ...) {
  if(is.null(data))
    stop("data missing when calculating residuals")
  data - predict(object, data, ...)$x
}
##' @aliases residuals,pcaRes-method
##' @name rediduals-methods
setMethod("residuals", "pcaRes", residuals.pcaRes)
##' @name rediduals-methods
##' @aliases resid,pcaRes-method
setMethod("resid", "pcaRes", residuals.pcaRes)

##' Fitted values of a PCA model
##' 
##' This function extracts the fitted values from a pcaResobject. For
##' PCA methods like SVD, Nipals, PPCA etc this is basically just the
##' scores multipled by the loadings and adjusted for pre-processing.
##' for non-linear PCA the original data is propagated through the
##' network to obtain the approximated data.
##' @title Extract fitted values from PCA.
##' @param object the \code{pcaRes} object of interest.
##' @param data For standard PCA methods this can safely be left null
##' to get scores x loadings but if set, then the scores are obtained
##' by projecting provided data onto the loadings.  If data contains
##' missing values the result will be all NA. Non-linear PCA is an
##' exception, here if data is NULL then data is set to the
##' completeObs and propaged through the network.
##' @param nPcs The number of PC's to consider
##' @param pre pre-process \code{data} based on the pre-processing
##' chosen for the PCA model
##' @param post unpre-process the final data (add the center back etc
##' to get the final estimate)
##' @param ... Not used
##' @return A matrix representing the fitted data
##' @keywords multivariate
##' @method fitted pcaRes
##' @examples
##' pc <- pca(iris[,1:4], nPcs=4, center=TRUE, scale="uv")
##' sum( (fitted(pc) - iris[,1:4])^2 )
##' @export
##' @name fitted-methods
##' @author Henning Redestig
fitted.pcaRes <- function(object, data=NULL, nPcs=nP(object),
                          pre=TRUE, post=TRUE, ...) {
  if(method(object) %in% listPcaMethods("nonlinear")) {
    if(is.null(data) & is.null(completeObs(object)))
      stop("completeObs slot is empty -- provide the training data")
    if(is.null(data) & !is.null(completeObs(object)))
      data <- completeObs(object)
    if(is.null(data))
      stop("nlpca requires original data to be provide")
    if(pre)
      data <- prep(data, scl(object), center(object))
    recData <-
      errorHierarchic(object@network, t(scores(object)), t(data))$out[,,nPcs]
    recData <- t(recData / object@network@scalingFactor)
  }
  else  {
    if(!is.null(data)) {
      if(pre)
        data <- prep(data, scl(object), center(object))
      tt <- data %*% loadings(object)[,1:nPcs, drop=FALSE]
    }
    if(is.null(data))
      tt <- scores(object)[,1:nPcs, drop=FALSE]
    recData <- tcrossprod(tt, loadings(object)[,1:nPcs, drop=FALSE])
  }
  if(post)
    recData <- prep(recData, scl(object), center(object), reverse=TRUE)
  return(recData)
}
## @importFrom stats fitted
##' @name fitted-methods
##' @aliases fitted,pcaRes-method
setMethod("fitted", "pcaRes", fitted.pcaRes)

##' Plot the computed diagnostics of PCA model to get an idea of their
##' importance. Note though that the standard screeplot shows the
##' standard deviations for the PCs this method shows the R2 values
##' which empirically shows the importance of the P's and is thus
##' applicable for any PCA method rather than just SVD based PCA.
##'
##' If cross-validation was done for the PCA the plot will also show
##' the CV based statistics.  A common rule-of-thumb for determining
##' the optimal number of PCs is the PC where the CV diagnostic is at
##' its maximum but not very far from \eqn{R^2}. 
##' @title Plot diagnostics (screeplot) 
##' @param x \code{pcaRes} The pcaRes object.
##' @param y not used
##' @param main title of the plot
##' @param col Colors of the bars
##' @param ... further arguments to barplot
##' @return None, used for side effect.
##' @seealso \link{screeplot}
##' @examples
##' data(metaboliteData)
##' pc <- pca(t(metaboliteData), nPcs=5, cv="q2", scale="uv")
##' plot(pc)
##' @method plot pcaRes
##' @aliases plot.pcaRes plot,pcaRes-method
##' @export
##' @author Henning Redestig
plot.pcaRes <- function(x, y=NULL, main=deparse(substitute(object)),
                        col=gray(c(0.9, 0.5)), ...) {
  y <- NULL
  ## the deparse(subsitute(object)) later fails otherwise
  main <- main
  if(!is.null(cvstat(x))) {
    cvs <- cvstat(x)
    if(length(cvs) != nP(x))
      cvs <- c(cvs, rep(NA, nP(x) - length(cvs)))
    xx <- rbind(R2cum(x), cvs)
    barplot(xx, beside=TRUE, ylim=c(0,1.1), col=col, main=main,
            names.arg=paste("PC", 1:nP(x), sep=""), ...)
    legend(x="topleft", fill=col,
           legend=c(expression(R^2), expression(Q^2)))
  } else 
    barplot(R2cum(x), ylim=c(0,1.1), ylab=expression(R^2), main=main,
            names.arg=paste("PC", 1:nP(x), sep=""), col=col[1], ...)
}
setMethod("plot", "pcaRes", plot.pcaRes)

##' A common way of visualizing two principal components
##'
##' This method is meant to be used as a quick way to visualize
##' results, if you want a more specific plot you probably want to
##' get the scores, loadings with \code{scores(object)},
##' \code{loadings(object)} and then design your own plotting method.
##' @title Side by side scores and loadings plot
##' @usage slplot(object, pcs=c(1,2), scoresLoadings=c(TRUE, TRUE),
##' sl="def", ll="def", hotelling=0.95, rug=TRUE, sub=NULL,...)
##' @param object a pcaRes object
##' @param pcs which two pcs to plot
##' @param scoresLoadings Which should be shown scores and or loadings
##' @param sl labels to plot in the scores plot
##' @param ll labels to plot in the loadings plot
##' @param hotelling confidence interval for ellipse in the score plot
##' @param rug logical, rug x axis in score plot or not
##' @param sub Subtitle, defaults to annotate with amount of explained
##' variance.
##' @param ... Further arguments to plot functions. Prefix arguments
##' to \code{par()} with 's' for the scores plot and 'l' for the
##' loadings plot. I.e. cex become scex for setting character
##' expansion in the score plot and lcex for the loadings plot.
##' @return None, used for side effect.
##' @note Uses layout instead of par to provide side-by-side so it
##' works with Sweave (but can not be combined with
##' \code{par(mfrow=..))}
##' @author Henning Redestig
##' @seealso \code{\link{pca}}, \code{\link{biplot}}
##' @aliases slplot slplot,pcaRes-method
##' @examples
##' data(iris)
##' pcIr <- pca(iris[,1:4], scale="uv")
##' slplot(pcIr, sl=NULL, spch=5)
##' slplot(pcIr, sl=NULL, lcex=1.3, scol=as.integer(iris[,5]))
##' @keywords multivariate
setMethod("slplot", "pcaRes",
function(object, pcs=c(1,2), scoresLoadings=c(TRUE, TRUE),
         sl=rownames(scores(object)),
         ll=rownames(loadings(object)), hotelling=0.95, rug=FALSE,
         sub=NULL,...) {

  opar <- par(no.readonly=TRUE)

  cl <- match.call()
  mainArgs <- c(1,match(c("ll", "sl", "scoresLoadings", "sub"),
                        names(cl), 0))
  scoreArgs <- grep("^s", names(cl)[-mainArgs])
  loadingArgs <- grep("^l", names(cl)[-mainArgs])

  if(!is.null(ll) & length(ll) != nVar(object))
    stop("Loading labels do not match the object dimensions")
  if(!is.null(sl) & length(sl) != nObs(object))
    stop("Score labels do not match the object dimensions")
  if(is.null(sl))
    sl <- NA
  if(is.null(ll))
    ll <- NA
  
  ## no loadings for non-linear pca
  if(method(object) %in% listPcaMethods("nonlinear") && scoresLoadings[2])
    scoresLoadings[2] <- FALSE
  
  if(length(pcs) > 2)
    plotPcs(object, pcs, scoresLoadings=scoresLoadings,...)
  else {
    if(is.null(sub))
      sub <- paste(sprintf("%.2f", R2cum(object)[max(pcs)]
                           * 100),
                   "% of the variance explained", sep="")

    if(sum(scoresLoadings) == 2)
      layout(matrix(c(1,2), 1, 2, TRUE), respect=matrix(c(1,1), 1, 2))
    ## exception plot if one dimensional
    if (length(pcs) == 1 | nP(object) == 1) {
      pcs <- 1
      
      ## score plot
      if(scoresLoadings[1]) {
        newCall <- call("barplot",
                        height=scores(object)[,pcs],
                        main="Scores", las=3, ylab=paste("PC", pcs), sub=sub,
                        names.arg=sl)
        tmp <- cl[-mainArgs][scoreArgs]
        names(tmp) <- gsub("^s", "", names(tmp))
        for(i in 1:length(tmp)) {
          newCall[[length(newCall) + 1]] <- tmp[[i]]
          names(newCall)[length(newCall)] <- names(tmp)[i]
        }
        eval(newCall)
      }

      ## loadingplot
      if(scoresLoadings[2]) {
        newCall <- call("barplot",
                        height=loadings(object)[,pcs],
                        main="Loadings", las=3, ylab=paste("PC", pcs), 
                        names.arg=ll)
        if(length(loadingArgs) > 0) {
          tmp <- cl[-mainArgs][loadingArgs]
          names(tmp) <- gsub("^l", "", names(tmp))
          for(i in 1:length(tmp)) {
            newCall[[length(newCall) + 1]] <- tmp[[i]]
            names(newCall)[length(newCall)] <- names(tmp)[i]
          }
        }
        eval(newCall)
      }
      return(invisible(TRUE))
    }
    
    ## the score plot
    if(scoresLoadings[1]) {
      ## setup plot
      plotCall <- call("plot",
                       x=scores(object)[,pcs],
                       main="Scores", ylab=paste("PC", pcs[2]), 
                       sub=sub, xlab=paste("PC", pcs[1]))
      if(length(scoreArgs) > 0) {
        tmp <- cl[-mainArgs][scoreArgs]
        names(tmp) <- gsub("^s", "", names(tmp))
        for(i in 1:length(tmp)) {
          plotCall[[length(plotCall) + 1]] <- tmp[[i]]
          names(plotCall)[length(plotCall)] <- names(tmp)[i]
        }
      }
      ## add text
      if (!is.null(sl) & !all(is.na(sl))) {
        plotCall[[length(plotCall) + 1]] <- "n"
        names(plotCall)[length(plotCall)] <- "type"
        textCall <- call("text",
                         x=scores(object)[,pcs], labels=sl)
        if(length(scoreArgs) > 0) {
          tmp <- cl[-mainArgs][scoreArgs]
          names(tmp) <- gsub("^s", "", names(tmp))
          for(i in 1:length(tmp)) {
            textCall[[length(textCall) + 1]] <- tmp[[i]]
            names(textCall)[length(textCall)] <- names(tmp)[i]
          }
        }
      }
      eval(plotCall)
      if (!is.null(sl) & !all(is.na(sl)))
        eval(textCall)
      if(rug)
        rug(scores(object)[,1])
      abline(h=0, v=0)
      if(!is.null(hotelling)) {
        A <- length(pcs)
        el <- simpleEllipse(scores(object)[,pcs[1]],
                            scores(object)[,pcs[2]], alfa=hotelling)
        lines(el)
      }
    }

    ## the loading plot
    if(scoresLoadings[2]) {
      ## setup plot
      plotCall <- call("plot",
                       x=loadings(object)[,pcs],
                       main="Loadings", ylab=paste("PC", pcs[2]), 
                       xlab=paste("PC", pcs[1]))
      if(length(loadingArgs) > 0) {
        tmp <- cl[-mainArgs][loadingArgs]
        names(tmp) <- gsub("^l", "", names(tmp))
        for(i in 1:length(tmp)) {
          plotCall[[length(plotCall) + 1]] <- tmp[[i]]
          names(plotCall)[length(plotCall)] <- names(tmp)[i]
        }
      }
      ## add text
      if (!is.null(ll) & !all(is.na(ll))) {
        plotCall[[length(plotCall) + 1]] <- "n"
        names(plotCall)[length(plotCall)] <- "type"
        textCall <- call("text",
                         x=loadings(object)[,pcs], labels=ll)
        if(length(loadingArgs) > 0) {
          tmp <- cl[-mainArgs][loadingArgs]
          names(tmp) <- gsub("^l", "", names(tmp))
          for(i in 1:length(tmp)) {
            textCall[[length(textCall) + 1]] <- tmp[[i]]
            names(textCall)[length(textCall)] <- names(tmp)[i]
          }
        }
      }
      eval(plotCall)
      if (!is.null(ll) & !all(is.na(ll)))
        eval(textCall)
      abline(h=0, v=0)
    }
  }
  par(opar)
})

##' Perform cross validation to estimate the optimal number of
##' components for missing value estimation. Cross validation is
##' done for the complete subset of a variable.
##'
##' The assumption hereby is that variables that are highly correlated
##' in a distinct region (here the non-missing observations) are also
##' correlated in another (here the missing observations).  This also
##' implies that the complete subset must be large enough to be
##' representative.  For each incomplete variable, the available
##' values are divided into a user defined number of cv-segments. The
##' segments have equal size, but are chosen from a random equal
##' distribution. The non-missing values of the variable are covered
##' completely.  PPCA, BPCA, SVDimpute, Nipals PCA, llsImpute an NLPCA
##' may be used for imputation.
##'
##' The whole cross validation is repeated several times so, depending
##' on the parameters, the calculations can take very long time.  As
##' error measure the NRMSEP (see Feten et. al, 2005) or the Q2
##' distance is used.  The NRMSEP basically normalises the RMSD
##' between original data and estimate by the variable-wise
##' variance. The reason for this is that a higher variance will
##' generally lead to a higher estimation error.  If the number of
##' samples is small, the variable - wise variance may become an
##' unstable criterion and the Q2 distance should be used
##' instead. Also if variance normalisation was applied previously.
##' 
##' The method proceeds variable - wise, the NRMSEP / Q2 distance is
##' calculated for each incomplete variable and averaged
##' afterwards. This allows to easily see for wich set of variables
##' missing value imputation makes senes and for wich set no
##' imputation or something like mean-imputation should be used.  Use
##' \code{kEstimateFast} or \code{Q2} if you are not interested in
##' variable wise CV performance estimates.
##' 
##' Run time may be very high on large data sets. Especially when used
##' with complex methods like BPCA or Nipals PCA.  For PPCA, BPCA,
##' Nipals PCA and NLPCA the estimation method is called
##' \eqn{(v_{miss} \cdot segs \cdot nruncv \cdot)}{(v\_miss * segs *
##' nruncv)} times as the error for all numbers of principal
##' components can be calculated at once.  For LLSimpute and SVDimpute
##' this is not possible, and the method is called \eqn{(v_{miss}
##' \cdot segs \cdot nruncv \cdot length(evalPcs))}{(v\_miss * segs *
##' nruncv * length(evalPcs))} times. This should still be fast for
##' LLSimpute because the method allows to choose to only do the
##' estimation for one particular variable.  This saves a lot of
##' iterations.  Here, \eqn{v_{miss}}{v\_miss} is the number of
##' variables showing missing values.
##'
##' As cross validation is done variable-wise, in this function Q2 is
##' defined on single variables, not on the entire data set. This is
##' Q2 is calculated as as \eqn{\frac{\sum(x -
##' xe)^2}{\sum(x^2)}}{sum(x - xe)^2 \ sum(x^2)}, where x is the
##' currently used variable and xe it's estimate. The values are then
##' averaged over all variables. The NRMSEP is already defined
##' variable-wise. For a single variable it is then
##' \eqn{\sqrt(\frac{\sum(x - xe)^2}{(n \cdot var(x))})}{sqrt(sum(x -
##' xe)^2 \ (n * var(x)))}, where x is the variable and xe it's
##' estimate, n is the length of x.  The variable wise estimation
##' errors are returned in parameter variableWiseError.
##' @title Estimate best number of Components for missing value
##' estimation
##' @param Matrix \code{matrix} -- numeric matrix containing
##' observations in rows and variables in columns
##' @param method \code{character} -- of the methods found with
##' pcaMethods() The option llsImputeAll calls llsImpute with the
##' allVariables = TRUE parameter.
##' @param evalPcs \code{numeric} -- The principal components to use
##' for cross validation or the number of neighbour variables if used
##' with llsImpute.  Should be an array containing integer values,
##' eg. \code{evalPcs = 1:10} or \code{evalPcs = c(2,5,8)}. The NRMSEP
##' or Q2 is calculated for each component.
##' @param segs \code{numeric} -- number of segments for cross validation
##' @param nruncv \code{numeric} -- Times the whole cross validation
##' is repeated
##' @param em \code{character} -- The error measure. This can be nrmsep or q2
##' @param allVariables \code{boolean} -- If TRUE, the NRMSEP is
##' calculated for all variables, If FALSE, only the incomplete ones
##' are included. You maybe want to do this to compare several methods
##' on a  complete data set.
##' @param verbose \code{boolean} -- If TRUE, some output like the
##' variable indexes are printed to the console each iteration.
##' @param ...  Further arguments to \code{pca} or \code{nni}
##' @return   A list with:
##' \item{bestNPcs}{number of PCs or k for which the minimal average
##' NRMSEP or the maximal Q2 was obtained.}
##' \item{eError}{an array of of size length(evalPcs). Contains the
##' average error of the cross validation runs for each number of
##' components.}
##' \item{variableWiseError}{Matrix of size
##' \code{incomplete_variables} x length(evalPcs).  Contains the
##' NRMSEP or Q2 distance for each variable and each number of PCs.
##' This allows to easily see for wich variables imputation makes
##' sense and for which one it should not be done or mean imputation
##' should be used.}
##' \item{evalPcs}{The evaluated numbers of components or number of
##' neighbours  (the same as the evalPcs input parameter).}
##' \item{variableIx}{Index of the incomplete variables. This can be
##' used to map  the variable wise error to the original data.}
##' @seealso \code{\link{kEstimateFast}, \link{Q2}, \link{pca}, \link{nni}}.
##' @examples
##' ## Load a sample metabolite dataset with 5\% missing values (metaboliteData)
##' data(metaboliteData)
##' # Do cross validation with ppca for component 2:4
##' esti <- kEstimate(metaboliteData, method = "ppca", evalPcs = 2:4, nruncv=1, em="nrmsep")
##' # Plot the average NRMSEP
##' barplot(drop(esti$eError), xlab = "Components",ylab = "NRMSEP (1 iterations)")
##' # The best result was obtained for this number of PCs:
##' print(esti$bestNPcs)
##' # Now have a look at the variable wise estimation error
##' barplot(drop(esti$variableWiseError[, which(esti$evalPcs == esti$bestNPcs)]), 
##'         xlab = "Incomplete variable Index", ylab = "NRMSEP")
##' @keywords multivariate
##' @export 
##' @author Wolfram Stacklies
kEstimate <- function(Matrix, method="ppca", evalPcs=1:3, segs=3, nruncv=5,
                      em="q2", allVariables=FALSE,
                      verbose=interactive(), ...) {

  fastKE <- FALSE
  if (method == "ppca" | method == "bpca" | method == "nipals" |
      method == "nlpca")
    fastKE <- TRUE

  method <- match.arg(method, listPcaMethods())
  em <- match.arg(em, c("nrmsep", "q2"))
  maxPcs <- max(evalPcs)
  lengthPcs <- length(evalPcs)

  ## If the data is a data frame, convert it into a matrix
  Matrix <- as.matrix(Matrix, rownames.force=TRUE)
  if(maxPcs > (ncol(Matrix) - 1))
    stop("maxPcs exceeds matrix size, choose a lower value!")

  ## And now check if everything is right...
  if( !checkData(Matrix, verbose=interactive()) )
    stop("Invalid data format! Use checkData(Matrix, verbose = TRUE) for details.\n")

  if( (sum(is.na(Matrix)) == 0) && (allVariables == FALSE) )
    stop("No missing values. Maybe you want to set allVariables = TRUE. Exiting\n")


  missing <- apply(is.na(Matrix), 2, sum) > 0
  missIx     <- which(missing == TRUE)
  if (allVariables)
    missIx <- 1:ncol(Matrix)

  complete <- !missing
  compIx    <- which(complete == TRUE)

  error <- matrix(0, length(missIx), length(evalPcs))
  iteration <- 0
  for(nPcs in evalPcs) {
    ## If the estimated observations are just scores %*% t(loadings)
    ## we can calculate all we need at once, this saves many
    ## iterations...
    if (fastKE) nPcs = maxPcs

    iteration = iteration + 1
    if (verbose && !fastKE) { cat("Doing CV for ", nPcs, " component(s) \n") }
    else if (verbose && fastKE) {cat("Doing CV ... \n")}
    for(cviter in 1:nruncv) {
      pos <- 0
      if (verbose) cat("Incomplete variable index: ")
      for (index in missIx) {
        pos <- pos + 1
        cat(pos, ":", sep="")
        target <- Matrix[, index, drop = FALSE]
        compObs <- !is.na(target)
        missObs <- is.na(target)
        nObs <- sum(compObs)

        ## Remove all observations that are missing in the target genes,
        ## as additional missing values may tamper the results
        set <- Matrix[compObs,]

        if (nObs >= (2 * segs)) {
          segments <- segs
        } else
        segments <- ceiling(nObs / 2)

        ## We assume uniformly distributed missing values when
        ## choosing the segments
        tt <- gl(segments, ceiling(nObs / segments))[1:nObs]
        cvsegs <- split(sample(nObs), tt)
        set <- Matrix[compObs,]
        if (fastKE) {
          nrmsep <- array(0, length(evalPcs))
          q2 <- array(0, length(evalPcs))
        } else {
          nrmsep <- 0; q2 <- 0
        }
        
        for (i in 1:length(cvsegs)) {
          n <- length(cvsegs[[i]]) # n is the number of created
                                   # missing values
          ## Impute values using the given regression method
          testSet <- set
          testSet[cvsegs[[i]], index] <- NA
          if (method == "llsImpute") {
            estimate <- llsImpute(testSet, k = nPcs, verbose = FALSE,
                                  allVariables = FALSE, 
                                  center = FALSE, xval = index)
          } else if (method == "llsImputeAll") {
            estimate <- llsImpute(testSet, k = nPcs, verbose = FALSE,
                                  allVariables = TRUE, 
                                  center = FALSE, xval = index)
          } else {
            estimate <- pca(testSet, nPcs = nPcs, verbose = FALSE,
                            method = method, center = TRUE,...)
          }

          if (fastKE) {
            for (np in evalPcs) {
              estiFitted <- fitted(estimate, data = NULL, nPcs = np)
              estimateVec <- estiFitted[, index]
              original <- target[compObs, ]
              estimateVec[-cvsegs[[i]]] <- testSet[-cvsegs[[i]], index]
              ## Error of prediction, error is calculated for removed
              ## elements only
              nIx <- which(evalPcs == np) 
              if (em == "nrmsep") {
                nrmsep[nIx] <- nrmsep[nIx] + sum( (original - estimateVec)^2) 
              } else {
                q2[nIx] <- q2[nIx] + sum( (original - estimateVec)^2 )
              }
            }    
          } else {
            estimate <- estimate@completeObs[, index]
            original <- target[compObs, ]
            ## Error of prediction, error is calculated for removed
            ## elements only
            if (em == "nrmsep") {
              nrmsep <- nrmsep + sum( (original - estimate)^2)
            } else {
              q2 <- q2 + sum( (original - estimate)^2 )
            }
          }
        } ## iteration over cv segments
        
        if (fastKE) {
          if (em == "nrmsep") {
            error[pos, ] <-
              error[pos, ] + nrmsep / (nrow(set) * var(set[,index]))
          } else
          error[pos, ] <- error[pos, ] + (1 - (q2 / sum(set[, index]^2)))
        } else {
          if (em == "nrmsep") {
            error[pos, iteration] <- error[pos, iteration] + 
              nrmsep / (nrow(set) * var(set[,index]))
          } else
          error[pos, iteration] <-
            error[pos, iteration] + (1 - (q2 / sum(set[, index]^2)))
        }
      } # iteration over variables
      if (verbose) cat("\n")
      
    } #iteration over nruncv
    
    ## The error is the sum over the independent cross validation runs
    error <- error / nruncv
    
    if (verbose && !fastKE)
      cat("The average", em, "for k =", iteration, "is", 
          sum(error[,iteration]) / nrow(error), "\n")

    ## if nlpca, ppca, bpca, nipals we do not need to iterate over the
    ## number of components...
    if (fastKE) break
  } # iteration over number components

  if (em == "nrmsep")
    avgError <- sqrt(apply(error, 2, sum) / nrow(error))
  else
    avgError <- apply(error, 2, sum) / nrow(error)

  ret <- list()
  if (em == "nrmsep")
    ret$bestNPcs <- evalPcs[which(avgError == min(avgError))]
  else ret$bestNPcs <- evalPcs[which(avgError == max(avgError))]
  ret$eError <- avgError
  if(em == "nrmsep") ret$variableWiseError <- sqrt(error)
  else ret$variableWiseError <- error
  ret$evalPcs <- evalPcs
  ret$variableIx <- missIx

  return(ret)
}

##' The function contains the actual implementation of the BPCA
##' component  estimation. It performs one step of the BPCA EM
##' algorithm.  It is called 'maxStep' times from within the main loop
##' in BPCAestimate.
##'
##' This function is NOT intended to be run standalone.
##' @title Do BPCA estimation step
##' @param M Data structure containing all needed information. See the
##' source documentation of BPCA_initmodel for details
##' @param y Numeric original data matrix
##' @return Updated version of the data structure
##' @author Wolfram Stacklies
BPCA_dostep <- function(M,y) {

  ## Empty matrix in which the scores are copied
  M$scores <- matrix(NA, M$rows, M$comps)

  ## Expectation step for data without missing values
  Rx <- diag(M$comps) + M$tau * t(M$PA) %*% M$PA + M$SigW
  Rxinv <- solve(Rx)
  idx <- M$row_nomiss

  if (length(idx) == 0) {
    trS <- 0
    T <- 0
  } else {
    dy <- y[idx,, drop=FALSE] - repmat(M$mean, length(idx), 1)
    x <- M$tau * Rxinv %*% t(M$PA) %*% t(dy)
    T <- t(dy) %*% t(x)
    trS <- sum(sum(dy * dy))

    ## Assign the scores for complete rows
    xTranspose <- t(x)
    for (i in 1:length(idx)) {
      M$scores[idx[i],] <- xTranspose[i,]
    }
  }
  ## Expectation step for incomplete data
  if( length(M$row_miss) > 0) {
    for(n in 1:length(M$row_miss)) {
      i  <- M$row_miss[n]
      dyo <- y[ i, !M$nans[i,], drop=FALSE] - M$mean[ !M$nans[i,], drop=FALSE]
      Wm <- M$PA[ M$nans[i,],, drop=FALSE]
      Wo <- M$PA[ !M$nans[i,],, drop=FALSE]
      Rxinv <- solve( (Rx - M$tau * t(Wm) %*% Wm))
      ex  <- M$tau * t(Wo) %*% t(dyo)
      x <- Rxinv %*% ex
      dym <- Wm %*% x
      dy <- y[i,, drop=FALSE]
      dy[ !M$nans[i,] ] <- t(dyo)
      dy[ M$nans[i,] ] <- t(dym)
      M$yest[i,] <- dy + M$mean
      T <- T + t(dy) %*% t(x)
      T[ M$nans[i,], ] <- T[ M$nans[i,],, drop=FALSE] + Wm %*% Rxinv
      trS <- trS + dy %*% t(dy) + sum(M$nans[i,]) / M$tau + 
        sum( diag(Wm %*% Rxinv %*% t(Wm)) ) 
      trS <- trS[1,1]
      ## Assign the scores for rows containing missing values
      M$scores[M$row_miss[n],] <- t(x)
    }
  }
  T <- T / M$rows
  trS <- trS / M$rows

  ## Maximation step
  Rxinv <- solve(Rx)
  Dw <- Rxinv + M$tau * t(T) %*% M$PA %*% Rxinv + 
    diag(M$alpha, nrow = length(M$alpha)) / M$rows
  Dwinv <- solve(Dw)
  M$PA <- T %*% Dwinv ## The new estimate of the principal axes (loadings)
  M$tau <- (M$cols + 2 * M$gtau0 / M$rows) / (trS - sum(diag(t(T) %*% M$PA)) +
                                              (M$mean %*% t(M$mean) * M$gmu0 + 2 * M$gtau0 / M$btau0) / M$rows)
  M$tau <- M$tau[1,1] ## convert to scalar
  M$SigW <- Dwinv * (M$cols / M$rows)
  M$alpha <- (2 * M$galpha0 + M$cols) / (M$tau * diag(t(M$PA) %*% M$PA) + 
                                         diag(M$SigW) + 2 * M$galpha0 / M$balpha0)

  return(M)
}

setGeneric("vector2matrices", function(object, ...)
           standardGeneric("vector2matrices"))
##' @exportMethod leverage
setGeneric("leverage", function(object, ...) standardGeneric("leverage"))
##' @exportMethod DModX
setGeneric("DModX", function(object, dat, newdata=FALSE, type=c("normalized","absolute"), ...)
    standardGeneric("DModX"))
##' @exportMethod nP
setGeneric("nP", function(object, ...) standardGeneric("nP"))
##' @exportMethod cvstat
setGeneric("cvstat", function(object, ...) standardGeneric("cvstat"))
##' @exportMethod nPcs
setGeneric("nPcs", function(object, ...) standardGeneric("nPcs"))
##' @exportMethod nObs 
setGeneric("nObs", function(object, ...) standardGeneric("nObs"))
##' @exportMethod nVar 
setGeneric("nVar", function(object, ...) standardGeneric("nVar"))
##' @exportMethod centered 
setGeneric("centered", function(object, ...) standardGeneric("centered"))
##' @exportMethod center
setGeneric("center", function(object, ...) standardGeneric("center"))
##' @exportMethod completeObs
setGeneric("completeObs", function(object, ...) standardGeneric("completeObs"))
##' @exportMethod method
setGeneric("method", function(object, ...) standardGeneric("method"))
##' @exportMethod nmissing
setGeneric("nmissing", function(object, ...) standardGeneric("nmissing"))
##' @exportMethod wasna
setGeneric("wasna", function(object, ...) standardGeneric("wasna"))
##' @exportMethod sDev
setGeneric("sDev", function(object, ...) standardGeneric("sDev"))
##' @exportMethod scaled
setGeneric("scaled", function(object, ...) standardGeneric("scaled"))
##' @exportMethod scl
setGeneric("scl", function(object, ...) standardGeneric("scl"))
##' @exportMethod R2cum 
setGeneric("R2cum", function(object, ...) standardGeneric("R2cum"))
##' @exportMethod slplot
setGeneric("slplot", function(object, pcs=c(1,2),
                              scoresLoadings=c(TRUE, TRUE),
                              sl="def", ll="def",
                              hotelling=0.95, rug=TRUE, sub=NULL,...)
           standardGeneric("slplot"))
##' @exportMethod scores
setGeneric("scores", function(object, ...) standardGeneric("scores"))
##' @exportMethod loadings
setGeneric("loadings", function(object, ...) standardGeneric("loadings"))
##' @exportMethod R2VX
setGeneric("R2VX", function(object, ...) standardGeneric("R2VX"))
## @exportMethod prep
#setGeneric("prep", function(object, ...) standardGeneric("prep"))

##' Print a brief description of nniRes model
##' @title Print a nniRes model
##' @param x An \code{nniRes} object
##' @param ... Not used
##' @return Nothing, used for side-effect
##' @export
##' @author Henning Redestig
showNniRes <- function(x, ...) {
  summary(x)
  cat(dim(x)["nVar"], "\tVariables\n")
  cat(dim(x)["nObs"],"\tSamples\n")
  cat(nmissing(x), "\tNAs (",
      round(100 * nmissing(x) / (nObs(x) * nVar(x)),
            getOption("str")$digits.d), "%)\n")
  cat("k was set to", x@k, "\n")
  if(centered(x))
    cat("Data was mean centered before running LLSimpute \n")
  else
    cat("Data was NOT mean centered before running LLSimpute \n")
  if(scaled(x))
    cat("Data was scaled before running LLSimpute \n")
  else
    cat("Data was NOT scaled before running LLSimpute \n")
}
setMethod("print", "nniRes", showNniRes)
setMethod("show", "nniRes", function(object) showNniRes(object))

##' Implements a Bayesian PCA missing value estimator.  The script
##' is a port of the Matlab version provided by Shigeyuki OBA.  See
##' also \url{http://ishiilab.jp/member/oba/tools/BPCAFill.html}.
##' BPCA combines an EM approach for PCA with a Bayesian model.  In
##' standard PCA data far from the training set but close to the
##' principal subspace may have the same reconstruction error.  BPCA
##' defines a likelihood function such that the likelihood for data
##' far from the training set is much lower, even if they are close to
##' the principal subspace.
##' 
##' Scores and loadings obtained with Bayesian PCA slightly differ
##' from those obtained with conventional PCA.  This is because BPCA
##' was developed especially for missing value estimation.  The
##' algorithm does not force orthogonality between factor loadings, as
##' a result factor loadings are not necessarily orthogonal.  However,
##' the BPCA authors found that including an orthogonality criterion
##' made the predictions worse.
##'
##' The authors also state that the difference between real and
##' predicted Eigenvalues becomes larger when the number of
##' observation is smaller, because it reflects the lack of
##' information to accurately determine true factor loadings from the
##' limited and noisy data.  As a result, weights of factors to
##' predict missing values are not the same as with conventional PCA,
##' but the missing value estimation is improved.
##' 
##' BPCA works iteratively, the complexity is growing with
##' \eqn{O(n^3)}{O(n^3)} because several matrix inversions are
##' required.  The size of the matrices to invert depends on the
##' number of components used for re-estimation.
##'
##' Finding the optimal number of components for estimation is not a
##' trivial task; the best choice depends on the internal structure of
##' the data.  A method called \code{kEstimate} is provided to
##' estimate the optimal number of components via cross validation.
##' In general few components are sufficient for reasonable estimation
##' accuracy. See also the package documentation for further
##' discussion about on what data PCA-based missing value estimation
##' makes sense.
##' 
##' It is not recommended to use this function directely but rather to
##' use the pca() wrapper function.
##'
##' There is a difference with respect the interpretation of rows
##' (observations) and columns (variables) compared to matlab
##' implementation. For estimation of missing values for microarray
##' data, the suggestion in the original bpca is to intepret genes as
##' observations and the samples as variables. In pcaMethods however,
##' genes are interpreted as variables and samples as observations
##' which arguably also is the more natural interpretation. For bpca
##' behavior like in the matlab implementation, simply transpose your
##' input matrix.
##' 
##' Details about the probabilistic model underlying BPCA are found in
##' Oba et. al 2003. The algorithm uses an expectation maximation
##' approach together with a Bayesian model to approximate the
##' principal axes (eigenvectors of the covariance matrix in PCA).
##' The estimation is done iteratively, the algorithm terminates if
##' either the maximum number of iterations was reached or if the
##' estimated increase in precision falls below \eqn{1e^{-4}}{1e^-4}.
##' 
##' \bold{Complexity:} The relatively high complexity of the method is
##' a result of several matrix inversions required in each step.
##' Considering the case that the maximum number of iteration steps is
##' needed, the approximate complexity is given by the term
##' \deqn{maxSteps \cdot row_{miss} \cdot O(n^3)}{maxSteps * row_miss
##' * O(n^3)} Where \eqn{row_{miss}}{row_miss} is the number of rows
##' containing missing values and \eqn{O(n^3)}{O(n^3)} is the
##' complexity for inverting a matrix of size
##' \eqn{components}{components}. Components is the number of
##' components used for re-estimation.
##' @title Bayesian PCA missing value estimation
##' @param Matrix \code{matrix} -- Pre-processed matrix (centered,
##'   scaled) with variables in columns and observations in rows. The
##'   data may contain missing values, denoted as \code{NA}.
##' @param nPcs \code{numeric} -- Number of components used for
##'   re-estimation. Choosing few components may decrease the
##'   estimation precision.
##' @param maxSteps \code{numeric} -- Maximum number of estimation
##'   steps.
##' @param verbose \code{boolean} -- BPCA prints the number of steps
##'   and the increase in precision if set to TRUE. Default is
##'   interactive().
##' @param threshold convergence threshold
##' @param ... Reserved for future use. Currently no further
##'   parameters are used
##' @return Standard PCA result object used by all PCA-based methods
##'   of this package. Contains scores, loadings, data mean and
##'   more. See \code{\link{pcaRes}} for details.
##' @references Shigeyuki Oba, Masa-aki Sato, Ichiro Takemasa, Morito
##'   Monden, Ken-ichi Matsubara and Shin Ishii.  A Bayesian missing
##'   value estimation method for gene expression profile
##'   data. \emph{Bioinformatics, 19(16):2088-2096, Nov 2003},
##'   \url{https://doi.org/10.1093/bioinformatics/btg287}.
##' @seealso \code{\link{ppca}}, \code{\link{svdImpute}},
##'   \code{\link{prcomp}}, \code{\link{nipalsPca}},
##'   \code{\link{pca}},
##'   \code{\link{pcaRes}}. \code{\link{kEstimate}}.
##' @note Requires \code{MASS}.
##' @examples
##' ## Load a sample metabolite dataset with 5\% missig values (metaboliteData)e
##' data(metaboliteData)
##' ## Perform Bayesian PCA with 2 components
##' pc <- pca(t(metaboliteData), method="bpca", nPcs=2)
##' ## Get the estimated principal axes (loadings)
##' loadings <- loadings(pc)
##' ## Get the estimated scores
##' scores <- scores(pc)
##' ## Get the estimated complete observations
##' cObs <- completeObs(pc)
##' ## Now make a scores and loadings plot
##' slplot(pc)
##' \dontshow{stopifnot(sum((fitted(pc) - t(metaboliteData))^2, na.rm=TRUE) < 200)}
##' @keywords multivariate
##' @export
##' @author Wolfram Stacklies
bpca <- function(Matrix, nPcs=2, maxSteps=100, 
                 verbose=interactive(), threshold=1e-4, ... ) {

  ## R implementation of a Bayesion PCA missing value estimator.
  ## After the Matlab script of Shigeyuki OBA (2002  May. 5th)
  ## See also: http://hawaii.aist-nara.ac.jp/%7Eshige-o/tools/
  ## Great thanks to them!
  M <- BPCA_initmodel(Matrix, nPcs)
  tauold <- 1000

  for( step in 1:maxSteps ) {
    M <- BPCA_dostep(M, Matrix)
    if( step %% 10 == 0 ) {
      tau <- M$tau
      dtau <- abs(log10(tau) - log10(tauold))
      if ( verbose ) {
        cat("Step Number           : ", step, '\n')
        cat("Increase in precision : ", dtau, '\n')
        cat("----------", '\n')
      }
      if (dtau < threshold) {
        break
      }
      tauold <- tau
    }
  }
  
  R2cum <- rep(NA, nPcs)
  TSS <- sum(Matrix^2, na.rm=TRUE)
  for (i in 1:nPcs) {
    difference <-
      Matrix - (M$scores[,1:i, drop=FALSE] %*% t(M$PA[,1:i, drop=FALSE]) )
    R2cum[i] <- 1 - (sum(difference^2, na.rm=TRUE) / TSS)
  }

  result <- new("pcaRes")
  result@scores <- M$scores 
  result@loadings <- M$PA
  result@R2cum <- R2cum
  result@method <- "bpca"
  return(result)
}

##' Conjugate gradient optimization
##' @param nlnet The nlnet
##' @param trainIn Training data
##' @param trainOut fitted data
##' @param verbose logical, print messages
##' @return ...
##' @author Henning Redestig, Matthias Scholz
optiAlgCgd <- function(nlnet,
                       trainIn,
                       trainOut,
                       verbose=FALSE) {
  
  tmpnet <- forkNlpcaNet(nlnet)
  derr <- tmpnet@gradient(tmpnet, trainIn, trainOut)
  dw <- derr$dwTotal
  e <- derr$Etotal

  dv <- -dw

  if(tmpnet@featureSorting)
    eSortLast <- e

  eHist <- rep(0, tmpnet@maxIter)
  ttLast <- rep(0.0001, 6)

  for(i in 1:tmpnet@maxIter) {
    if(verbose) {
      if(i %% 10 == 0)
        cat("*")
      if(i %% 100 == 0)
        cat(" [", i, "]\n")
    }

    eHist[i] <- e
    eLast <- e
    # line search in direction dv (downhill)
    ttGuess <- max(min(ttLast), 0.00001)
    
    linSe <- lineSearch(tmpnet, dv, e, ttGuess, trainIn, trainOut, verbose)
    tmpnet@weights$set(cbind(linSe$wBest))
    e <- linSe$eBest
    tt <- linSe$ttBest
    
    ttLast <- c(ttLast[2:length(ttLast)], tt) #shift and add new tt

    gradRes <- tmpnet@gradient(tmpnet, trainIn, trainOut)
    dwNew <- gradRes$dwTotal
    e <- gradRes$Etotal

    ## define new search direction dv (conjugate direction)
    
    ## b1=dw_new'*dw_new; # Fletcher-Reeves
    b1 <- crossprod(dwNew, (dwNew - dw))#Polak-Ribiere
    b2 <- crossprod(dw)
    beta <- b1 / b2
    
    dv <- -dwNew + dv %*% beta
    dw <- dwNew

    if(e > eLast) {
      dv <- -dwNew
      if(verbose)
        cat("!", sep="")
    }

    if(is.na(e))
      stop("Square error is NA (critical) - accuracy in line-search might be too small")

    if(tmpnet@featureSorting) 
      if(e / eSortLast < 0.90 || i == tmpnet@maxIter ||
         i == tmpnet@maxIter - 1 || i == tmpnet@maxIter - 2) {
        eSortLast <- e
        if(verbose)
          cat("<", i, ">", sep="")
        ## somewhat secret method, sortFeatures calls
        ## nlnet@weights$set(x) so the weights are updated here
        ## 'behind the scenes'
        sortFeatures(tmpnet, trainIn, trainOut)
      }

  }
  tmpnet
}

##' This implements the SVDimpute algorithm as proposed by Troyanskaya
##' et al, 2001.  The idea behind the algorithm is to estimate the
##' missing values as a linear combination of the \code{k} most
##' significant eigengenes.
##'
##' Missing values are denoted as \code{NA}. It is not recommended
##' to use this function directely but rather to use the pca() wrapper
##' function.
##' 
##' As SVD can only be performed on complete matrices, all missing
##' values are initially replaced by 0 (what is in fact the mean on
##' centred data).  The algorithm works iteratively until the change
##' in the estimated solution falls below a certain threshold.  Each
##' step the eigengenes of the current estimate are calculated and
##' used to determine a new estimate. Eigengenes denote the loadings
##' if pca is performed considering variable (for Microarray data
##' genes) as observations.
##'
##' An optimal linear combination is found by regressing the
##' incomplete variable against the \code{k} most significant
##' eigengenes. If the value at position \code{j} is missing, the
##' \eqn{j^th}{j^th} value of the eigengenes is not used when
##' determining the regression coefficients.
##' @title SVDimpute algorithm
##' @param Matrix \code{matrix} -- Pre-processed (centered, scaled)
##' data with variables in columns and observations in rows. The data
##' may contain missing values, denoted as \code{NA}.
##' @param nPcs \code{numeric} -- Number of components to
##' estimate. The preciseness of the missing value estimation depends
##' on the number of components, which should resemble the internal
##' structure of the data.
##' @param threshold The iteration stops if the change in the matrix
##' falls below this threshold.
##' @param maxSteps Maximum number of iteration steps.
##' @param verbose Print some output if TRUE.
##' @param ... Reserved for parameters used in future version of the
##' algorithm
##' @note Each iteration, standard PCA (\code{prcomp}) needs to be
##' done for each incomplete variable to get the eigengenes. This is
##' usually fast for small data sets, but complexity may rise if the
##' data sets become very large.
##' @return Standard PCA result object used by all PCA-based methods
##' of this package. Contains scores, loadings, data mean and
##' more. See \code{\link{pcaRes}} for details.
##' @examples
##' ## Load a sample metabolite dataset with 5\% missing values
##' data(metaboliteData)
##' ## Perform svdImpute using the 3 largest components
##' result <- pca(metaboliteData, method="svdImpute", nPcs=3, center = TRUE)
##' ## Get the estimated complete observations
##' cObs <- completeObs(result)
##' ## Now plot the scores
##' plotPcs(result, type = "scores")
##' @keywords multivariate
##' @references Troyanskaya O. and Cantor M. and Sherlock G. and Brown
##' P. and Hastie T. and Tibshirani R. and Botstein D. and Altman
##' RB. - Missing value estimation methods for DNA
##' microarrays. \emph{Bioinformatics. 2001 Jun;17(6):520-5.}
##' @author Wolfram Stacklies
##' @export
svdImpute <- function(Matrix, nPcs=2, threshold=0.01, maxSteps=100,
                      verbose=interactive(), ...) {
  missing <- is.na(Matrix)
  temp <- apply(missing, 2, sum)
  missIx <- which(temp != 0)

  ## Initially set estimates to 0
  Matrix[missing] <- 0

  ## Now do the regression
  count <- 0
  error <- Inf

  while ( (error > threshold) && (count < maxSteps) ) {
    res <- prcomp(t(Matrix), center = FALSE, scale = FALSE, retx = TRUE)
    loadings <- res$rotation[,1:nPcs, drop = FALSE]
    sDev <- res$sdev

    ## Estimate missing values as a linear combination of the eigenvectors
    ## The optimal solution is found by regression against the k eigengenes
    for (index in missIx) {
      target <- Matrix[!missing[,index],index, drop = FALSE]
      Apart <- loadings[!missing[,index], , drop = FALSE]
      Bpart <- loadings[missing[,index], , drop = FALSE]
      X <- MASS::ginv(Apart) %*% target
      estimate <- Bpart %*% X
      
      Matrix[missing[,index], index] <- estimate
    }
    
    count <- count + 1
    if (count > 5) {
      error <- sqrt(sum( (MatrixOld - Matrix)^2 ) / sum(MatrixOld^2))
      if (verbose) { cat("change in estimate: ", error, "\n") }
    }
    MatrixOld <- Matrix
  }

  tmp <- prcomp(Matrix, center = FALSE, scale = FALSE, retx = TRUE)
  loadings <- cbind(tmp$rotation[,1:nPcs])
  scores <- cbind(tmp$x[,1:nPcs])

  ## Calculate R2cum
  R2cum <- rep(NA, nPcs)
  TSS <- sum(Matrix^2, na.rm=TRUE)
  for (i in 1:nPcs) {
    difference <-
      Matrix - (scores[,1:i, drop=FALSE] %*% t(loadings[,1:i, drop=FALSE]))
    R2cum[i] <- 1 - (sum(difference^2) / TSS)
  }

  result <- new("pcaRes")
  result@scores <- scores
  result@loadings <- loadings
  result@R2cum <- R2cum
  result@method <- "svdImpute"

  return(result)
}

##' ONB = orth(mat) is an orthonormal basis for the range of matrix
##' mat.  That is, ONB' * ONB = I, the columns of ONB span the same
##' space as the columns of mat, and the number of columns of ONB is
##' the rank of mat.
##' @title Calculate an orthonormal basis
##' @param mat matrix to calculate orthonormal base
##' @param skipInac  do not include components with precision below
##' .Machine$double.eps if TRUE
##' @return orthonormal basis for the range of matrix
##' @author Wolfram Stacklies
orth <- function(mat, skipInac = FALSE) {

  if(nrow(mat) > ncol(mat)) {
    leftSVs <- ncol(mat)
  } else {
    leftSVs <- nrow(mat)
  }

  result <- svd(mat, nu =  leftSVs, nv = ncol(mat))
  U <- result[[2]]
  S <- result[[1]]
  V <- result[[3]]

  m <- nrow(mat)
  n <- ncol(mat)

  if(m > 1) { 
    s <- diag(S, nrow = length(S))
  } else     if(m == 1) { 
    s <- S[1] 
  } else { 
    s <- 0 
  }

  tol <- max(m,n) * max(s) * .Machine$double.eps
  r <- sum(s > tol)
  if ( r < ncol(U) ) {
    if (skipInac) {
      warning("Precision for components ", r + 1 , " - ", ncol(U), 
              " is below .Machine$double.eps. \n",
              "Results for those components are likely to be inaccurate!!\n",
              "These component(s) are not included in the returned solution!!\n")
    } else {
      warning("Precision for components ", r + 1 , " - ", ncol(U), 
              " is below .Machine$double.eps. \n",
              "Results for those components are likely to be inaccurate!!\n")
    }
  }
  
  if (skipInac) {
    ONB <- U[, 1:r, drop=FALSE]
    ## Assing correct row and colnames
    rownames(ONB) <- labels(mat[, 1:r, drop=FALSE])[[1]];
    colnames(ONB) <- labels(mat[, 1:r, drop=FALSE])[[2]];
  } else {
    ONB<-U
    ## Assing correct row and colnames
    rownames(ONB) <- labels(mat)[[1]];
    colnames(ONB) <- labels(mat)[[2]];
  }

  return(ONB)
}

##' Line search for conjugate gradient
##' @param nlnet The nlnet
##' @param dw ..
##' @param e0 ..
##' @param ttGuess ..
##' @param trainIn Training data
##' @param trainOut Fitted data
##' @param verbose  logical, print messages
##' @return ...
##' @author Henning Redestig, Matthias Scholz
lineSearch <- function(nlnet, dw, e0, ttGuess, trainIn, trainOut, verbose) {
  iterGoldenSectionSearch <- 6
  alpha <- 0.618034
  tt <- rep(0, 4)
  e <- rep(0, 4)
  tmpnlnet <- forkNlpcaNet(nlnet)
  

  tt[1] <- 0
  e[1] <- e0
  tt[4] <- ttGuess
  tmpnlnet@weights$set(nlnet@weights$current() + tt[4] * dw)
  e[4] <- nlnet@error(tmpnlnet, trainIn, trainOut)$error

  if(e[4] > e[1]) {                     #got final interval calculate tt[2] and tt[3]
    tt[2] <- tt[1] + (1 - alpha) * (tt[4] - tt[1])
    tmpnlnet@weights$set(nlnet@weights$current() + tt[2] * dw)
    e[2] <- nlnet@error(tmpnlnet, trainIn, trainOut)$error
    tt[3] <- tt[1] + alpha * (tt[4] - tt[1])
    tmpnlnet@weights$set(nlnet@weights$current() + tt[3] * dw)
    e[3] <- nlnet@error(tmpnlnet, trainIn, trainOut)$error
  }
  else {                                #expand, add new tt[4]
    tt[3] <- tt[4]
    e[3] <- e[4]
    tt[4] <- (1 + alpha) * tt[4]
    tmpnlnet@weights$set(nlnet@weights$current() + tt[4] * dw)
    e[4] <- nlnet@error(tmpnlnet, trainIn, trainOut)$error
    if(e[4] > e[3]) {                   #got final interval, calculate tt[2]
      tt[2] <- tt[1] + (1 - alpha) * (tt[4] - tt[1])
      tmpnlnet@weights$set(nlnet@weights$current() + tt[2] * dw)
      e[2] <- nlnet@error(tmpnlnet, trainIn, trainOut)$error
    }
    else {                              #expand: add new tt[4]
      i <- 1
      while(e[4] < e[3] && i < 50) {
        tt[2] <- tt[3]
        e[2] <- e[3]
        tt[3] <- tt[4]
        e[3] <- e[4]
        tt[4] <- (1 + alpha) * tt[4]
        tmpnlnet@weights$set(nlnet@weights$current() + tt[4] * dw)
        e[4] <- nlnet@error(tmpnlnet, trainIn, trainOut)$error
        i <- i + 1
        if(verbose && i == 50)
          cat("^")
      }
    }
  }

  ## golden section search

  for(i in 1:iterGoldenSectionSearch) {
    if(e[3] > e[2]) {
      tt[4] <- tt[3]                      #remove right value tt[4]
      e[4] <- e[3]
      tt[3] <- tt[2]
      e[3] <- e[2]
      tt[2] <- tt[1] + (1 - alpha) * (tt[4] - tt[1]) #split left interval
      tmpnlnet@weights$set(nlnet@weights$current() + tt[2] * dw)
      e[2] <- nlnet@error(tmpnlnet, trainIn, trainOut)$error
    }
    else {
      tt[1] <- tt[2]                      #remove left t value tt[1]
      e[1] <- e[2]
      tt[2] <- tt[3]
      e[2] <- e[3]
      tt[3] <- tt[1] + alpha * (tt[4] - tt[1]) #split right interval
      tmpnlnet@weights$set(nlnet@weights$current() + tt[3] * dw)
      e[3] <- nlnet@error(tmpnlnet, trainIn, trainOut)$error
    }
  }
  if(e[2] < e[3]) {
    eBest <- e[2]
    ttBest <- tt[2]
  }
  else {
    eBest <- e[3]
    ttBest <- tt[3]
  }

  wBest <- nlnet@weights$current() + ttBest * dw
  return(list(wBest=wBest, eBest=eBest, ttBest=ttBest))
}

##' Linear kernel
##' @param x datum
##' @return Input value
##' @author Henning Redestig, Matthias Scholz
linr <- function(x) x

##' Simulated data set looking like a helix
##' 
##' 
##' A matrix containing 1000 observations (rows) and three variables
##' (columns).
##' @title A helix structured toy data set
##' @name helix
##' @aliases helix
##' @usage data(helix)
##' @docType data
##' @references Matthias Scholz, Fatma Kaplan, Charles L. Guy, Joachim
##' Kopka and Joachim Selbig. - Non-linear PCA: a missing data
##' approach.  \emph{Bioinformatics 2005 21(20):3887-3895}
##' @keywords datasets
##' @author Henning Redestig 
NULL

##' A complete subset from a larger metabolite data set.  This is the
##' original, complete data set and can be used to compare estimation
##' results created with the also provided incomplete data (called
##' metaboliteData).  The data was created during an in house
##' Arabidopsis coldstress experiment.
##'
##' A matrix containing 154 observations (rows) and 52 metabolites
##' (columns).
##' @name metaboliteDataComplete
##' @docType data
##' @aliases metaboliteDataComplete
##' @title A complete metabolite data set from an Arabidopsis
##' coldstress experiment
##' @keywords datasets
##' @seealso  \code{\link{metaboliteData}}
##' @references Matthias Scholz, Fatma Kaplan, Charles L. Guy, Joachim
##' Kopka and Joachim Selbig. - Non-linear PCA: a missing data
##' approach.\emph{Bioinformatics 2005 21(20):3887-3895}
##' @author Wolfram Stacklies
NULL

##' A incomplete subset from a larger metabolite data set.  This is the
##' original, complete data set and can be used to compare estimation
##' results created with the also provided incomplete data (called
##' metaboliteData).  
##'
##' A matrix containing 154 observations (rows) and 52 metabolites
##' (columns). The data contains 5\% of artificially created uniformly
##' distributed misssing values. The data was created during an in
##' house Arabidopsis coldstress experiment.
##' @name metaboliteData
##' @docType data
##' @aliases metaboliteData
##' @title A incomplete metabolite data set from an Arabidopsis
##' coldstress experiment
##' @keywords datasets
##' @seealso  \code{\link{metaboliteDataComplete}}
##' @references Matthias Scholz, Fatma Kaplan, Charles L. Guy, Joachim
##' Kopka and Joachim Selbig. - Non-linear PCA: a missing data
##' approach.\emph{Bioinformatics 2005 21(20):3887-3895}
##' @author Wolfram Stacklies
NULL


##' Principal Component Analysis in R
##'
##' \tabular{ll}{
##' Package: \tab pcaMethods \cr
##' Type: \tab Package \cr
##' Developed since: \tab 2006 \cr
##' License: \tab GPL (>=3) \cr
##' LazyLoad: \tab yes \cr
##' }
##'
##' Provides Bayesian PCA, Probabilistic PCA, Nipals PCA, Inverse
##' Non-Linear PCA and the conventional SVD PCA. A cluster  based
##' method for missing value estimation is included for comparison.
##' BPCA, PPCA and NipalsPCA may be used to perform PCA on incomplete
##' data as well as for accurate missing value estimation.  A set of
##' methods for printing and plotting the results is also provided.
##' All PCA methods make use of the same data structure (pcaRes) to
##' provide a unique interface to the PCA results. Developed at the
##' Max-Planck Institute for Molecular Plant Physiology, Golm,
##' Germany, RIKEN Plant Science Center Yokohama, Japan, and CAS-MPG
##' Partner Institute for Computational Biology (PICB) Shanghai,
##' P.R. China
##'
##' @name pcaMethods
##' @aliases pcaMethods
##' @docType package
##' @importFrom Rcpp evalCpp
##' @import Biobase
##' @import BiocGenerics
##' @import methods
##' @title pcaMethods
##' @useDynLib pcaMethods
##' @author Wolfram Stacklies, Henning Redestig
NULL

##' \describe{
##' \item{plotR2}{Lack of relevance for this plot and the fact that it
##' can not show cross-validation based diagnostics in the same plot
##' makes it redundant with the introduction of a dedicated
##' \code{plot} function for  \code{pcaRes}. The new plot only shows
##' R2cum but the result is pretty much the same.}}
##' @name pcaMethods-deprecated
##' @aliases pcaMethods-deprecated
##' @title Deprecated methods for pcaMethods
##' @author Henning Redestig 
NULL

##' Model initialization for Bayesian PCA. This function is NOT
##' inteded to be run separately!
##'
##' The function calculates the initial Eigenvectors by use of SVD
##' from the complete rows.  The data structure M is created and
##' initial values are  assigned.
##' @title Initialize BPCA model
##' @param y numeric matrix containing missing values. Missing values
##' are denoted as 'NA'
##' @param components Number of components used for estimation
##' @return List containing
##' \item{rows}{Row number of input matrix}
##' \item{cols}{Column number of input matrix}
##' \item{comps}{Number of components to use}
##' \item{yest}{(working variable) current estimate of complete data}
##' \item{row_miss}{(Array) Indizes of rows containing missing values}
##' \item{row_nomiss}{(Array) Indices of complete rows (such with no
##' missing values)}
##' \item{nans}{Matrix of same size as input data. TRUE if \code{input == NA},
##' false otherwise}
##' \item{mean}{Column wise data mean}
##' \item{PA}{ (d x k) Estimated principal axes (eigenvectors,
##' loadings) The matrix ROWS are the vectors}
##' \item{tau}{Estimated precision of the residual error}
##' \item{scores}{ Estimated scores}
##' Further elements are: galpha0, balpha0, alpha, gmu0, btau0, gtau0,
##' SigW. These are working variables or constants.
##' @author Wolfram Stacklies
BPCA_initmodel <- function(y, components) {
  ## Initialization, write static parameters to the central
  M <- NULL 
  M$rows <- nrow(y)
  M$cols <- ncol(y) 
  M$comps <- components ## Column number
  M$yest <- y ## Original data, NAs are set to 0 later on

  ## Find rows with missing values, etc...
  M$nans <- is.na(y)
  temp <- apply(M$nans, 1, sum)
  M$row_nomiss <- which(temp == 0)
  M$row_miss <- which(temp != 0)
  M$yest[M$nans] <- 0
  M$scores <- NULL

  ## Get the SVD of the complete rows
  covy <- cov(M$yest)
  values <- svd(covy, components, components)
  U <- values[[2]]
  S <- diag( values[[1]][1:components], nrow = components, ncol = components)
  V <- values[[3]]

  ## M$mean: column wise mean of the original data
  M$mean <- matrix(0, 1, M$cols)
  for(j in 1:M$cols) {
    idx <- which(!is.na(y[,j]))
    M$mean[j] <- mean(y[idx,j])
  }

  M$PA <- U %*% sqrt(S)
  M$tau <- 1 / ( sum(diag(covy)) - sum(diag(S)) )
  
  ## Constants etc
  taumax <- 1e10
  taumin <- 1e-10
  M$tau <- max( min(M$tau, taumax), taumin )

  M$galpha0 <- 1e-10
  M$balpha0 <- 1
  M$alpha <- (2 * M$galpha0 + M$cols) / (M$tau * diag(t(M$PA) %*% M$PA) + 2 * M$galpha0 / M$balpha0)

  M$gmu0 <- 0.001

  M$btau0 <- 1
  M$gtau0 <- 1e-10
  M$SigW <- diag(components)
  return(M)
}

##' Check a given data matrix for consistency with the format
##' required for further analysis.
##' The data must be a numeric matrix and not contain:
##' \itemize{
##' \item Inf values
##' \item NaN values
##' \item Rows or columns that consist of NA only
##' }
##' @title Do some basic checks on a given data matrix
##' @param data \code{matrix} -- Data to check.
##' @param verbose  \code{boolean} -- If TRUE, the function prints
##' messages whenever an error in the data set is found.
##' @return \item{isValid}{\code{boolean} -- TRUE if no errors were
##' found, FALSE otherwise.  isValid contains a set of attributes,
##' these are: \itemize{ \item isNumeric - TRUE if data is numeric,
##' false otherwise \item isInfinite - TRUE if data contains 'Inf'
##' values, false otherwise \item isNaN - TRUE if data contains 'NaN'
##' values, false otherwise \item isMatrix - TRUE if the data is in
##' matrix format, FALSE otherwise \item naRows - TRUE if data
##' contains rows in which all elements are 'NA',  FALSE otherwise
##' \item naCols - TRUE if data contains columns in which all elements
##' are 'NA',  FALSE otherwise }}
##' @keywords multivariate
##' @export 
##' @author Wolfram Stacklies
checkData <- function(data, verbose = FALSE) {
  isValid <- TRUE

  isNumeric <- TRUE
  isInfinite <- FALSE
  isNaN <- FALSE
  isMatrix <- TRUE
  naRows <- FALSE
  naCols <- FALSE

  if (!is.numeric(data)) {
    isNumeric <- FALSE
    isValid <- FALSE
    if (verbose)
      message("Error: Data is not numeric")
  } 

  if ( sum(is.infinite(data) >= 1) ) {
    isInfinite <- TRUE
    isValid <- FALSE
    if (verbose)
      message("Error: Data contains 'Inf' values")
  } 

  if (sum(is.nan(data) >= 1)) {
    isNaN <- TRUE
    isValid <- FALSE
    if (verbose)
      message("Error: Data contains 'NaN' values. Missing values must be denoted by 'NA'")
  } 

  if (!is.matrix(data)) {
    isMatrix <- FALSE
    isValid <- FALSE
    if (verbose)
      message("Error: data is not a matrix. Try to use as.matrix(data)")
  }

  ## Check for entire rows that are NA only
  if (sum(apply(is.na(data), 1, sum) == ncol(data)) >= 1 ) {
    naRows <- TRUE
    isValid <- FALSE
    if (verbose)
      message("Error: Data contains rows in which all elements are 'NA'. Remove them first")
  }

  ## Check for entire columns that are NA only
  if (sum(apply(is.na(data), 2, sum) == nrow(data)) >= 1 ) {
    naCols <- TRUE
    isValid <- FALSE
    if (verbose)
      message("Error: Data contains columns in which all elements are 'NA'. Remove them first")
  } 
  
  attr(isValid, "isNumeric") <- isNumeric
  attr(isValid, "isInfinite") <- isInfinite
  attr(isValid, "isNaN") <- isNaN
  attr(isValid, "isMatrix") <- isMatrix
  attr(isValid, "naRows") <- naRows
  attr(isValid, "naCols") <- naCols
  return(isValid)
}

\documentclass[a4paper]{article}

%\VignetteIndexEntry{Data with outliers}

\usepackage{hyperref}

\title{Handling of data containing outliers}
\author{Wolfram Stacklies and Henning Redestig\\
CAS-MPG Partner Institute for Computational Biology (PICB)\\
Shanghai, P.R. China \\
and\\
Max Planck Institute for Molecular Plant Physiology\\
Potsdam, Germany\\
\url{http://bioinformatics.mpimp-golm.mpg.de/}
}
\date{\today}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}
@
\maketitle

\section{PCA robust to outliers}
Away from often showing missing values, Microarray or Metabolite data are often
corrupted with extreme values (outliers). 
Standard SVD is highly susceptible to outliers. In
the extreme case, an individual data point, if sufficiently outlying, can draw even the
leading principal component toward itself.
This problem can be addressed by using 
a robust analysis method. Hereto we provide \texttt{robustSvd}, a singular value
decomposition robust to outliers. \texttt{robustPca} is a PCA implementation that
resembles the original \texttt{R} \texttt{prcomp} method, with the difference
that it uses \texttt{robustSvd} instead of the standard \texttt{svd} function.\\
Robust SVD and its application to microarray data were proposed in \cite{hawkins01} 
and \cite{liu03}. The algorithm is based on the idea to use a sequential estimation 
of the eigenvalues and left and right eigenvectors that
ignores missing values and is resistant to outliers. \\
The \texttt{robustSvd} script included here was contributed by Kevin Wright. Thanks
a lot to him! 

\section{Outliers and missing value imputation}
The problem of outliers is similar to the missing data problem in the sense that
extreme values provide no or wrong information. They are generally artifacts of
the experiment and provide no information about the underlying biological
processes. \\
Most of the PCA methods coming with the package were not designed to be robust to
outliers in the sense that they will converge to the standard PCA solution on a complete
data set.
Yet, an applicable solution is to remove obvious outliers from the data first (by setting
them NA) and to then estimate the PCA solution on the incomplete data. This is likely
to produce accurate results if the number of missing data does not exceed a certain
amount, less than 10\% should be a good number.

The following example illustrates the effect of outliers and the use of robust methods.
First, we attach the complete metabolite data set and create 5\% outliers. We
mean center the data before we create outliers because these large artificial
outliers will strongly shift the original means. This would not allow for objective comparison
between the differnt results obtained, e.g. when doing scatterplots.
<<results=hide>>=
library(pcaMethods)
@
<<>>=
data(metaboliteDataComplete)
mdc          <- scale(metaboliteDataComplete, center=TRUE, scale=FALSE)
cond         <- runif(length(mdc)) < 0.05
mdcOut       <- mdc
mdcOut[cond] <- 10
@
Then we calculate a PCA solution using standard SVD and robust SVD.
<<results=hide>>=
resSvd       <- pca(mdc, method="svd", nPcs=5, center=FALSE)
resSvdOut    <- pca(mdcOut, method="svd", nPcs=5, center=FALSE)
resRobSvd    <- pca(mdcOut, method="robustPca", nPcs=5, center=FALSE)
@
Now we use \texttt{PPCA} to estimate the PCA solution, 
but set the outliers NA before.
<<results=hide>>=
mdcNa        <- mdc
mdcNa[cond]  <- NA
resPPCA      <- pca(mdcNa, method="ppca", nPcs=5, center=FALSE)
@

To check the robustness to outliers we can just do a scatterplot comparing the
results to the optimal PCA solution for the complete data set (which is \texttt{resSvd}).
In Figure \ref{fig:svdPlot} we plot the estimated and original loadings against each other.
\begin{figure}[!ht]
  \centering
<<fig=true, width=8, height=8, results=hide, echo=false>>=
par(mfrow=c(2,2))
plot(loadings(resSvd)[,1], loadings(resSvdOut)[,1], 
xlab="Loading 1 SVD", ylab="Loading 1 SVD with outliers")
plot(loadings(resSvd)[,1], loadings(resRobSvd)[,1],
xlab="Loading 1 SVD", ylab="Loading 1 robustSVD with outliers")
plot(loadings(resSvd)[,1], loadings(resPPCA)[,1],
xlab="Loading 1 SVD", ylab="Loading 1 PPCA with outliers=NA")
plot(loadings(resRobSvd)[,1], loadings(resPPCA)[,1],
xlab="Loading 1 robust SVD with outliers", ylab="Loading 1 svdImpute with outliers=NA")
@
  \caption{Figures show (from left to right): \newline
  Original PCA solution vs. solution on data with outliers; \newline
  Original PCA solution vs. robust PCA solution on data with outliers; \newline
  Original PCA solution vs. PPCA solution on data where outliers=NA; \newline
  Robust PCA solution vs. PPCA solution on data with outliers / outliers=NA.
  \label{fig:svdPlot}
  }
\end{figure}

\begin{thebibliography}{2006}
\bibitem{hawkins01} Hawkins, D.M., Liu, L. and Young, S.S.
{\sl Robust Singular Value Decomposition.}
National Institute of Statistical Sciences, 2001, Tech Report 122.

\bibitem{liu03} Liu, L., Hawkins, D.M., Ghosh, S. and Young, S.S.
{\sl Robust singular value decomposition analysis of microarray data.}
PNAS, 2003;100:13167--13172.
\end{thebibliography}

\end{document}

\documentclass[a4paper]{article}

%\VignetteIndexEntry{Introduction}

\usepackage{hyperref}

\title{The pcaMethods Package}
\author{Wolfram Stacklies and Henning Redestig\\
CAS-MPG Partner Institute for Computational Biology (PICB)\\
Shanghai, P.R. China \\
and\\
Max Planck Institute for Molecular Plant Physiology\\
Potsdam, Germany\\
\url{http://bioinformatics.mpimp-golm.mpg.de/}
}
\date{\today}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}
@
\maketitle
\section*{Overview}
The \texttt{pcaMethods} package \cite{stacklies07} provides a set of
different PCA implementations, together with tools for cross
validation and visualisation of the results.  The methods basically
allow to perform PCA on incomplete data and thus may also be used for
missing value estimation.

When doing PCA one assumes that the data is restricted to a subspace
of lower dimensionality, e.g. correlation patterns between jointly regulated
genes. PCA aims to extract these structures thereby filtering noise out.
If only the most significant loadings (eigenvectors, also referred to as principal components)
are used for projection this can be written as:
\begin{equation}
X = 1\times{}\bar{x}^T + TP^T + V
\end{equation}
Where the term $1\times{}\bar{x}^T$ represents the original variable
averages, $X$ denotes the observations, $T={t_1, t_2,\ldots,t_k}$ the
latent variables or scores, $P={p_1, p_2,\ldots,p_k}$ 
the transformation matrix (consisting of the most significant eigenvectors of the covariance matrix)
and $V$ are the residuals.

Missing values may be estimated by projecting the scores back into the original
space using $\hat{X} = 1\times{}\bar{x}^T + TP^T$.
Optimally, this produces an estimate of the missing data based on the
underlying correlation structure, thereby ignoring noise.
This will only produce reasonable results
if the residuals $V$ are sufficiently small, implying that most of the
important information is captured by the first $k$ components.

In order to calculate the transformation matrix $P$ one needs to determine
the covariance matrix between variables or alternatively calculate $P$
directly via SVD. In both cases, this can only be done on complete matrices.
However, an approximation may be obtained by use of different
regression methods.
The PCA methods provided in this package implement algorithms to accurately
estimate the PCA solution on incomplete data.

Although the focus of this package is clearly to provide a collection of
PCA methods we also provide a cluster based method for missing value imputation.
This allows to better rate and compare the results.

\section{Algorithms}
All methods return a common class called \texttt{pcaRes} as a container
for the results. This guarantees maximum flexibility for the user. A wrapper
function called \texttt{pca()} is provided that receives the desired
type of pca as a string.

\subsection*{svdPca}
This is a wrapper function for $R's$ standard \texttt{prcomp}
function. It delivers the results as a \texttt{pcaRes} object
for compatibility with the rest of the package.

\subsection*{svdImpute}
This implements the SVDimpute algorithm as proposed by Troyanskaya et~al
\cite{troyanskaya01}.
The idea behind the algorithm is to estimate the missing values as a
linear combination of the $k$ most significant eigengenes\footnote{The
term ``eigengenes'' denotes the loadings when PCA was applied considering
variables (here the genes) as observations.}.
The algorithm works iteratively until the change in the estimated solution
falls below a certain threshold.
Each step the eigengenes of the current estimate are calculated and
used to determine a new estimate.

An optimal linear combination is found by regressing an incomplete
variable against the $k$ most significant eigengenes. If the value at
position $j$ is missing,
the $j^{th}$ value of the eigengenes is not used when determining
the regression coefficients.\\
SVDimpute seems to be tolerant to relatively high amount of missing data
(> 10\%).

\subsection*{Probabilistic PCA (ppca)}
Probabilistic PCA combines an EM approach for PCA with
a probabilistic model. The EM approach is based on the assumption that
the latent variables as well as the noise are normal distributed.

In standard PCA data which is far from the training set but close to the
principal subspace may have the same reconstruction error, see Figure
\ref{fig:pcaSubspace} for explanation.
<<results=hide, echo=false>>=
library(pcaMethods)
x <- c(-4,7); y <- c(-3,4)
distX <- rnorm(100, sd=0.3)*3
distY <- rnorm(100, sd=0.3) + distX * 0.3
mat <- cbind(distX, distY)
res <- pca(mat, nPcs=2, method="svd", center=F)
loading <- loadings(res)[1,]
grad <- loading[2] / loading[1]
if (grad < 0)
   grad <- grad * -1
lx <- c(-4,7)
ly <- c(grad * -4, grad * 7)
@
\begin{figure}
	\centering
<<fig=true, width=8, height=5, results=hide, echo=false>>=
par(mar=c(2, 3, 2, 2))
plot(x,y, type="n", xlab="", ylab="")
abline(v=0, col="dark gray", lwd = 2); abline(h=0, col = "dark gray", lwd = 2)
points(distX, distY, type = 'p', col = "blue")
lines(lx,ly, lwd = 2)
points(-1, -1 * grad + 0.5, pch = 19, col = "red", lwd=4)
points(6, 6 * grad + 0.5, pch = 19, col = "red", lwd=4)
@
\caption{Normal distributed data with the first loading plotted in black.
The two red points have the same reconstruction error because PCA does
not define a density model. Thus the only measure of how well new data fits
the model is the distance from the principal subspace. Data points far from
the bulk of data but still close to the principal subspace will have a low
reconstruction error. \label{fig:pcaSubspace}}
\end{figure}

PPCA defines a likelihood function such that the likelihood for data
far from the training set is much lower, even if they are close to the
principal subspace.
This allows to improve the estimation accuracy.\\
PPCA is tolerant to amounts of missing values between 10\% to 15\%.
If more data is missing the algorithm is likely not to converge to
a reasonable solution.

The method was implemented after the draft
``\texttt{EM Algorithms for PCA and Sensible PCA}''
written by Sam Roweis and after the Matlab \texttt{ppca} script implemented
by \emph{Jakob Verbeek}\footnote{\url{http://lear.inrialpes.fr/~verbeek/}}.

Please check also the PPCA help file.

\subsection*{Bayesian PCA (bpca)}
Similar to probabilistic PCA, Bayesian PCA uses an EM approach together
with a Bayesian model to calculate the likelihood for a reconstructed value.\\
The algorithm seems to be tolerant to relatively high amounts of missing data
(> 10\%).
Scores and loadings obtained with Bayesian PCA slightly differ
from those obtained with conventional PCA.
This is because BPCA was developed especially for missing value estimation and
is based on a variational Bayesian
framework (VBF), with automatic relevance
determination (ARD). In BPCA, ARD leads to a different scaling of the
scores, loadings and eigenvalues when compared to standard PCA or
PPCA.
The algorithm does not force orthogonality between loadings.
However, the authors of the BPCA paper found that including an orthogonality criterion made the
predictions worse.
They also state that the difference between ``real'' and predicted
Eigenvalues becomes larger when the number of observation is smaller,
because it reflects the lack of information to accurately determine
true loadings from the limited and noisy data.
As a result, weights of factors to predict missing values are not the same as
with conventional PCA, but the missing value estimation is improved.

BPCA was proposed by Oba et~al \cite{oba03}.
The method available in this package is a port of the \texttt{bpca} Matlab
script also provided by the authors\footnote{
\url{http://hawaii.aist-nara.ac.jp/\%7Eshige-o/tools/}}.

\subsection*{Inverse non-linear PCA (NLPCA)} 
NLPCA \cite{scholz05} is especially suitable for data from experiments
where the studied response is non-linear. Examples of such experiments
are ubiquitous in biology -- enzyme kinetics are inherently non-linear
as are gene expression responses influenced by the cell cycle or
diurnal oscillations.  NLPCA is based on training an auto-associative
neural network composed of a component layer which serves as the
``bottle-neck'', a hidden non-linear layer and an output layer
corresponding to the reconstructed data.  The loadings can be seen as
hidden in the network.  Missing values in the training data are simply
ignored when calculating the error during back-propagation. Thus NLPCA
can be used to impute missing values in the same way as for
conventional PCA. The only difference is that the
loadings $P$ are now represented by a neural network.\\
A shortcoming of the current implementation is that there is no
reasonable stop criterion. The quality of the estimated solution
depends on the number of iterations. This should in most cases be
somewhat between 500 and 1500.  We recommend to use \texttt{kEstimate}
or \texttt{kEstimateFast} to determine this parameter.

\subsection*{Nipals PCA}
Nipals (Nonlinear Estimation by Iterative Partial Least Squares)
\cite{wold66} is an algorithm at the root of PLS regression which can
execute PCA with missing values by simply leaving those out from the
appropriate inner products.  It is tolerant to small amounts
(generally not more than 5\%) of missing data.

\subsection{Local least squares (LLS) imputation}
The package provides an algorithm called \texttt{llsImpute} for
missing value estimation based on a linear combination of the $k$
nearest neighbours of an incomplete variable (in Microarray
experiments normally a gene).  The distance between variables is
defined as the absolute value of the Pearson, Spearman or Kendall
correlation coefficient.  The optimal linear combination is found by
solving a local least squares problem as described in \cite{kim05}. In
tests performed in the cited paper the llsImpute algorithm is able to
outperform knnImpute\cite{troyanskaya01} and competes well with BPCA.

In the current implementation two slightly different ways for missing
value estimation are provided. The first one is to restrict the
neighbour searching to the subset of complete variables.  This is
preferable when the number of incomplete variables is relatively
small.

The second way is to consider all variables as candidates.
Here, missing values are initially replaced by the columns wise mean.
The method then iterates, using the current estimate as input for the
LLS regression until the change between new and old estimate falls below a 
certain threshold (0.001).

\section{Getting started}
\paragraph{Installing the package.} To install the package first
download the appropriate file for your platform from the Bioconductor
website (\url{http://www.bioconductor.org/}). For Windows, start
\texttt{R} and select the \texttt{Packages} menu, then \texttt{Install
  package from local zip file}.  Find and highlight the location of
the zip file and click on \texttt{open}.

For Linux/Unix, use the usual command \texttt{R CMD INSTALL} or set
the option \texttt{CRAN} to your nearest mirror site and use the
command \texttt{install.packages} from within an \texttt{R} session.

\paragraph{Loading the package:} To load the \texttt{pcaMethods}
package in your \texttt{R} session, type \texttt{library(pcaMethods)}.

\paragraph{Help files:} Detailed information on \texttt{pcaMethods}
package functions can be obtained from the help files.  For example,
to get a description of \texttt{bpca} type \texttt{help("bpca")}.

\paragraph{Sample data:} Two sample data sets are coming with the
package. \texttt{metaboliteDataComplete} contains a complete subset
from a larger metabolite data set. \texttt{metaboliteData} is the same
data set but with 10 \% values removed from an equal distribution.

\section{Some examples}
<<echo=false, results=hide>>=
library(lattice)
library(pcaMethods)
@
To load the package and the two sample data sets type:
<<results=hide>>=
library(pcaMethods)
data(metaboliteData)
data(metaboliteDataComplete)
@
Now centre the data
<<>>=
md  <- prep(metaboliteData, scale="none", center=TRUE)
mdC  <- prep(metaboliteDataComplete, scale="none", center=TRUE)
@
Run SVD pca, PPCA, BPCA, SVDimpute and nipalsPCA on the data, using
the \texttt{pca()} wrapper function. The result is always a \texttt{pcaRes}
object.
<<results=hide>>=
resPCA  <- pca(mdC, method="svd", center=FALSE, nPcs=5)
resPPCA  <- pca(md, method="ppca", center=FALSE, nPcs=5)
resBPCA  <- pca(md, method="bpca", center=FALSE, nPcs=5)
resSVDI  <- pca(md, method="svdImpute", center=FALSE, nPcs=5)
resNipals  <- pca(md, method="nipals", center=FALSE, nPcs=5)
resNLPCA <- pca(md, method="nlpca", center=FALSE, nPcs=5, maxSteps=300)
@
Figure \ref{fig:eigenvalues} shows a plot of the eigenvalue structure
(\texttt{sDev(pcaRes)}).
If most of the variance is captured
with few loadings PCA is likely to produce good missing value
estimation results.
For the sample data all methods show similar eigenvalues.
One can also see
that most of the variance is already captured by the first loading,
thus estimation is likely to work fine on this data.
For BPCA, the eigenvalues are scaled differently for reasons discussed above,
see Figure \ref{fig:loadingBPCA}. The order of the loadings remains the
same.
\begin{figure}
	\centering
<<fig=true, width=6, height=4, results=hide, echo=false>>=
sDevs <- cbind(sDev(resPCA), sDev(resPPCA), sDev(resBPCA), sDev(resSVDI), sDev(resNipals), sDev(resNLPCA))
matplot(sDevs, type = 'l', xlab="Eigenvalues", ylab="Standard deviation of PC", lwd=3)
legend(x="topright", legend=c("PCA", "PPCA", "BPCA", "SVDimpute","Nipals PCA","NLPCA"), lty=1:6, col=1:6, lwd=3)
@
\caption{Eigenvalue structure as obtained with different methods\label{fig:eigenvalues}}
\end{figure}

To get an impression of the correctness of the estimation
it is a good idea to plot the scores / loadings obtained with classical
PCA and one of the probabilistic methods against each other. This of
course requires a complete data set from which data is randomly removed.
Figure \ref{fig:loadingBPCA} shows this for BPCA on the sample data.
\begin{figure}
  \centering
<<fig=true, width=7, height=4, results=hide, echo=false>>=
par(mfrow=c(1,2))
plot(loadings(resBPCA)[,1], loadings(resPCA)[,1], xlab="BPCA", ylab="classic PCA", main = "Loading 1")
plot(loadings(resBPCA)[,2], loadings(resPCA)[,2], xlab="BPCA", ylab="classic PCA", main = "Loading 2")
@
\caption{Loading 1 and 2 calculated with BPCA plotted against those
  calculated with standard PCA. \label{fig:loadingBPCA}}
\end{figure}

\section{Cross validation}
\texttt{Q2} is the goodness measure used for internal cross
validation. This allows to estimate the level of structure in a data
set and to optimise the choice of number of loadings.
Cross validation is performed by removing random elements of
the data matrix, then estimating these using the PCA algorithm of
choice and then calculating $Q^2$ accordingly. At the moment,
cross-validation can only be performed with algorithms that allow
missing values (i.e. not SVD). Missing value independent
cross-validation is scheduled for implementation in later versions.
$Q^2$ is defined as following for the mean centered data (and possibly
scaled) matrix $X$.

$$\mathrm{SSX}=\sum (x_{ij})^2$$
$$\mathrm{PRESS}=\sum (x_{ij} - \hat{x}_{ij})^2$$
$$Q^2=1 - \mathrm{PRESS}/\mathrm{SSX}$$
The maximum value for $Q^2$ is thus 1 which means that all variance in
$X$ is represented in the predictions; $X=\hat{X}$. 
<<results=hide>>=
q2SVDI <- Q2(resSVDI, mdC, fold=10)
q2PPCA <- Q2(resPPCA, mdC, fold=10)
@ 
<<results=hide, echo=false>>=
# PPCA does not converge / misestimate a value in very rare cases.
# This is a workaround to avoid that such a case will break the
# diagram displayed in the vignette.
# From the 2.0 release of bioconductor on, the convergence threshold
# for PPCA was lowert to 1e-5, this should make the method much more
# stable. So this workaround might be obsolete now...
# [nope it is not, ppca is unstable]
while( sum((abs(q2PPCA)) > 1) >= 1 ) {
    q2PPCA <- Q2(resPPCA, mdC, fold=10)
}
@
\begin{figure}[!ht]
  \centering
<<fig=true, width=12, height=7, results=hide, echo=false>>=
q2 <- data.frame(Q2=c(drop(q2PPCA), drop(q2SVDI)), 
                 method=c("PPCA", "SVD-Impute")[gl(2, 5)], PC=rep(1:5, 2))
print(xyplot(Q2~PC|method, q2, ylab=expression(Q^2), type="h", lwd=4))
@
\caption{Boxplot of the \texttt{Q2} results for BPCA, Nipals PCA,
	SVDimpute and PPCA. PPCA and SVDimpute both
	deliver better results than BPCA and Nipals in this example.\label{fig:Q2}}
\end{figure}

The second method called \texttt{kEstimate} uses cross validation to
estimate the optimal number of loadings for missing value
estimation.  The \texttt{NRMSEP} (normalised root mean square error of prediction)
\cite{feten05} or Q2 can be used to define the average error of prediction. The
NRMSEP normalises the square difference between real and estimated
values for a certain variable by the variance within this variable. The idea
behind this normalisation is that the error of prediction will
automatically be higher if the variance is higher.  The
\texttt{NRMSEP} for mean imputation is $\sqrt{\frac{nObs}{nObs - 1}}$
when cross validation is used, where $nObs$ is the number of
observations.  The exact definition is:
\begin{equation}
NRMSEP_k = \sqrt{\frac{1}{g} \sum_{j \in G} \frac{\sum_{i \in O_j}
(x_{ij} - \hat{x}_{ijk})^2}{o_j s_{x_j}^2}}
\end{equation}
where $s^2_{x_j} = \sum_{i=1}^n (x_{ij} - \overline{x}_j)^2 / (n - 1)$,
this is the variance within a certain variable.
Further, $G$ denotes the set of incomplete variables, $g$ is the number of
incomplete varialbes. $O_j$ is the set of missing observations in
variable $j$ and $o_j$ is the number of missing observations in variable $j$.
$\hat{x}_{ijk}$ stands for the estimate of value $i$ of variable $j$ using $k$
loadings. See Figure \ref{fig:kEstimate} for an example.
The NRMSEP should be the error measure of choice. But if the number of
observations is small, the variance within a certain variable may become
and unstable criterion. If so or if variance scaling was applied we
recommend to use Q2 instead.
<<echo=true, results=hide>>=
errEsti <- kEstimate(md, method = "ppca", evalPcs=1:5, nruncv=1, em="nrmsep")
@
\begin{figure}[!ht]
  \centering
  \begin{minipage}[c]{0.6\textwidth}
    \centering  
<<fig=true, width=4, height=5, results=hide, echo=false>>=
barplot(drop(errEsti$eError), xlab="Loadings", ylab="NRMSEP (Single iteration)")
@
  \end{minipage}
  \begin{minipage}[c]{0.3\textwidth}
    \caption{Boxplot showing the \texttt{NRMSEP} versus the number of
    loadings. In this example only 1 iteration of the whole cross validation were performed.
    It is normally advisable to do more than just one iteration.
    \label{fig:kEstimate}}
  \end{minipage}
\end{figure}

\texttt{kEstimate} also provides information about the estimation error for individual variables.
The $Q^2$ distance or the NRMSEP are calculated separately for each variable. 
See the manpage for \texttt{kEstimate} and \texttt{kEstimateFast} for details.
Plotting the variable - wise results gives information about for which variables
missing value estimation makes sense, and for which no imputation or mean imputation
is preferable, see Figure \ref{fig:variableWiseError}.
If you are not interested in variable - wise information we recommend to use the faster
\texttt{kEstimateFast} instead.
\begin{figure}[!ht]
  \centering
  \begin{minipage}[c]{0.6\textwidth}
    \centering
<<fig=true, width=5, height=5, results=hide, echo=false>>=
barplot(drop(errEsti$variableWiseError[, which(errEsti$evalPcs == errEsti$bestNPcs)]), xlab="Incomplete variable Index", ylab="NRMSEP")
@
  \end{minipage}
  \begin{minipage}[c]{0.3\textwidth}
    \caption{Boxplot showing the \texttt{NRMSEP} for all incomplete variables in the data set.
    For the first 7 variables missing value imputation does not seem to make too much sense. 
    \label{fig:variableWiseError}}
  \end{minipage}
\end{figure}

\newpage
\section{Visualisation of the results}

\subsection{Quick scores and loadings plot}
Some methods for display of scores and loadings are also provided.
The function \texttt{slplot()} aims to be a simple way to quickly
visualise scores and loadings in an intuitive way, see Figure
\ref{fig:slplot}. Barplots are provided when plotting only one PC and
colours can be specified differently for the scores and loadings
plots. For a more specific scatter plot it is however recommended to
access scores and loadings slots and define own plot functions.

\begin{figure}[!h]
 \centering
<<fig=true, width=11, height=7, results=hide, echo=false>>=
slplot(resPCA)
@
\caption{\texttt{slplot} for scores and loadings obtained with
 classical SVD based PCA. \label{fig:slplot}}
\end{figure}

\noindent Another method called \texttt{plotPcs()} allows to visualise many
PCs plotted against each other, see Figure \ref{fig:plotPcs}.
\begin{figure}[!ht]
  \centering
<<fig=true, width=8, height=8, results=hide, echo=false>>=
plotPcs(resPPCA, pc=1:3, type="score")
@
\caption{A plot of score 1:3 for PPCA created with \texttt{plotPcs()}
  \label{fig:plotPcs}}
\end{figure}

\subsection{Using ggplot2}

For using ggplot, the scores and loadings should best be added to a data frame that add other relevant descriptive factors. For example, after doing PCA on the Iris dataset, we may add the scores back to the original data frame and use ggplot to visualise, see Figure \ref{fig:ggplot}.

\begin{figure}[!ht]
  \centering
<<fig=true, width=4.5, height=3>>=
pc <- pca(iris)
irdf <- merge(iris, scores(pc), by=0)
library(ggplot2)
ggplot(irdf, aes(PC1, PC2, colour=Species)) +
  geom_point() +
  stat_ellipse()
@ 
\caption{Score plot using ggplot2}
  \label{fig:ggplot}
\end{figure}

\cleardoublepage
\begin{thebibliography}{2006}
\bibitem{stacklies07} Stacklies W., Redestig H., Scholz M., and
  Walther D., and Selbig J. {\sl pcaMethods -- a Bioconductor package
    providing PCA methods for incomplete data}
Bioinformatics. 2007, 23, 1164-1167.
{\sl Non-linear PCA: a missing data approach.}
Bioinformatics. 2005, 21, 3887-3895.
\bibitem{scholz05} Scholz, M. , Kaplan, F., Guy, C.L., Kopka, J. and Selbig, J.
{\sl Non-linear pca: a missing data approach.}
Bioinformatics. 2005, 21, 3887-3895.
\bibitem{troyanskaya01} Troyanskaya O. and Cantor M. and Sherlock G. and Brown
P. and Hastie T. and Tibshirani R. and Botstein D. and Altman RB.
{\sl Missing value estimation methods for DNA microarrays.}
Bioinformatics. 2001 Jun;17(6):520-525.
\bibitem{feten05} Feten G. and Almoy T. and Aastveit A.H.
{\sl Prediction of Missing Values in Microarray and Use of
Mixed Models to Evaluate the Predictors.}, Stat. Appl. Genet. Mol. Biol.
2005;4(1):Article 10
\bibitem{oba03} Oba S. and Sato MA. and Takemasa I. and Monden M. and
Matsubara K. and Ishii S. {\sl A Bayesian missing value estimation method for gene expression profile data.} Bioinformatics. 2003 Nov 1;19(16):2088-96.
\bibitem{wold66} Wold H. {Estimation of principal components and
related models by iterative least squares.} In Multivariate Analysis (Ed. P.R.
Krishnaiah), Academic Press, NY, 391-420.
\bibitem{kim05} Kim H. and Golub G.H. and Park H.
{\sl Missing value estimation for DNA microarray gene expression data: local least
squares imputation}
Bioinformatics. 2005 21(2) :187-198
\end{thebibliography}

\end{document}

\documentclass[a4paper]{article}

%\VignetteIndexEntry{Missing value imputation}

\usepackage{hyperref}

\title{Imputing missing values using the pcaMethods package}
\author{Wolfram Stacklies and Henning Redestig\\
CAS-MPG Partner Institute for Computational Biology (PICB)\\
Shanghai, P.R. China \\
and\\
Max Planck Institute for Molecular Plant Physiology\\
Potsdam, Germany\\
\url{http://bioinformatics.mpimp-golm.mpg.de/}
}
\date{\today}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}
@
\maketitle

\section{Missing value imputation}
One application for missing value robust principal component analysis
is that it effectively can be used to impute the missing values and
thus obtain an estimated complete data set. The pcaMethods package was
partly written with this application in mind.

PCA is a way of creating a model of a matrix, $X$, by defining two
parameter matrices, the scores, $T$, and the loadings, $P$, which
together have less values than the original matrix but when multiplied
with each other well reconstruct the original matrix. I.e.:
$$X=1\times{}\bar{x} + TP' + E$$
where $E$ is the error matrix and $1\times{}\bar{x}$ denotes the
original variable averages. Now if $X$ contains missing values but we
still are able to get complete estimates of $P$ and $T$ than we can
use:
$$\hat{X}=1\times{}\bar{x} + TP'$$
as an estimate for $x_{i,j}$ if $x_{i,j}$ is missing. 

This is can be done as the following example illustrates. First we
attach the metabolite data set with missing values.
<<results=hide>>=
library(pcaMethods)
@
<<>>=
data(metaboliteData)
mD <- metaboliteData
sum(is.na(mD))
@ 
Now we get the estimated data set by using PPCA and three principal components.
<<>>=
pc <- pca(mD, nPcs=3, method="ppca")
imputed <- completeObs(pc)
@ 
If we compare with the original values we see  that the error is
rather low.
<<>>=
data(metaboliteDataComplete)
mdComp <- metaboliteDataComplete
sum((mdComp[is.na(mD)] - imputed[is.na(mD)])^2) / sum(mdComp[is.na(mD)]^2)
@
When using a different PCA algorithm, we get different performance.
<<>>=
imputedNipals <- completeObs(pca(mD, nPcs=3, method="nipals"))
sum((mdComp[is.na(mD)] - imputedNipals[is.na(mD)])^2) / sum(mdComp[is.na(mD)]^2)
@ 

If the data we are interested in was gene expression set of class
'ExpressionSet' we could simply do
<<>>= 
library(Biobase)
data(sample.ExpressionSet)
exSet <- sample.ExpressionSet
exSetNa <- exSet
exprs(exSetNa)[sample(13000, 200)] <- NA
lost <- is.na(exprs(exSetNa))
pc <- pca(exSetNa, nPcs=2, method="ppca")
impExSet <- asExprSet(pc, exSetNa)
sum((exprs(exSet)[lost] - exprs(impExSet)[lost])^2) / sum(exprs(exSet)[lost]^2)
@ 
Different results will be obtained with different PCA
algorithms. Which one to use depends on the general structure of the
data set and the imputation performance can be estimated by
cross-validation. Please see the 'introduction' vignette on further
details on how to use the cross-validation capabilities of this
package.

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robustPca.R
\name{robustSvd}
\alias{robustSvd}
\title{Alternating L1 Singular Value Decomposition}
\usage{
robustSvd(x)
}
\arguments{
\item{x}{A matrix whose SVD decomposition is to be
computed. Missing values are allowed.}
}
\value{
The robust SVD of the matrix is x=u d v'. \item{d}{A
vector containing the singular values of \code{x}.} \item{u}{A
matrix whose columns are the left singular vectors of \code{x}.}
\item{v}{A matrix whose columns are the right singular vectors of
\code{x}.}
}
\description{
A robust approximation to the singular value decomposition of a
rectangular matrix is computed using an alternating L1 norm
(instead of the more usual least squares L2 norm). As the SVD is
a least-squares procedure, it is highly susceptible to outliers
and in the extreme case, an individual cell (if sufficiently
outlying) can draw even the leading principal component toward
itself.
}
\details{
See Hawkins et al (2001) for details on the robust SVD algorithm.
Briefly, the idea is to sequentially estimate the left and right
eigenvectors using an L1 (absolute value) norm minimization.

Note that the robust SVD is able to accomodate missing values in
the matrix \code{x}, unlike the usual \code{svd} function.

Also note that the eigenvectors returned by the robust SVD
algorithm are NOT (in general) orthogonal and the eigenvalues need
not be descending in order.
}
\note{
Two differences from the usual SVD may be noted. One relates
to orthogonality. In the conventional SVD, all the eigenvectors
are orthogonal even if not explicitly imposed.  Those returned by
the AL1 algorithm (used here) are (in general) not orthogonal.
Another difference is that, in the L2 analysis of the conventional
SVD, the successive eigen triples (eigenvalue, left eigenvector,
right eigenvector) are found in descending order of
eigenvalue. This is not necessarily the case with the AL1
algorithm.  Hawkins et al (2001) note that a larger eigen value
may follow a smaller one.
}
\examples{
## Load a complete sample metabolite data set and mean center the data
data(metaboliteDataComplete)
mdc <- prep(metaboliteDataComplete, center=TRUE, scale="none")
## Now create 5\% of outliers.
cond   <- runif(length(mdc)) < 0.05;
mdcOut <- mdc
mdcOut[cond] <- 10
## Now we do a conventional SVD and a robustSvd on both, the original and the 
## data with outliers.
resSvd <- svd(mdc)
resSvdOut <- svd(mdcOut)
resRobSvd <- robustSvd(mdc)
resRobSvdOut <- robustSvd(mdcOut)
## Now we plot the results for the original data against those with outliers
## We can see that robustSvd is hardly affected by the outliers.
plot(resSvd$v[,1], resSvdOut$v[,1])
plot(resRobSvd$v[,1], resRobSvdOut$v[,1])
}
\references{
Hawkins, Douglas M, Li Liu, and S Stanley Young (2001)
Robust Singular Value Decomposition, National Institute of
Statistical Sciences, Technical Report Number
122. \url{http://www.niss.org/technicalreports/tr122.pdf}
}
\seealso{
\code{\link{svd}}, \code{\link[ade4:nipals]{nipals}} for
an alternating L2 norm method that also accommodates missing data.
}
\author{
Kevin Wright, modifications by Wolfram Stacklies
}
\keyword{algebra}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{nni}
\alias{nni}
\title{Nearest neighbour imputation}
\usage{
nni(object, method = c("llsImpute"), subset = numeric(), ...)
}
\arguments{
\item{object}{Numerical matrix with (or an object coercible to
such) with samples in rows and variables as columns. Also takes
\code{ExpressionSet} in which case the transposed expression
matrix is used.}

\item{method}{For convenience one can pass a large matrix but only
use the variable specified as subset. Can be colnames or indices.}

\item{subset}{Currently "llsImpute" only.}

\item{...}{Further arguments to the chosen method.}
}
\value{
A \code{clusterRes} object. Or a list containing a
clusterRes object as first and an ExpressionSet object as second
entry if the input was of type ExpressionSet.
}
\description{
Wrapper function for imputation methods based on nearest neighbour
clustering. Currently llsImpute only.
}
\details{
This method is wrapper function to llsImpute, See documentation
for \code{link{llsImpute}}.
}
\examples{
data(metaboliteData)
llsRes <- nni(metaboliteData, k=6, method="llsImpute", allGenes=TRUE)
}
\seealso{
\code{\link{llsImpute}}, \code{\link{pca}}
}
\author{
Wolfram Stacklies
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{predict-methods}
\alias{predict-methods}
\alias{predict.pcaRes}
\alias{predict,pcaRes-method}
\title{Predict values from PCA.}
\usage{
\method{predict}{pcaRes}(object, newdata, pcs = nP(object), pre = TRUE,
  post = TRUE, ...)

\S4method{predict}{pcaRes}(object, newdata, pcs = nP(object),
  pre = TRUE, post = TRUE, ...)
}
\arguments{
\item{object}{\code{pcaRes} the \code{pcaRes} object of interest.}

\item{newdata}{\code{matrix} new data with same number of columns
as the used to compute \code{object}.}

\item{pcs}{\code{numeric} The number of PC's to consider}

\item{pre}{pre-process \code{newdata} based on the pre-processing
chosen for the PCA model}

\item{post}{unpre-process the final data (add the center back etc)}

\item{...}{Not passed on anywhere, included for S3 consistency.}
}
\value{
A list with the following components: \item{scores}{The
predicted scores} \item{x}{The predicted data}
}
\description{
Predict data using PCA model
}
\details{
This function extracts the predict values from a pcaRes object for
the PCA methods SVD, Nipals, PPCA and BPCA.  Newdata is first
centered if the PCA model was and then scores (\eqn{T}) and data
(\eqn{X}) is 'predicted' according to :
\eqn{\hat{T}=X_{new}P}{That=XnewP}
\eqn{\hat{X}_{new}=\hat{T}P'}{Xhat=ThatP'}.  Missing values are
set to zero before matrix multiplication to achieve NIPALS like
treatment of missing values.
}
\examples{
data(iris)
hidden <- sample(nrow(iris), 50)
pcIr <- pca(iris[-hidden,1:4])
pcFull <- pca(iris[,1:4])
irisHat <- predict(pcIr, iris[hidden,1:4])
cor(irisHat$scores[,1], scores(pcFull)[hidden,1])
}
\author{
Henning Redestig
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{nP,pcaRes-method}
\alias{nP,pcaRes-method}
\alias{nP}
\title{Get number of PCs}
\usage{
nP(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{not used}
}
\value{
Number of PCs
}
\description{
Get number of PCs
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{R2VX,pcaRes-method}
\alias{R2VX,pcaRes-method}
\alias{R2VX}
\title{R2 goodness of fit}
\usage{
\S4method{R2VX}{pcaRes}(object, direction = c("variables",
  "observations", "complete"), data = completeObs(object),
  pcs = nP(object))
}
\arguments{
\item{object}{a PCA model object}

\item{direction}{choose between calculating R2 per variable, per
observation or for the entire data with 'variables',
'observations' or 'complete'.}

\item{data}{the data used to fit the model}

\item{pcs}{the number of PCs to use to calculate R2}
}
\value{
A vector with R2 values
}
\description{
Flexible calculation of R2 goodness of fit.
}
\examples{
R2VX(pca(iris))
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BPCA_dostep.R
\name{BPCA_dostep}
\alias{BPCA_dostep}
\title{Do BPCA estimation step}
\usage{
BPCA_dostep(M, y)
}
\arguments{
\item{M}{Data structure containing all needed information. See the
source documentation of BPCA_initmodel for details}

\item{y}{Numeric original data matrix}
}
\value{
Updated version of the data structure
}
\description{
The function contains the actual implementation of the BPCA
component  estimation. It performs one step of the BPCA EM
algorithm.  It is called 'maxStep' times from within the main loop
in BPCAestimate.
}
\details{
This function is NOT intended to be run standalone.
}
\author{
Wolfram Stacklies
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{wasna,pcaRes-method}
\alias{wasna,pcaRes-method}
\alias{wasna}
\title{Get a matrix with indicating the elements that were missing in the
input data. Convenient for estimating imputation performance.}
\usage{
wasna(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{Not used}
}
\value{
A matrix with logicals
}
\description{
Get a matrix with indicating the elements that were missing in the
input data. Convenient for estimating imputation performance.
}
\examples{
data(metaboliteData)
data(metaboliteDataComplete)
result <- pca(metaboliteData, nPcs=2)
plot(completeObs(result)[wasna(result)], metaboliteDataComplete[wasna(result)])
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-nniRes.R
\name{showNniRes}
\alias{showNniRes}
\title{Print a nniRes model}
\usage{
showNniRes(x, ...)
}
\arguments{
\item{x}{An \code{nniRes} object}

\item{...}{Not used}
}
\value{
Nothing, used for side-effect
}
\description{
Print a brief description of nniRes model
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{center,pcaRes-method}
\alias{center,pcaRes-method}
\alias{center}
\title{Get the centers of the original variables}
\usage{
center(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{Not used}
}
\value{
Vector with the centers
}
\description{
Get the centers of the original variables
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{sDev,pcaRes-method}
\alias{sDev,pcaRes-method}
\alias{sDev}
\title{Get the standard deviations of the scores (indicates their
relevance)}
\usage{
sDev(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{Not used}
}
\value{
Standard devations of the scores
}
\description{
Get the standard deviations of the scores (indicates their
relevance)
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkData.R
\name{checkData}
\alias{checkData}
\title{Do some basic checks on a given data matrix}
\usage{
checkData(data, verbose = FALSE)
}
\arguments{
\item{data}{\code{matrix} -- Data to check.}

\item{verbose}{\code{boolean} -- If TRUE, the function prints
messages whenever an error in the data set is found.}
}
\value{
\item{isValid}{\code{boolean} -- TRUE if no errors were
found, FALSE otherwise.  isValid contains a set of attributes,
these are: \itemize{ \item isNumeric - TRUE if data is numeric,
false otherwise \item isInfinite - TRUE if data contains 'Inf'
values, false otherwise \item isNaN - TRUE if data contains 'NaN'
values, false otherwise \item isMatrix - TRUE if the data is in
matrix format, FALSE otherwise \item naRows - TRUE if data
contains rows in which all elements are 'NA',  FALSE otherwise
\item naCols - TRUE if data contains columns in which all elements
are 'NA',  FALSE otherwise }}
}
\description{
Check a given data matrix for consistency with the format
required for further analysis.
The data must be a numeric matrix and not contain:
\itemize{
\item Inf values
\item NaN values
\item Rows or columns that consist of NA only
}
}
\author{
Wolfram Stacklies
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcaMethods-package.R
\docType{data}
\name{metaboliteData}
\alias{metaboliteData}
\title{A incomplete metabolite data set from an Arabidopsis
coldstress experiment}
\description{
A incomplete subset from a larger metabolite data set.  This is the
original, complete data set and can be used to compare estimation
results created with the also provided incomplete data (called
metaboliteData).
}
\details{
A matrix containing 154 observations (rows) and 52 metabolites
(columns). The data contains 5\% of artificially created uniformly
distributed misssing values. The data was created during an in
house Arabidopsis coldstress experiment.
}
\references{
Matthias Scholz, Fatma Kaplan, Charles L. Guy, Joachim
Kopka and Joachim Selbig. - Non-linear PCA: a missing data
approach.\emph{Bioinformatics 2005 21(20):3887-3895}
}
\seealso{
\code{\link{metaboliteDataComplete}}
}
\author{
Wolfram Stacklies
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{fitted-methods}
\alias{fitted-methods}
\alias{fitted.pcaRes}
\alias{fitted,pcaRes-method}
\title{Extract fitted values from PCA.}
\usage{
\method{fitted}{pcaRes}(object, data = NULL, nPcs = nP(object),
  pre = TRUE, post = TRUE, ...)

\S4method{fitted}{pcaRes}(object, data = NULL, nPcs = nP(object),
  pre = TRUE, post = TRUE, ...)
}
\arguments{
\item{object}{the \code{pcaRes} object of interest.}

\item{data}{For standard PCA methods this can safely be left null
to get scores x loadings but if set, then the scores are obtained
by projecting provided data onto the loadings.  If data contains
missing values the result will be all NA. Non-linear PCA is an
exception, here if data is NULL then data is set to the
completeObs and propaged through the network.}

\item{nPcs}{The number of PC's to consider}

\item{pre}{pre-process \code{data} based on the pre-processing
chosen for the PCA model}

\item{post}{unpre-process the final data (add the center back etc
to get the final estimate)}

\item{...}{Not used}
}
\value{
A matrix representing the fitted data
}
\description{
Fitted values of a PCA model
}
\details{
This function extracts the fitted values from a pcaResobject. For
PCA methods like SVD, Nipals, PPCA etc this is basically just the
scores multipled by the loadings and adjusted for pre-processing.
for non-linear PCA the original data is propagated through the
network to obtain the approximated data.
}
\examples{
pc <- pca(iris[,1:4], nPcs=4, center=TRUE, scale="uv")
sum( (fitted(pc) - iris[,1:4])^2 )
}
\author{
Henning Redestig
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{slplot,pcaRes-method}
\alias{slplot,pcaRes-method}
\alias{slplot}
\title{Side by side scores and loadings plot}
\usage{
slplot(object, pcs=c(1,2), scoresLoadings=c(TRUE, TRUE),
sl="def", ll="def", hotelling=0.95, rug=TRUE, sub=NULL,...)
}
\arguments{
\item{object}{a pcaRes object}

\item{pcs}{which two pcs to plot}

\item{scoresLoadings}{Which should be shown scores and or loadings}

\item{sl}{labels to plot in the scores plot}

\item{ll}{labels to plot in the loadings plot}

\item{hotelling}{confidence interval for ellipse in the score plot}

\item{rug}{logical, rug x axis in score plot or not}

\item{sub}{Subtitle, defaults to annotate with amount of explained
variance.}

\item{...}{Further arguments to plot functions. Prefix arguments
to \code{par()} with 's' for the scores plot and 'l' for the
loadings plot. I.e. cex become scex for setting character
expansion in the score plot and lcex for the loadings plot.}
}
\value{
None, used for side effect.
}
\description{
A common way of visualizing two principal components
}
\details{
This method is meant to be used as a quick way to visualize
results, if you want a more specific plot you probably want to
get the scores, loadings with \code{scores(object)},
\code{loadings(object)} and then design your own plotting method.
}
\note{
Uses layout instead of par to provide side-by-side so it
works with Sweave (but can not be combined with
\code{par(mfrow=..))}
}
\examples{
data(iris)
pcIr <- pca(iris[,1:4], scale="uv")
slplot(pcIr, sl=NULL, spch=5)
slplot(pcIr, sl=NULL, lcex=1.3, scol=as.integer(iris[,5]))
}
\seealso{
\code{\link{pca}}, \code{\link{biplot}}
}
\author{
Henning Redestig
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nlpca.R
\name{getHierarchicIdx}
\alias{getHierarchicIdx}
\title{Index in hiearchy}
\usage{
getHierarchicIdx(hierarchicNum)
}
\arguments{
\item{hierarchicNum}{A number}
}
\value{
...
}
\description{
Index in hiearchy
}
\author{
Henning Redestig, Matthias Scholz
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{show-methods}
\alias{show-methods}
\alias{showPcaRes}
\alias{print,pcaRes-method}
\alias{print,nniRes-method}
\alias{show,pcaRes-method}
\alias{show,nniRes-method}
\title{Print/Show for pcaRes}
\usage{
showPcaRes(x, ...)

\S4method{print}{pcaRes}(x, ...)

\S4method{show}{pcaRes}(object)
}
\arguments{
\item{x}{a pcaRes object}

\item{...}{not used}

\item{object}{the object to print information about}
}
\value{
nothing, used for its side effect
}
\description{
Print basic information about pcaRes object
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robustPca.R
\name{robustPca}
\alias{robustPca}
\title{PCA implementation based on robustSvd}
\usage{
robustPca(Matrix, nPcs = 2, verbose = interactive(), ...)
}
\arguments{
\item{Matrix}{\code{matrix} -- Data containing the variables in
columns and observations in rows. The data may contain missing
values, denoted as \code{NA}.}

\item{nPcs}{\code{numeric} -- Number of components to
estimate. The preciseness of the missing value estimation depends
on the number of components, which should resemble the internal
structure of the data.}

\item{verbose}{\code{boolean} Print some output to the command
line if TRUE}

\item{...}{Reserved for future use. Currently no further
parameters are used}
}
\value{
Standard PCA result object used by all PCA-based methods
of this package. Contains scores, loadings, data mean and
more. See \code{\link{pcaRes}} for details.  are used.
}
\description{
This is a PCA implementation robust to outliers in a data set. It
can also handle missing values, it is however NOT intended to be
used for missing value estimation.  As it is based on robustSVD we
will get an accurate estimation for the loadings also for
incomplete data or for data with outliers.  The returned scores
are, however, affected by the outliers as they are calculated
inputData X loadings. This also implies that you should look at
the returned R2/R2cum values with caution.  If the data show
missing values, scores are caluclated by just setting all NA -
values to zero. This is not expected to produce accurate results.
Please have also a look at the manual page for \code{robustSvd}.
Thus this method should mainly be seen as an attempt to integrate
\code{robustSvd()} into the framework of this package.  Use one of
the other methods coming with this package (like PPCA or BPCA) if
you want to do missing value estimation.  It is not recommended to
use this function directely but rather to use the pca() wrapper
function.
}
\details{
The method is very similar to the standard \code{prcomp()}
function.  The main difference is that \code{robustSvd()} is used
instead of the conventional \code{svd()} method.
}
\examples{
## Load a complete sample metabolite data set and mean center the data
data(metaboliteDataComplete)
mdc <- scale(metaboliteDataComplete, center=TRUE, scale=FALSE)
## Now create 5\\\% of outliers.
cond   <- runif(length(mdc)) < 0.05;
mdcOut <- mdc
mdcOut[cond] <- 10
## Now we do a conventional PCA and robustPca on the original and the data
## with outliers.
## We use center=FALSE here because the large artificial outliers would
## affect the means and not allow to objectively compare the results.
resSvd    <- pca(mdc, method="svd", nPcs=10, center=FALSE)
resSvdOut <- pca(mdcOut, method="svd", nPcs=10, center=FALSE)
resRobPca <- pca(mdcOut, method="robustPca", nPcs=10, center=FALSE)
## Now we plot the results for the original data against those with outliers
## We can see that robustPca is hardly effected by the outliers.
plot(loadings(resSvd)[,1], loadings(resSvdOut)[,1])
plot(loadings(resSvd)[,1], loadings(resRobPca)[,1])
}
\seealso{
\code{\link{robustSvd}, \link{svd}, \link{prcomp},
\link{pcaRes}}.
}
\author{
Wolfram Stacklies
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{listPcaMethods}
\alias{listPcaMethods}
\title{List PCA methods}
\usage{
listPcaMethods(which = c("all", "linear", "nonlinear"))
}
\arguments{
\item{which}{the type of methods to get. E.g. only  get the PCA
methods based on the classical model where the fitted data is a
direct multiplication of scores and loadings.}
}
\value{
A character vector with the current methods for doing PCA
}
\description{
Vector with current valid PCA methods
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{biplot-methods}
\alias{biplot-methods}
\alias{biplot.pcaRes}
\alias{biplot,pcaRes-method}
\title{Plot a overlaid scores and loadings plot}
\usage{
\method{biplot}{pcaRes}(x, choices = 1:2, scale = 1,
  pc.biplot = FALSE, ...)

\S4method{biplot}{pcaRes}(x, choices = 1:2, scale = 1,
  pc.biplot = FALSE, ...)
}
\arguments{
\item{x}{a pcaRes object}

\item{choices}{which two pcs to plot}

\item{scale}{The variables are scaled by
\eqn{\lambda^{scale}}{lambda^scale} and the observations are
scaled    by \eqn{\lambda^{scale}}{lambda ^ (1-scale)} where
\code{lambda} are  the singular values as computed by
\code{princomp}.  Normally  \eqn{0\le{}scale\le{}1}{0 <= scale <=
1}, and a warning will be issued if the specified 'scale' is
outside this range.}

\item{pc.biplot}{If true, use what Gabriel (1971) refers to as a
"principal component biplot", with \eqn{\lambda=1}{lambda = 1} and
observations scaled up by sqrt(n) and variables scaled down by
sqrt(n). Then the inner products between variables approximate
covariances and distances between observations approximate
Mahalanobis distance.}

\item{...}{optional arguments to be passed to
\code{biplot.default}.}
}
\value{
a plot is produced on the current graphics device.
}
\description{
Visualize two-components simultaneously
}
\details{
This is a method for the generic function 'biplot'.  There is
considerable confusion over the precise definitions: those of the
original paper, Gabriel (1971), are followed here.  Gabriel and
Odoroff (1990) use the same definitions, but their plots actually
correspond to \code{pc.biplot = TRUE}.
}
\examples{
data(iris)
pcIr <- pca(iris[,1:4])
biplot(pcIr)
}
\seealso{
\code{prcomp}, \code{pca}, \code{princomp}
}
\author{
Kevin Wright, Adapted from \code{biplot.prcomp}
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\name{weightsAccount}
\alias{weightsAccount}
\title{Create an object that holds the weights for nlpcaNet. Holds and
sets weights in using an environment object.}
\usage{
weightsAccount(w)
}
\arguments{
\item{w}{\code{matrix} -- New weights}
}
\value{
A weightsAccound with \code{set} and \code{current}
functions.
}
\description{
Create an object that holds the weights for nlpcaNet. Holds and
sets weights in using an environment object.
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{svdPca}
\alias{svdPca}
\title{Perform principal component analysis using singular value
decomposition}
\usage{
svdPca(Matrix, nPcs = 2, varLimit = 1, verbose = interactive(), ...)
}
\arguments{
\item{Matrix}{Pre-processed (centered and possibly scaled)
numerical matrix samples in rows and variables as columns. No
missing values allowed.}

\item{nPcs}{Number of components that should be extracted.}

\item{varLimit}{Optionally the ratio of variance that should be
explained. \code{nPcs} is ignored if varLimit < 1}

\item{verbose}{Verbose complaints to matrix structure}

\item{...}{Only used for passing through arguments.}
}
\value{
A \code{pcaRes} object.
}
\description{
A wrapper function for \code{prcomp} to deliver the result as a
\code{pcaRes} method. Supplied for  compatibility with  the rest
of the pcaMethods package.  It is not recommended to use this
function directely but rather to use  the \code{pca()} wrapper
function.
}
\examples{
data(metaboliteDataComplete)
mat <- prep(t(metaboliteDataComplete))
pc <- svdPca(mat, nPcs=2)
## better use pca()
pc <- pca(t(metaboliteDataComplete), method="svd", nPcs=2)
\dontshow{stopifnot(sum((fitted(pc) - t(metaboliteDataComplete))^2, na.rm=TRUE) < 200)}
}
\seealso{
\code{prcomp}, \code{princomp}, \code{pca}
}
\author{
Henning Redestig
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/errorHierarchic.R
\name{errorHierarchic}
\alias{errorHierarchic}
\title{Later}
\usage{
errorHierarchic(nlnet, trainIn, trainOut)
}
\arguments{
\item{nlnet}{The nlnet}

\item{trainIn}{training data}

\item{trainOut}{fitted data}
}
\value{
error
}
\description{
Later
}
\author{
Henning Redestig, Matthias Scholz
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forkNlpcaNet.R
\name{forkNlpcaNet}
\alias{forkNlpcaNet}
\title{Complete copy of nlpca net object}
\usage{
forkNlpcaNet(nlnet)
}
\arguments{
\item{nlnet}{a nlnet}
}
\value{
A copy of the input nlnet
}
\description{
Complete copy of nlpca net object
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sortFeatures.R
\name{sortFeatures}
\alias{sortFeatures}
\title{Sort the features of NLPCA object}
\usage{
sortFeatures(nlnet, trainIn, trainOut)
}
\arguments{
\item{nlnet}{The nlnet}

\item{trainIn}{Training data in}

\item{trainOut}{Training data after it passed through the net}
}
\value{
...
}
\description{
Sort the features of NLPCA object
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\name{plot.pcaRes}
\alias{plot.pcaRes}
\alias{plot,pcaRes-method}
\title{Plot diagnostics (screeplot)}
\usage{
\method{plot}{pcaRes}(x, y = NULL, main = deparse(substitute(object)),
  col = gray(c(0.9, 0.5)), ...)
}
\arguments{
\item{x}{\code{pcaRes} The pcaRes object.}

\item{y}{not used}

\item{main}{title of the plot}

\item{col}{Colors of the bars}

\item{...}{further arguments to barplot}
}
\value{
None, used for side effect.
}
\description{
Plot the computed diagnostics of PCA model to get an idea of their
importance. Note though that the standard screeplot shows the
standard deviations for the PCs this method shows the R2 values
which empirically shows the importance of the P's and is thus
applicable for any PCA method rather than just SVD based PCA.
}
\details{
If cross-validation was done for the PCA the plot will also show
the CV based statistics.  A common rule-of-thumb for determining
the optimal number of PCs is the PC where the CV diagnostic is at
its maximum but not very far from \eqn{R^2}.
}
\examples{
data(metaboliteData)
pc <- pca(t(metaboliteData), nPcs=5, cv="q2", scale="uv")
plot(pc)
}
\seealso{
\link{screeplot}
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{nVar,pcaRes-method}
\alias{nVar,pcaRes-method}
\alias{nVar}
\title{Get the number of variables used to build the PCA model.}
\usage{
nVar(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{Not used}
}
\value{
Number of variables
}
\description{
Get the number of variables used to build the PCA model.
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{plotPcs}
\alias{plotPcs}
\title{Plot many side by side scores XOR loadings plots}
\usage{
plotPcs(object, pcs = 1:nP(object), type = c("scores", "loadings"),
  sl = NULL, hotelling = 0.95, ...)
}
\arguments{
\item{object}{\code{pcaRes} a pcaRes object}

\item{pcs}{\code{numeric} which pcs to plot}

\item{type}{\code{character} Either "scores" or "loadings" for
scores or loadings plot respectively}

\item{sl}{\code{character} Text labels to plot instead of a point,
if NULL points are plotted instead of text}

\item{hotelling}{\code{numeric} Significance level for the
confidence ellipse. NULL means that no ellipse is drawn.}

\item{...}{Further arguments to \code{\link{pairs}} on which this
function is based.}
}
\value{
None, used for side effect.
}
\description{
A function that can be used to visualise many PCs plotted against
each other
}
\details{
Uses \code{\link{pairs}} to provide side-by-side plots. Note that
this function only plots scores or loadings but not both in the
same plot.
}
\examples{
data(iris)
pcIr <- pca(iris[,1:4], nPcs=3,  method="svd")
plotPcs(pcIr, col=as.integer(iris[,4]) + 1)
}
\seealso{
\code{prcomp}, \code{pca}, \code{princomp}, \code{slplot}
}
\author{
Henning Redestig
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{pcaNet}
\alias{pcaNet}
\alias{nlpcaNet}
\alias{nlpcaNet-class}
\title{Class representation of the NLPCA neural net}
\description{
This is a class representation of a non-linear PCA neural
network. The \code{nlpcaNet} class is not meant for user-level
usage.
}
\details{
Creating Objects

\code{new("nlpcaNet", net=[the network structure],
hierarchic=[hierarchic design],
fct=[the functions at each layer], fkt=[the functions used for
forward propagation], weightDecay=[incremental decrease of weight
changes over iterations (between 0 and 1)], featureSorting=[sort
features or not], dataDist=[represents the present values],
inverse=[net is inverse mode or not], fCount=[amount of times
features were sorted], componentLayer=[which layer is the
'bottleneck' (principal components)],
erro=[the used error function], gradient=[the used gradient method],
weights=[the present weights],
maxIter=[the amount of iterations that was done], scalingFactor=[the
scale of the original matrix])}

Slots

\describe{
\item{net}{"matrix",  matrix showing the representation of the
neural network, e.g. (2,4,6) for a network with two features, a
hidden layer and six output neurons (original variables).}
\item{hierarchic}{"list",  the hierarchic design of the network,
holds 'idx' (), 'var' () and layer (which layer is the principal
component layer).}
\item{fct}{"character",  a vector naming the functions that will be
applied on each layer. "linr" is linear (i.e.) standard matrix
products and "tanh" means that the arcus tangens is applied on the
result of the matrix product (for non-linearity).}
\item{fkt}{"character",  same as fct but the functions used during
back propagation.}
\item{weightDecay}{"numeric",  the value that is used to
incrementally decrease the weight changes to ensure convergence.}
\item{featureSorting}{"logical", indicates if features will be
sorted or not. This is used to make the NLPCA assume properties
closer to those of standard PCA were the first component is more
important for reconstructing the data than the second component.}
\item{dataDist}{"matrix", a matrix of ones and zeroes indicating
which values will add to the errror.}
\item{inverse}{"logical", network is inverse mode (currently only
inverse is supported) or not. Eg. the case when we have truly
missing values and wish to impute them.}
\item{fCount}{"integer", Counter for the amount of times features
were really sorted.}
\item{componentLayer}{"numeric", the index of 'net' that is the
component layer.}
\item{error}{"function", the used error function. Currently only one
is provided \code{errorHierarchic}.}
\item{gradient}{"function", the used gradient function. Currently
only one is provided  \code{derrorHierarchic}}
\item{weights}{"list", A list holding managements of the
weights. The list has two functions, weights$current() and
weights$set() which access a matrix in the local environment of
this object.}
\item{maxIter}{"integer", the amount of iterations used to train
this network.}
\item{scalingFactor}{"numeric", training the network is best made
with 'small' values so the original data is scaled down to a
suitable range by division with this number.}}

Methods

\describe{ \item{vector2matrices}{Returns the
weights in a matrix representation.} }
}
\seealso{
\code{\link{nlpca}}
}
\author{
Henning Redestig
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep.R
\name{prep}
\alias{prep}
\title{Pre-process a matrix for PCA}
\usage{
prep(object, scale = c("none", "pareto", "vector", "uv"),
  center = TRUE, eps = 1e-12, simple = TRUE, reverse = FALSE, ...)
}
\arguments{
\item{object}{Numerical matrix (or an object coercible to such)
with samples in rows and variables as columns. Also takes
\code{ExpressionSet} in which case the transposed expression
matrix is used.}

\item{scale}{One of "UV" (unit variance \eqn{a=a/\sigma_{a}})
"vector" (vector normalisation \eqn{b=b/||b||}), "pareto" (sqrt
UV) or "none" to indicate which scaling should be used to scale
the matrix with \eqn{a} variables and \eqn{b} samples. Can also be
a vector of scales which should be used to scale the
matrix. \code{NULL} value is interpreted as \code{"none"}.}

\item{center}{Either a logical which indicates if the matrix
should be mean centred or not, or a vector with averages which
should be suntracted from the matrix. \code{NULL} value is
interpreted as \code{FALSE}}

\item{eps}{Minimum variance, variable with lower variance are not
scaled and warning is issued instead.}

\item{simple}{Logical indicating if only the data should be
returned or a list with the pre-processing statistics as well.}

\item{reverse}{Logical indicating  if matrix should be
'post-processed' instead by multiplying each column with its scale
and adding the center. In this case, center and scale should be
vectors with the statistics (no warning is issued if not, instead
output becomes the same as input).}

\item{...}{Only used for passing through arguments.}
}
\value{
A pre-processed matrix or a list with
\item{center}{a vector with the estimated centers}
\item{scale}{a vector with the estimated scales}
\item{data}{the pre (or post) processed data}
}
\description{
Scaling and centering a matrix.
}
\details{
Does basically the same as \code{\link{scale}} but adds some
alternative scaling options and functionality for treating
pre-processing as part of a model.
}
\examples{
object <- matrix(rnorm(50), nrow=10)
res <- prep(object, scale="uv", center=TRUE, simple=FALSE)
obj <- prep(object, scale=res$scale, center=res$center)
## same as original
sum((object - prep(obj, scale=res$scale, center=res$center, rev=TRUE))^2)
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{pca}
\alias{pca}
\title{Perform principal component analysis}
\usage{
pca(object, method, nPcs = 2, scale = c("none", "pareto", "vector",
  "uv"), center = TRUE, completeObs = TRUE, subset = NULL,
  cv = c("none", "q2"), ...)
}
\arguments{
\item{object}{Numerical matrix with (or an object coercible to
such) with samples in rows and variables as columns. Also takes
\code{ExpressionSet} in which case the transposed expression
matrix is used. Can also be a data frame in which case all
numberic variables are used to fit the PCA.}

\item{method}{One of the methods reported by
\code{listPcaMethods()}. Can be left missing in which case the
\code{svd} PCA is chosen for data wihout missing values and
\code{nipalsPca} for data with missing values}

\item{nPcs}{Number of principal components to calculate.}

\item{scale}{Scaling, see \code{\link{prep}}.}

\item{center}{Centering, see \code{\link{prep}}.}

\item{completeObs}{Sets the \code{completeObs} slot on the
resulting \code{pcaRes} object containing the original data with
but with all NAs replaced with the estimates.}

\item{subset}{A subset of variables to use for calculating the
model. Can be column names or indices.}

\item{cv}{character naming a the type of cross-validation
to be performed.}

\item{...}{Arguments to \code{\link{prep}}, the chosen pca
method and \code{\link{Q2}}.}
}
\value{
A \code{pcaRes} object.
}
\description{
Perform PCA on a numeric matrix for visualisation, information
extraction and missing value imputation.
}
\details{
This method is wrapper function for the following set of pca
methods:

\describe{\item{svd:}{Uses classical \code{prcomp}. See
documentation for \code{\link{svdPca}}.}

\item{nipals:}{An iterative method capable of handling small
amounts of missing values. See documentation for
\code{\link{nipalsPca}}.}

\item{rnipals:}{Same as nipals but implemented in R.}

\item{bpca:}{An iterative method using a Bayesian model to handle
missing values. See documentation for \code{\link{bpca}}.}

\item{ppca:}{An iterative method using a probabilistic model to
handle missing values. See documentation for \code{\link{ppca}}.}

\item{svdImpute:}{Uses expectation maximation to perform SVD PCA
on incomplete data. See documentation for
\code{\link{svdImpute}}.}}

Scaling and centering is part of the PCA model and handled by
\code{\link{prep}}.
}
\examples{
data(iris)
##  Usually some kind of scaling is appropriate
pcIr <- pca(iris, method="svd", nPcs=2)
pcIr <- pca(iris, method="nipals", nPcs=3, cv="q2")
## Get a short summary on the calculated model
summary(pcIr)
plot(pcIr)
## Scores and loadings plot
slplot(pcIr, sl=as.character(iris[,5]))

## use an expressionset and ggplot
data(sample.ExpressionSet)
pc <- pca(sample.ExpressionSet)
df <- merge(scores(pc), pData(sample.ExpressionSet), by=0)
library(ggplot2)
ggplot(df, aes(PC1, PC2, shape=sex, color=type)) +
  geom_point() +
  xlab(paste("PC1", pc@R2[1] * 100, "\% of the variance")) +
  ylab(paste("PC2", pc@R2[2] * 100, "\% of the variance"))
}
\references{
Wold, H. (1966) Estimation of principal components and
related models by iterative least squares. In Multivariate
Analysis (Ed., P.R. Krishnaiah), Academic Press, NY, 391-420.
  
Shigeyuki Oba, Masa-aki Sato, Ichiro Takemasa, Morito Monden,
Ken-ichi Matsubara and Shin Ishii.  A Bayesian missing value
estimation method for gene expression profile
data. \emph{Bioinformatics, 19(16):2088-2096, Nov 2003}.

Troyanskaya O. and Cantor M. and Sherlock G. and Brown P. and
Hastie T. and Tibshirani R. and Botstein D. and Altman RB.  -
Missing value estimation methods for DNA microarrays.
\emph{Bioinformatics. 2001 Jun;17(6):520-5}.
}
\seealso{
\code{\link{prcomp}}, \code{\link{princomp}},
\code{\link{nipalsPca}}, \code{\link{svdPca}}
}
\author{
Wolfram Stacklies, Henning Redestig
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\name{summary}
\alias{summary}
\alias{summary.pcaRes}
\alias{summary,pcaRes-method}
\title{Summary of PCA model}
\usage{
\method{summary}{pcaRes}(object, ...)
}
\arguments{
\item{object}{a pcaRes object}

\item{...}{Not used}
}
\value{
Nothing, used for side-effect
}
\description{
Print a brief description of the PCA model
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector2matrices.R
\docType{methods}
\name{vector2matrices,matrix-method}
\alias{vector2matrices,matrix-method}
\title{Tranform the vectors of weights to matrix structure}
\usage{
\S4method{vector2matrices}{matrix}(object, net)
}
\arguments{
\item{object}{an nlpcaNet}

\item{net}{the neural network}
}
\value{
weights in matrix structure
}
\description{
Tranform the vectors of weights to matrix structure
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppca.R
\name{ppca}
\alias{ppca}
\title{Probabilistic PCA}
\usage{
ppca(Matrix, nPcs = 2, seed = NA, threshold = 1e-05,
  maxIterations = 1000, ...)
}
\arguments{
\item{Matrix}{\code{matrix} -- Data containing the variables in
columns and observations in rows. The data may contain missing
values, denoted as \code{NA}.}

\item{nPcs}{\code{numeric} -- Number of components to
estimate. The preciseness of the missing value estimation depends
on the number of components, which should resemble the internal
structure of the data.}

\item{seed}{\code{numeric} Set the seed for the random number
generator. PPCA creates fills the initial loading matrix with
random numbers chosen from a normal distribution. Thus results may
vary slightly. Set the seed for exact reproduction of your
results.}

\item{threshold}{Convergence threshold.}

\item{maxIterations}{the maximum number of allowed iterations}

\item{...}{Reserved for future use. Currently no further
parameters are used.}
}
\value{
Standard PCA result object used by all PCA-based methods
of this package. Contains scores, loadings, data mean and
more. See \code{\link{pcaRes}} for details.
}
\description{
Implementation of probabilistic PCA (PPCA). PPCA allows to perform
PCA on incomplete data and may be used for missing value
estimation.  This script was implemented after the Matlab version
provided by Jakob Verbeek ( see
\url{http://lear.inrialpes.fr/~verbeek/}) and the draft \emph{``EM
Algorithms for PCA and Sensible PCA''} written by Sam Roweis.
}
\details{
Probabilistic PCA combines an EM approach for PCA with a
probabilistic model. The EM approach is based on the assumption
that the latent variables as well as the noise are normal
distributed.

In standard PCA data which is far from the training set but close
to the principal subspace may have the same reconstruction error.
PPCA defines a likelihood function such that the likelihood for
data far from the training set is much lower, even if they are
close to the principal subspace.  This allows to improve the
estimation accuracy.

A method called \code{kEstimate} is provided to estimate the
optimal number of components via cross validation.  In general few
components are sufficient for reasonable estimation accuracy. See
also the package documentation for further discussion on what kind
of data PCA-based missing value estimation is advisable.

\bold{Complexity:}\cr Runtime is linear in the number of data,
number of data dimensions and number of principal components.

\bold{Convergence:}  The threshold indicating convergence was
changed from 1e-3 in 1.2.x to 1e-5 in the current version  leading
to more stable results.  For reproducability you can set the seed
(parameter seed) of the random number generator. If used for
missing value estimation, results may be checked by simply running
the algorithm several times with changing seed, if the estimated
values show little variance the algorithm converged well.
}
\note{
Requires \code{MASS}. It is not recommended to use this
function directely but rather to use the pca() wrapper function.
}
\examples{
## Load a sample metabolite dataset with 5\\\% missing values (metaboliteData)
data(metaboliteData)
## Perform probabilistic PCA using the 3 largest components
result <- pca(t(metaboliteData), method="ppca", nPcs=3, seed=123)
## Get the estimated complete observations
cObs <- completeObs(result)
## Plot the scores
plotPcs(result, type = "scores")
\dontshow{
  stopifnot(sum((fitted(result) - t(metaboliteData))^2, na.rm=TRUE) < 200)
}
}
\seealso{
\code{\link{bpca}, \link{svdImpute}, \link{prcomp},
\link{nipalsPca}, \link{pca}, \link{pcaRes}}.
}
\author{
Wolfram Stacklies
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{loadings,ANY-method}
\alias{loadings,ANY-method}
\alias{loadings}
\title{Crude way to unmask the function with the same name from
\code{stats}}
\usage{
\S4method{loadings}{ANY}(object, ...)
}
\arguments{
\item{object}{any object}

\item{...}{not used}
}
\value{
The loadings
}
\description{
Crude way to unmask the function with the same name from
\code{stats}
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/svdImpute.R
\name{svdImpute}
\alias{svdImpute}
\title{SVDimpute algorithm}
\usage{
svdImpute(Matrix, nPcs = 2, threshold = 0.01, maxSteps = 100,
  verbose = interactive(), ...)
}
\arguments{
\item{Matrix}{\code{matrix} -- Pre-processed (centered, scaled)
data with variables in columns and observations in rows. The data
may contain missing values, denoted as \code{NA}.}

\item{nPcs}{\code{numeric} -- Number of components to
estimate. The preciseness of the missing value estimation depends
on the number of components, which should resemble the internal
structure of the data.}

\item{threshold}{The iteration stops if the change in the matrix
falls below this threshold.}

\item{maxSteps}{Maximum number of iteration steps.}

\item{verbose}{Print some output if TRUE.}

\item{...}{Reserved for parameters used in future version of the
algorithm}
}
\value{
Standard PCA result object used by all PCA-based methods
of this package. Contains scores, loadings, data mean and
more. See \code{\link{pcaRes}} for details.
}
\description{
This implements the SVDimpute algorithm as proposed by Troyanskaya
et al, 2001.  The idea behind the algorithm is to estimate the
missing values as a linear combination of the \code{k} most
significant eigengenes.
}
\details{
Missing values are denoted as \code{NA}. It is not recommended
to use this function directely but rather to use the pca() wrapper
function.

As SVD can only be performed on complete matrices, all missing
values are initially replaced by 0 (what is in fact the mean on
centred data).  The algorithm works iteratively until the change
in the estimated solution falls below a certain threshold.  Each
step the eigengenes of the current estimate are calculated and
used to determine a new estimate. Eigengenes denote the loadings
if pca is performed considering variable (for Microarray data
genes) as observations.

An optimal linear combination is found by regressing the
incomplete variable against the \code{k} most significant
eigengenes. If the value at position \code{j} is missing, the
\eqn{j^th}{j^th} value of the eigengenes is not used when
determining the regression coefficients.
}
\note{
Each iteration, standard PCA (\code{prcomp}) needs to be
done for each incomplete variable to get the eigengenes. This is
usually fast for small data sets, but complexity may rise if the
data sets become very large.
}
\examples{
## Load a sample metabolite dataset with 5\\\% missing values
data(metaboliteData)
## Perform svdImpute using the 3 largest components
result <- pca(metaboliteData, method="svdImpute", nPcs=3, center = TRUE)
## Get the estimated complete observations
cObs <- completeObs(result)
## Now plot the scores
plotPcs(result, type = "scores")
}
\references{
Troyanskaya O. and Cantor M. and Sherlock G. and Brown
P. and Hastie T. and Tibshirani R. and Botstein D. and Altman
RB. - Missing value estimation methods for DNA
microarrays. \emph{Bioinformatics. 2001 Jun;17(6):520-5.}
}
\author{
Wolfram Stacklies
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kEstimate.R
\name{kEstimate}
\alias{kEstimate}
\title{Estimate best number of Components for missing value
estimation}
\usage{
kEstimate(Matrix, method = "ppca", evalPcs = 1:3, segs = 3,
  nruncv = 5, em = "q2", allVariables = FALSE,
  verbose = interactive(), ...)
}
\arguments{
\item{Matrix}{\code{matrix} -- numeric matrix containing
observations in rows and variables in columns}

\item{method}{\code{character} -- of the methods found with
pcaMethods() The option llsImputeAll calls llsImpute with the
allVariables = TRUE parameter.}

\item{evalPcs}{\code{numeric} -- The principal components to use
for cross validation or the number of neighbour variables if used
with llsImpute.  Should be an array containing integer values,
eg. \code{evalPcs = 1:10} or \code{evalPcs = c(2,5,8)}. The NRMSEP
or Q2 is calculated for each component.}

\item{segs}{\code{numeric} -- number of segments for cross validation}

\item{nruncv}{\code{numeric} -- Times the whole cross validation
is repeated}

\item{em}{\code{character} -- The error measure. This can be nrmsep or q2}

\item{allVariables}{\code{boolean} -- If TRUE, the NRMSEP is
calculated for all variables, If FALSE, only the incomplete ones
are included. You maybe want to do this to compare several methods
on a  complete data set.}

\item{verbose}{\code{boolean} -- If TRUE, some output like the
variable indexes are printed to the console each iteration.}

\item{...}{Further arguments to \code{pca} or \code{nni}}
}
\value{
A list with:
\item{bestNPcs}{number of PCs or k for which the minimal average
NRMSEP or the maximal Q2 was obtained.}
\item{eError}{an array of of size length(evalPcs). Contains the
average error of the cross validation runs for each number of
components.}
\item{variableWiseError}{Matrix of size
\code{incomplete_variables} x length(evalPcs).  Contains the
NRMSEP or Q2 distance for each variable and each number of PCs.
This allows to easily see for wich variables imputation makes
sense and for which one it should not be done or mean imputation
should be used.}
\item{evalPcs}{The evaluated numbers of components or number of
neighbours  (the same as the evalPcs input parameter).}
\item{variableIx}{Index of the incomplete variables. This can be
used to map  the variable wise error to the original data.}
}
\description{
Perform cross validation to estimate the optimal number of
components for missing value estimation. Cross validation is
done for the complete subset of a variable.
}
\details{
The assumption hereby is that variables that are highly correlated
in a distinct region (here the non-missing observations) are also
correlated in another (here the missing observations).  This also
implies that the complete subset must be large enough to be
representative.  For each incomplete variable, the available
values are divided into a user defined number of cv-segments. The
segments have equal size, but are chosen from a random equal
distribution. The non-missing values of the variable are covered
completely.  PPCA, BPCA, SVDimpute, Nipals PCA, llsImpute an NLPCA
may be used for imputation.

The whole cross validation is repeated several times so, depending
on the parameters, the calculations can take very long time.  As
error measure the NRMSEP (see Feten et. al, 2005) or the Q2
distance is used.  The NRMSEP basically normalises the RMSD
between original data and estimate by the variable-wise
variance. The reason for this is that a higher variance will
generally lead to a higher estimation error.  If the number of
samples is small, the variable - wise variance may become an
unstable criterion and the Q2 distance should be used
instead. Also if variance normalisation was applied previously.

The method proceeds variable - wise, the NRMSEP / Q2 distance is
calculated for each incomplete variable and averaged
afterwards. This allows to easily see for wich set of variables
missing value imputation makes senes and for wich set no
imputation or something like mean-imputation should be used.  Use
\code{kEstimateFast} or \code{Q2} if you are not interested in
variable wise CV performance estimates.

Run time may be very high on large data sets. Especially when used
with complex methods like BPCA or Nipals PCA.  For PPCA, BPCA,
Nipals PCA and NLPCA the estimation method is called
\eqn{(v_{miss} \cdot segs \cdot nruncv \cdot)}{(v\_miss * segs *
nruncv)} times as the error for all numbers of principal
components can be calculated at once.  For LLSimpute and SVDimpute
this is not possible, and the method is called \eqn{(v_{miss}
\cdot segs \cdot nruncv \cdot length(evalPcs))}{(v\_miss * segs *
nruncv * length(evalPcs))} times. This should still be fast for
LLSimpute because the method allows to choose to only do the
estimation for one particular variable.  This saves a lot of
iterations.  Here, \eqn{v_{miss}}{v\_miss} is the number of
variables showing missing values.

As cross validation is done variable-wise, in this function Q2 is
defined on single variables, not on the entire data set. This is
Q2 is calculated as as \eqn{\frac{\sum(x -
xe)^2}{\sum(x^2)}}{sum(x - xe)^2 \ sum(x^2)}, where x is the
currently used variable and xe it's estimate. The values are then
averaged over all variables. The NRMSEP is already defined
variable-wise. For a single variable it is then
\eqn{\sqrt(\frac{\sum(x - xe)^2}{(n \cdot var(x))})}{sqrt(sum(x -
xe)^2 \ (n * var(x)))}, where x is the variable and xe it's
estimate, n is the length of x.  The variable wise estimation
errors are returned in parameter variableWiseError.
}
\examples{
## Load a sample metabolite dataset with 5\\\% missing values (metaboliteData)
data(metaboliteData)
# Do cross validation with ppca for component 2:4
esti <- kEstimate(metaboliteData, method = "ppca", evalPcs = 2:4, nruncv=1, em="nrmsep")
# Plot the average NRMSEP
barplot(drop(esti$eError), xlab = "Components",ylab = "NRMSEP (1 iterations)")
# The best result was obtained for this number of PCs:
print(esti$bestNPcs)
# Now have a look at the variable wise estimation error
barplot(drop(esti$variableWiseError[, which(esti$evalPcs == esti$bestNPcs)]), 
        xlab = "Incomplete variable Index", ylab = "NRMSEP")
}
\seealso{
\code{\link{kEstimateFast}, \link{Q2}, \link{pca}, \link{nni}}.
}
\author{
Wolfram Stacklies
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{pcaRes}
\alias{pcaRes}
\alias{pcaRes-class}
\title{Class for representing a PCA result}
\description{
This is a class representation of a PCA result
}
\details{
\bold{Creating Objects}\cr
\code{new("pcaRes", scores=[the scores], loadings=[the loadings],
nPcs=[amount of PCs], R2cum=[cumulative R2], nObs=[amount of
observations], nVar=[amount of variables], R2=[R2 for each
individual PC], sDev=[stdev for each individual PC],
centered=[was data centered], center=[original means],
varLimit=[what variance limit was exceeded], method=[method used to
calculate PCA], missing=[amount of NAs], 
completeObs=[estimated complete observations])}

\bold{Slots}\cr
\describe{
    \item{scores}{"matrix",  the calculated scores}
    \item{loadings}{"matrix",  the calculated loadings}
    \item{R2cum}{"numeric",  the cumulative R2 values}
    \item{sDev}{"numeric",  the individual standard
      deviations of the score vectors}
    \item{R2}{"numeric",  the individual R2 values}
    \item{cvstat}{"numeric",  cross-validation statistics}
    \item{nObs}{"numeric", number of observations}
    \item{nVar}{"numeric", number of variables}
    \item{centered}{"logical", data was centered or not}
    \item{center}{"numeric", the original variable centers}
    \item{scaled}{"logical", data was scaled or not}
    \item{scl}{"numeric", the original variable scales}
    \item{varLimit}{"numeric", the exceeded variance limit}
    \item{nPcs,nP}{"numeric", the number of calculated PCs}
    \item{method}{"character", the method used to perform PCA}
    \item{missing}{"numeric", the total amount of missing values in
      original data}
    \item{completeObs}{"matrix", the estimated complete observations}
    \item{network}{"nlpcaNet", the network used by non-linear PCA}
  }

\bold{Methods (not necessarily exhaustive)}\cr
\describe{
    \item{print}{Print function}
    \item{summary}{Extract information about PC relevance}
    \item{screeplot}{Plot a barplot of standard deviations for PCs}
    \item{slplot}{Make a side by side score and loadings plot}
    \item{nPcs}{Get the number of PCs}
    \item{nObs}{Get the number of observations}
    \item{cvstat}{Cross-validation statistics}
    \item{nVar}{Get the number of variables}
    \item{loadings}{Get the loadings}
    \item{scores}{Get the scores}
    \item{dim}{Get the dimensions (number of observations, number of
      features)}
    \item{centered}{Get a logical indicating if centering was done as
      part of the model}
    \item{center}{Get the averages of the original variables.}
    \item{completeObs}{Get the imputed data set}
    \item{method}{Get a string naming the used PCA method}
    \item{sDev}{Get the standard deviations of the PCs}
    \item{scaled}{Get a logical indicating if scaling was done as
      part of the model}
    \item{scl}{Get the scales of the original variablesb}
    \item{R2cum}{Get the cumulative R2}
  }
}
\author{
Henning Redestig
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{centered,pcaRes-method}
\alias{centered,pcaRes-method}
\alias{centered}
\title{Check centering was part of the model}
\usage{
centered(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{Not used}
}
\value{
TRUE if model was centered
}
\description{
Check centering was part of the model
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{nniRes}
\alias{nniRes}
\alias{nniRes-class}
\title{Class for representing a nearest neighbour imputation result}
\description{
This is a class representation of nearest neighbour imputation
(nni) result
}
\details{
\bold{Creating Objects}\cr
\code{new("nniRes", completeObs=[the estimated complete
observations], k=[cluster size], nObs=[amount of observations],
nVar=[amount of variables], centered=[was the data centered befor
running LLSimpute], center=[original means], method=[method used
to perform clustering], missing=[amount of NAs])}

\bold{Slots}\cr
\describe{
  \item{completeObs}{"matrix", the estimated complete observations}
  \item{nObs}{"numeric", amount of observations}
  \item{nVar}{"numeric", amount of variables}
  \item{correlation}{"character", the correlation method used
        (pearson, kendall or spearman)}
  \item{centered}{"logical", data was centered or not}
  \item{center}{"numeric", the original variable centers}
  \item{k}{"numeric", cluster size}
  \item{method}{"character", the method used to perform the clustering}
  \item{missing}{"numeric", the total amount of missing values in
    original data}
}

\bold{Methods}\cr
\describe{ \item{print}{Print function} }
}
\author{
Wolfram Stacklies
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{method,pcaRes-method}
\alias{method,pcaRes-method}
\alias{method}
\title{Get the used PCA method}
\usage{
method(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{Not used}
}
\value{
The used pca method
}
\description{
Get the used PCA method
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcaMethods-package.R
\docType{package}
\name{pcaMethods}
\alias{pcaMethods}
\alias{pcaMethods-package}
\title{pcaMethods}
\description{
Principal Component Analysis in R
}
\details{
\tabular{ll}{
Package: \tab pcaMethods \cr
Type: \tab Package \cr
Developed since: \tab 2006 \cr
License: \tab GPL (>=3) \cr
LazyLoad: \tab yes \cr
}

Provides Bayesian PCA, Probabilistic PCA, Nipals PCA, Inverse
Non-Linear PCA and the conventional SVD PCA. A cluster  based
method for missing value estimation is included for comparison.
BPCA, PPCA and NipalsPCA may be used to perform PCA on incomplete
data as well as for accurate missing value estimation.  A set of
methods for printing and plotting the results is also provided.
All PCA methods make use of the same data structure (pcaRes) to
provide a unique interface to the PCA results. Developed at the
Max-Planck Institute for Molecular Plant Physiology, Golm,
Germany, RIKEN Plant Science Center Yokohama, Japan, and CAS-MPG
Partner Institute for Computational Biology (PICB) Shanghai,
P.R. China
}
\author{
Wolfram Stacklies, Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{scaled,pcaRes-method}
\alias{scaled,pcaRes-method}
\alias{scaled}
\title{Check if scaling was part of the PCA model}
\usage{
scaled(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{Not used}
}
\value{
TRUE if scaling was part of the PCA model
}
\description{
Check if scaling was part of the PCA model
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kEstimateFast.R
\name{kEstimateFast}
\alias{kEstimateFast}
\title{Estimate best number of Components for missing value estimation}
\usage{
kEstimateFast(Matrix, method = "ppca", evalPcs = 1:3, em = "nrmsep",
  allVariables = FALSE, verbose = interactive(), ...)
}
\arguments{
\item{Matrix}{\code{matrix} -- numeric matrix containing
observations in rows and variables in columns}

\item{method}{\code{character} -- a valid pca method (see
\code{\link{pca}}).}

\item{evalPcs}{\code{numeric} -- The principal components to use
for cross validation or cluster sizes if used with
llsImpute. Should be an array containing integer values,
eg. evalPcs = 1:10 or evalPcs = C(2,5,8).The NRMSEP is calculated
for each component.}

\item{em}{\code{character} -- The error measure. This can be
nrmsep or q2}

\item{allVariables}{\code{boolean} -- If TRUE, the NRMSEP is
calculated for all variables, If FALSE, only the incomplete ones
are included. You maybe want to do this to compare several methods
on a  complete data set.}

\item{verbose}{\code{boolean} -- If TRUE, the NRMSEP and the
variance are printed to the console each iteration.}

\item{...}{Further arguments to \code{pca}}
}
\value{
\item{list}{Returns a list with the elements:
\itemize{
\item minNPcs - number of PCs for which the minimal average NRMSEP
was obtained
\item eError - an array of of size length(evalPcs). Contains the
estimation error for each number of
components.
\item evalPcs - The evaluated numbers of components or
cluster sizes  (the same as the evalPcs input parameter). }}
}
\description{
This is a simple estimator for the optimal number of componets
when applying PCA or LLSimpute for missing value estimation.  No
cross validation is performed, instead the estimation quality is
defined as Matrix[!missing] - Estimate[!missing]. This will give a
relatively rough estimate, but the number of iterations equals the
length of the parameter evalPcs.\cr Does not work with LLSimpute!!
As error measure the NRMSEP (see Feten et. al, 2005) or the Q2
distance is used.  The NRMSEP basically normalises the RMSD
between original data and estimate by the variable-wise
variance. The reason for this is that a higher variance will
generally lead to a higher estimation error.  If the number of
samples is small, the gene - wise variance may become an unstable
criterion and the Q2 distance should be used instead. Also if
variance normalisation was applied previously.
}
\examples{
data(metaboliteData)
# Estimate best number of PCs with ppca for component 2:4
esti <- kEstimateFast(t(metaboliteData), method = "ppca", evalPcs = 2:4, em="nrmsep")
barplot(drop(esti$eError), xlab = "Components",ylab = "NRMSEP (1 iterations)")
# The best k value is:
print(esti$minNPcs)
}
\seealso{
\code{\link{kEstimate}}.
}
\author{
Wolfram Stacklies
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{DModX,pcaRes-method}
\alias{DModX,pcaRes-method}
\alias{DModX}
\title{DModX}
\usage{
DModX(object, dat, newdata=FALSE, type=c("normalized","absolute"), ...)
}
\arguments{
\item{object}{a pcaRes object}

\item{dat}{the original data, taken from \code{completeObs} if
left missing.}

\item{newdata}{logical indicating if this data was part of the
training data or not. If it was, it is adjusted by a near one factor
\eqn{v=(N/ (N-A-A0))^-1}}

\item{type}{if absolute or normalized values should be
given. Normalized values are adjusted to the the total RSD of the
model.}

\item{...}{Not used}
}
\value{
A vector with distances from observations to the PCA model
}
\description{
Distance to the model of X-space.
}
\details{
Measures how well described the observations are, i.e. how well
they fit in the mode. High DModX indicate a poor fit. Defined as:

\eqn{\frac{\sqrt{\frac{SSE_i}{K-A}}}{\sqrt{\frac{SSE}{(N-A-A_0)(K-A)}}}}

For observation \eqn{i}, in a model with \eqn{A} components,
\eqn{K} variables and \eqn{N} obserations. SSE is the squared sum
of the residuals. \eqn{A_0} is 1 if model was centered and 0
otherwise. DModX is claimed to be approximately F-distributed and
can therefore be used to check if an observation is significantly
far away from the PCA model assuming normally distributed data.

Pass original data as an argument if the model was calculated with
\code{completeObs=FALSE}.
}
\examples{
data(iris)
pcIr <- pca(iris[,1:4])
with(iris, plot(DModX(pcIr)~Species))
}
\references{
Introduction to Multi- and Megavariate Data Analysis
using Projection Methods (PCA and PLS), L. Eriksson, E. Johansson,
N. Kettaneh-Wold and S. Wold, Umetrics 1999, p. 468
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bpca.R
\name{bpca}
\alias{bpca}
\title{Bayesian PCA missing value estimation}
\usage{
bpca(Matrix, nPcs = 2, maxSteps = 100, verbose = interactive(),
  threshold = 1e-04, ...)
}
\arguments{
\item{Matrix}{\code{matrix} -- Pre-processed matrix (centered,
scaled) with variables in columns and observations in rows. The
data may contain missing values, denoted as \code{NA}.}

\item{nPcs}{\code{numeric} -- Number of components used for
re-estimation. Choosing few components may decrease the
estimation precision.}

\item{maxSteps}{\code{numeric} -- Maximum number of estimation
steps.}

\item{verbose}{\code{boolean} -- BPCA prints the number of steps
and the increase in precision if set to TRUE. Default is
interactive().}

\item{threshold}{convergence threshold}

\item{...}{Reserved for future use. Currently no further
parameters are used}
}
\value{
Standard PCA result object used by all PCA-based methods
  of this package. Contains scores, loadings, data mean and
  more. See \code{\link{pcaRes}} for details.
}
\description{
Implements a Bayesian PCA missing value estimator.  The script
is a port of the Matlab version provided by Shigeyuki OBA.  See
also \url{http://ishiilab.jp/member/oba/tools/BPCAFill.html}.
BPCA combines an EM approach for PCA with a Bayesian model.  In
standard PCA data far from the training set but close to the
principal subspace may have the same reconstruction error.  BPCA
defines a likelihood function such that the likelihood for data
far from the training set is much lower, even if they are close to
the principal subspace.
}
\details{
Scores and loadings obtained with Bayesian PCA slightly differ
from those obtained with conventional PCA.  This is because BPCA
was developed especially for missing value estimation.  The
algorithm does not force orthogonality between factor loadings, as
a result factor loadings are not necessarily orthogonal.  However,
the BPCA authors found that including an orthogonality criterion
made the predictions worse.

The authors also state that the difference between real and
predicted Eigenvalues becomes larger when the number of
observation is smaller, because it reflects the lack of
information to accurately determine true factor loadings from the
limited and noisy data.  As a result, weights of factors to
predict missing values are not the same as with conventional PCA,
but the missing value estimation is improved.

BPCA works iteratively, the complexity is growing with
\eqn{O(n^3)}{O(n^3)} because several matrix inversions are
required.  The size of the matrices to invert depends on the
number of components used for re-estimation.

Finding the optimal number of components for estimation is not a
trivial task; the best choice depends on the internal structure of
the data.  A method called \code{kEstimate} is provided to
estimate the optimal number of components via cross validation.
In general few components are sufficient for reasonable estimation
accuracy. See also the package documentation for further
discussion about on what data PCA-based missing value estimation
makes sense.

It is not recommended to use this function directely but rather to
use the pca() wrapper function.

There is a difference with respect the interpretation of rows
(observations) and columns (variables) compared to matlab
implementation. For estimation of missing values for microarray
data, the suggestion in the original bpca is to intepret genes as
observations and the samples as variables. In pcaMethods however,
genes are interpreted as variables and samples as observations
which arguably also is the more natural interpretation. For bpca
behavior like in the matlab implementation, simply transpose your
input matrix.

Details about the probabilistic model underlying BPCA are found in
Oba et. al 2003. The algorithm uses an expectation maximation
approach together with a Bayesian model to approximate the
principal axes (eigenvectors of the covariance matrix in PCA).
The estimation is done iteratively, the algorithm terminates if
either the maximum number of iterations was reached or if the
estimated increase in precision falls below \eqn{1e^{-4}}{1e^-4}.

\bold{Complexity:} The relatively high complexity of the method is
a result of several matrix inversions required in each step.
Considering the case that the maximum number of iteration steps is
needed, the approximate complexity is given by the term
\deqn{maxSteps \cdot row_{miss} \cdot O(n^3)}{maxSteps * row_miss
* O(n^3)} Where \eqn{row_{miss}}{row_miss} is the number of rows
containing missing values and \eqn{O(n^3)}{O(n^3)} is the
complexity for inverting a matrix of size
\eqn{components}{components}. Components is the number of
components used for re-estimation.
}
\note{
Requires \code{MASS}.
}
\examples{
## Load a sample metabolite dataset with 5\\\% missig values (metaboliteData)e
data(metaboliteData)
## Perform Bayesian PCA with 2 components
pc <- pca(t(metaboliteData), method="bpca", nPcs=2)
## Get the estimated principal axes (loadings)
loadings <- loadings(pc)
## Get the estimated scores
scores <- scores(pc)
## Get the estimated complete observations
cObs <- completeObs(pc)
## Now make a scores and loadings plot
slplot(pc)
\dontshow{stopifnot(sum((fitted(pc) - t(metaboliteData))^2, na.rm=TRUE) < 200)}
}
\references{
Shigeyuki Oba, Masa-aki Sato, Ichiro Takemasa, Morito
  Monden, Ken-ichi Matsubara and Shin Ishii.  A Bayesian missing
  value estimation method for gene expression profile
  data. \emph{Bioinformatics, 19(16):2088-2096, Nov 2003}.
}
\seealso{
\code{\link{ppca}}, \code{\link{svdImpute}},
  \code{\link{prcomp}}, \code{\link{nipalsPca}},
  \code{\link{pca}},
  \code{\link{pcaRes}}. \code{\link{kEstimate}}.
}
\author{
Wolfram Stacklies
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BPCA_initmodel.R
\name{BPCA_initmodel}
\alias{BPCA_initmodel}
\title{Initialize BPCA model}
\usage{
BPCA_initmodel(y, components)
}
\arguments{
\item{y}{numeric matrix containing missing values. Missing values
are denoted as 'NA'}

\item{components}{Number of components used for estimation}
}
\value{
List containing
\item{rows}{Row number of input matrix}
\item{cols}{Column number of input matrix}
\item{comps}{Number of components to use}
\item{yest}{(working variable) current estimate of complete data}
\item{row_miss}{(Array) Indizes of rows containing missing values}
\item{row_nomiss}{(Array) Indices of complete rows (such with no
missing values)}
\item{nans}{Matrix of same size as input data. TRUE if \code{input == NA},
false otherwise}
\item{mean}{Column wise data mean}
\item{PA}{ (d x k) Estimated principal axes (eigenvectors,
loadings) The matrix ROWS are the vectors}
\item{tau}{Estimated precision of the residual error}
\item{scores}{ Estimated scores}
Further elements are: galpha0, balpha0, alpha, gmu0, btau0, gtau0,
SigW. These are working variables or constants.
}
\description{
Model initialization for Bayesian PCA. This function is NOT
inteded to be run separately!
}
\details{
The function calculates the initial Eigenvectors by use of SVD
from the complete rows.  The data structure M is created and
initial values are  assigned.
}
\author{
Wolfram Stacklies
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xval.R
\name{cvseg}
\alias{cvseg}
\title{Get CV segments}
\usage{
cvseg(x, fold = 7, seed = NULL)
}
\arguments{
\item{x}{a factor, character or numeric vector that describes
class membership of a set of items, or, a numeric vector
indicating unique indices of items, or, a numeric of length 1 that
describes the number of items to segment (without any classes)}

\item{fold}{the desired number of segments}

\item{seed}{randomization seed for reproducibility}
}
\value{
a list where each element is a set of indices that defines
the CV segment.
}
\description{
Get cross-validation segments that have (as far as possible) the
same ratio of all classes (if classes are present)
}
\examples{
seg <- cvseg(iris$Species, 10)
sapply(seg, function(s) table(iris$Species[s]))
cvseg(20, 10)
}
\seealso{
the \code{cvsegments} function in the \code{pls} package
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-ExpressionSet.R
\name{asExprSet}
\alias{asExprSet}
\title{Convert pcaRes object to an expression set}
\usage{
asExprSet(object, exprSet)
}
\arguments{
\item{object}{\code{pcaRes} -- The object containing the completed
data.}

\item{exprSet}{\code{ExpressionSet} -- The object passed on to
\code{pca} for missing value estimation.}
}
\value{
An object without missing values of class \code{ExpressionSet}.
}
\description{
This function can be used to conveniently replace the expression
matrix in an \code{ExpressionSet} with the completed data from a
\code{pcaRes} object.
}
\details{
This is not a standard \code{as} function as \code{pcaRes}
object alone not can be converted to an \code{ExpressionSet} (the
\code{pcaRes} object does not hold any \code{phenoData} for
example).
}
\author{
Wolfram Stacklies \cr CAS-MPG Partner Institute for
Computational Biology, Shanghai, China
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{cvstat,pcaRes-method}
\alias{cvstat,pcaRes-method}
\alias{cvstat}
\title{Get cross-validation statistics (e.g. \eqn{Q^2}).}
\usage{
cvstat(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{not used}
}
\value{
vector CV statistics
}
\description{
Get cross-validation statistics (e.g. \eqn{Q^2}).
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\name{scores.pcaRes}
\alias{scores.pcaRes}
\title{Get scores from a pcaRes object}
\usage{
\method{scores}{pcaRes}(object, ...)
}
\arguments{
\item{object}{a pcaRes object}

\item{...}{not used}
}
\value{
The scores as a matrix
}
\description{
Get scores from a pcaRes object
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{R2cum,pcaRes-method}
\alias{R2cum,pcaRes-method}
\alias{R2cum}
\title{Cumulative R2 is the total ratio of variance that is being
explained by the model}
\usage{
\S4method{R2cum}{pcaRes}(object, ...)
}
\arguments{
\item{object}{a \code{pcaRes} model}

\item{...}{Not used}
}
\value{
Get the cumulative R2
}
\description{
Cumulative R2 is the total ratio of variance that is being
explained by the model
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/llsImpute.R
\name{llsImpute}
\alias{llsImpute}
\title{LLSimpute algorithm}
\usage{
llsImpute(Matrix, k = 10, center = FALSE, completeObs = TRUE,
  correlation = "pearson", allVariables = FALSE, maxSteps = 100,
  xval = NULL, verbose = FALSE, ...)
}
\arguments{
\item{Matrix}{\code{matrix} -- Data containing the variables
(genes) in columns and observations (samples) in rows. The data
may contain missing values, denoted as \code{NA}.}

\item{k}{\code{numeric} -- Cluster size, this is the number of
similar genes used for regression.}

\item{center}{\code{boolean} -- Mean center the data if TRUE}

\item{completeObs}{\code{boolean} -- Return the estimated complete
observations if  TRUE. This is the input data with NA values
replaced by the estimated values.}

\item{correlation}{\code{character} -- How to calculate the
distance between genes.  One out of pearson | kendall | spearman ,
see also help("cor").}

\item{allVariables}{\code{boolean} -- Use only complete genes to
do the regression if TRUE, all genes if FALSE.}

\item{maxSteps}{\code{numeric} -- Maximum number of iteration
steps if allGenes = TRUE.}

\item{xval}{\code{numeric} Use LLSimpute for cross
validation. xval is the index of the gene to estimate, all other
incomplete genes will be ignored if this parameter is set. We do
not consider them in the cross-validation.}

\item{verbose}{\code{boolean} -- Print step number and relative
change if TRUE and  allVariables = TRUE}

\item{...}{Reserved for parameters used in future version of the
algorithm}
}
\value{
\item{nniRes}{Standard nni (nearest neighbour
imputation) result object of this package. See
\code{\link{nniRes}} for details.}
}
\description{
Missing value estimation using local least squares (LLS).  First,
k variables (for Microarrya data usually the genes)  are selected
by pearson, spearman or kendall correlation coefficients.  Then
missing values are imputed by a linear combination of the k
selected variables. The optimal combination is found by LLS
regression.  The method was first described by Kim et al,
Bioinformatics, 21(2),2005.
}
\details{
Missing values are denoted as \code{NA}\cr It is not recommended
to use this function directely but rather to use the nni() wrapper
function. The methods provides two ways for missing value
estimation, selected by the \code{allVariables} option. The first
one is to use only complete variables for the  regression. This is
preferable when the number of incomplete variables is relatively
small.

The second way is to consider all variables as candidates for the
regression.  Hereby missing values are initially replaced by the
columns wise mean.  The method then iterates, using the current
estimate as input for the regression until the change between new
and old estimate falls below a threshold (0.001).
}
\note{
Each step the generalized inverse of a \code{miss} x k
matrix is calculated. Where \code{miss} is the number of missing
values in  variable j and \code{k} the number of neighbours. This
may be slow for large values of k and / or many missing
values. See also help("ginv").
}
\examples{
## Load a sample metabolite dataset (metaboliteData) with already 5\\\% of
## data missing
data(metaboliteData)
## Perform llsImpute using k = 10
## Set allVariables TRUE because there are very few complete variables
result <- llsImpute(metaboliteData, k = 10, correlation="pearson", allVariables=TRUE)
## Get the estimated complete observations
cObs <- completeObs(result)
}
\references{
Kim, H. and Golub, G.H. and Park, H.  - Missing value
estimation for DNA microarray gene expression data: local least
squares imputation.  \emph{Bioinformatics, 2005; 21(2):187-198.}

Troyanskaya O. and Cantor M. and Sherlock G. and Brown P. and
Hastie T. and Tibshirani R. and Botstein D. and Altman RB.  -
Missing value estimation methods for DNA microarrays.
\emph{Bioinformatics. 2001 Jun;17(6):520-525.}
}
\seealso{
\code{\link{pca}, \link{nniRes}, \link{nni}}.
}
\author{
Wolfram Stacklies
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{completeObs,nniRes-method}
\alias{completeObs,nniRes-method}
\alias{completeObs}
\alias{completeObs,pcaRes-method}
\title{Get the original data with missing values replaced with predicted
values.}
\usage{
completeObs(object, ...)
}
\arguments{
\item{object}{object to fetch complete data from}

\item{...}{Not used}
}
\value{
Completed data (matrix)
}
\description{
Get the original data with missing values replaced with predicted
values.
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{nPcs,pcaRes-method}
\alias{nPcs,pcaRes-method}
\alias{nPcs}
\title{Get number of PCs.}
\usage{
nPcs(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{not used}
}
\value{
Number of PCs
}
\description{
Get number of PCs.
}
\note{
Try to use \code{link{nP}} instead since \code{nPcs} tend to
clash with argument names.
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optiAlgCgd.R
\name{optiAlgCgd}
\alias{optiAlgCgd}
\title{Conjugate gradient optimization}
\usage{
optiAlgCgd(nlnet, trainIn, trainOut, verbose = FALSE)
}
\arguments{
\item{nlnet}{The nlnet}

\item{trainIn}{Training data}

\item{trainOut}{fitted data}

\item{verbose}{logical, print messages}
}
\value{
...
}
\description{
Conjugate gradient optimization
}
\author{
Henning Redestig, Matthias Scholz
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lineSearch.R
\name{linr}
\alias{linr}
\title{Linear kernel}
\usage{
linr(x)
}
\arguments{
\item{x}{datum}
}
\value{
Input value
}
\description{
Linear kernel
}
\author{
Henning Redestig, Matthias Scholz
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{nmissing,pcaRes-method}
\alias{nmissing,pcaRes-method}
\alias{nmissing}
\alias{nmissing,nniRes-method}
\title{Missing values}
\usage{
nmissing(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{Not used}
}
\value{
Get the number of missing values
}
\description{
Missing values
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\name{dim.pcaRes}
\alias{dim.pcaRes}
\title{Dimensions of a PCA model}
\usage{
\method{dim}{pcaRes}(x)
}
\arguments{
\item{x}{a pcaRes object}
}
\value{
Get the dimensions of this PCA model
}
\description{
Dimensions of a PCA model
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{leverage,pcaRes-method}
\alias{leverage,pcaRes-method}
\alias{leverage}
\title{Extract leverages of a PCA model}
\usage{
\S4method{leverage}{pcaRes}(object)
}
\arguments{
\item{object}{a \code{pcaRes} object}
}
\value{
The observation leverages as a numeric vector
}
\description{
The leverages of PCA model indicate how much influence each
observation has on the PCA model. Observations with high leverage
has caused the principal components to rotate towards them. It can
be used to extract both "unimportant" observations as well as
picking potential outliers.
}
\details{
Defined as \eqn{Tr(T(T'T)^{-1}T')}{Tr(T(T'T)^(-1)T')}
}
\examples{
data(iris)
pcIr <- pca(iris[,1:4])
## versicolor has the lowest leverage
with(iris, plot(leverage(pcIr)~Species))
}
\references{
Introduction to Multi- and Megavariate Data Analysis
using Projection Methods (PCA and PLS), L. Eriksson, E. Johansson,
N. Kettaneh-Wold and S. Wold, Umetrics 1999, p. 466
}
\author{
Henning Redestig
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xval.R
\name{Q2}
\alias{Q2}
\title{Cross-validation for PCA}
\usage{
Q2(object, originalData = completeObs(object), fold = 5, nruncv = 1,
  type = c("krzanowski", "impute"), verbose = interactive(),
  variables = 1:nVar(object), ...)
}
\arguments{
\item{object}{A \code{pcaRes} object (result from previous PCA
analysis.)}

\item{originalData}{The matrix (or ExpressionSet) that used to
obtain the pcaRes object.}

\item{fold}{The number of groups to divide the data in.}

\item{nruncv}{The number of times to repeat the whole
cross-validation}

\item{type}{krzanowski or imputation type cross-validation}

\item{verbose}{\code{boolean} If TRUE Q2 outputs a primitive
progress bar.}

\item{variables}{indices of the variables to use during
cross-validation calculation. Other variables are kept as they are
and do not contribute to the total sum-of-squares.}

\item{...}{Further arguments passed to the \code{\link{pca}} function called
within Q2.}
}
\value{
A matrix or vector with \eqn{Q^2} estimates.
}
\description{
Internal cross-validation can be used for estimating the level of
structure in a data set and to optimise the choice of number of
principal components.
}
\details{
This method calculates \eqn{Q^2} for a PCA model. This is the
cross-validated version of \eqn{R^2} and can be interpreted as the
ratio of variance that can be predicted independently by the PCA
model. Poor (low) \eqn{Q^2} indicates that the PCA model only
describes noise and that the model is unrelated to the true data
structure. The definition of \eqn{Q^2} is: \deqn{Q^2=1 -
\frac{\sum_{i}^{k}\sum_{j}^{n}(x -
\hat{x})^2}{\sum_{i}^{k}\sum_{j}^{n}x^2}}{Q^2=1 - sum_i^k
sum_j^n (x - \hat{x})^2 / \sum_i^k \sum_j^n(x^2)} for the matrix
\eqn{x} which has \eqn{n} rows and \eqn{k} columns. For a given
number of PC's x is estimated as \eqn{\hat{x}=TP'} (T are scores
and P are loadings). Although this defines the leave-one-out
cross-validation this is  not what is performed if fold is less
than the number of rows and/or columns.  In 'impute' type CV,
diagonal rows of elements in the matrix are deleted and the
re-estimated.  In 'krzanowski' type CV, rows are sequentially left
out to build fold PCA models which give the loadings. Then,
columns are sequentially left out to build fold models for
scores. By combining scores and loadings from different models, we
can estimate completely left out values.  The two types may seem
similar but can give very different results, krzanowski typically
yields more stable and reliable result for estimating data
structure whereas impute is better for evaluating missing value
imputation performance. Note that since Krzanowski CV operates on
a reduced matrix, it is not possible estimate Q2 for all
components and the result vector may therefore be shorter than
\code{nPcs(object)}.
}
\examples{
data(iris)
x <- iris[,1:4]
pcIr <- pca(x, nPcs=3)
q2 <- Q2(pcIr, x)
barplot(q2, main="Krzanowski CV", xlab="Number of PCs", ylab=expression(Q^2))
## q2 for a single variable
Q2(pcIr, x, variables=2)
pcIr <- pca(x, nPcs=3, method="nipals")
q2 <- Q2(pcIr, x, type="impute")
barplot(q2, main="Imputation CV", xlab="Number of PCs", ylab=expression(Q^2))
}
\references{
Krzanowski, WJ. Cross-validation in principal
component analysis. Biometrics. 1987(43):3,575-584
}
\author{
Henning Redestig, Ondrej Mikula
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcaMethods-package.R
\docType{data}
\name{metaboliteDataComplete}
\alias{metaboliteDataComplete}
\title{A complete metabolite data set from an Arabidopsis
coldstress experiment}
\description{
A complete subset from a larger metabolite data set.  This is the
original, complete data set and can be used to compare estimation
results created with the also provided incomplete data (called
metaboliteData).  The data was created during an in house
Arabidopsis coldstress experiment.
}
\details{
A matrix containing 154 observations (rows) and 52 metabolites
(columns).
}
\references{
Matthias Scholz, Fatma Kaplan, Charles L. Guy, Joachim
Kopka and Joachim Selbig. - Non-linear PCA: a missing data
approach.\emph{Bioinformatics 2005 21(20):3887-3895}
}
\seealso{
\code{\link{metaboliteData}}
}
\author{
Wolfram Stacklies
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lineSearch.R
\name{lineSearch}
\alias{lineSearch}
\title{Line search for conjugate gradient}
\usage{
lineSearch(nlnet, dw, e0, ttGuess, trainIn, trainOut, verbose)
}
\arguments{
\item{nlnet}{The nlnet}

\item{dw}{..}

\item{e0}{..}

\item{ttGuess}{..}

\item{trainIn}{Training data}

\item{trainOut}{Fitted data}

\item{verbose}{logical, print messages}
}
\value{
...
}
\description{
Line search for conjugate gradient
}
\author{
Henning Redestig, Matthias Scholz
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xval.R
\name{tempFixNas}
\alias{tempFixNas}
\title{Temporary fix for missing values}
\usage{
tempFixNas(mat)
}
\arguments{
\item{mat}{a matrix}
}
\value{
The original matrix with completely missing rows/cols
filled with zeroes.
}
\description{
Simply replace completely missing rows or cols with zeroes.
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/repmat.R
\name{repmat}
\alias{repmat}
\title{Replicate and tile an array.}
\usage{
repmat(mat, M, N)
}
\arguments{
\item{mat}{numeric matrix}

\item{M}{number of copies in vertical direction}

\item{N}{number of copies in horizontal direction}
}
\value{
Matrix consiting of M-by-N tiling copies of input matrix
}
\description{
Creates a large matrix B consisting of an M-by-N tiling of copies
of A
}
\author{
Wolfram Stacklies
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{simpleEllipse}
\alias{simpleEllipse}
\title{Hotelling's T^2 Ellipse}
\usage{
simpleEllipse(x, y, alfa = 0.95, len = 200)
}
\arguments{
\item{x}{first variable}

\item{y}{second variable}

\item{alfa}{confidence level of the circle}

\item{len}{Number of points in the circle}
}
\value{
A matrix with X and Y coordinates for the circle
}
\description{
Get a confidence ellipse for uncorrelated bivariate data
}
\details{
As described in 'Introduction to multi and megavariate data analysis
using PCA and
PLS' by Eriksson et al. This produces very similar ellipse as
compared to the ellipse function the ellipse package except that
this function assumes that and y are uncorrelated (which they of
are if they are scores or loadings from a PCA).
}
\seealso{
ellipse
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{scl,pcaRes-method}
\alias{scl,pcaRes-method}
\alias{scl}
\title{Get the scales (e.g. standard deviations) of the original
variables}
\usage{
scl(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{Not used}
}
\value{
Vector with the scales
}
\description{
Get the scales (e.g. standard deviations) of the original
variables
}
\seealso{
\code{\link{prep}}
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcaMethods-package.R
\name{pcaMethods-deprecated}
\alias{pcaMethods-deprecated}
\title{Deprecated methods for pcaMethods}
\description{
\describe{
\item{plotR2}{Lack of relevance for this plot and the fact that it
can not show cross-validation based diagnostics in the same plot
makes it redundant with the introduction of a dedicated
\code{plot} function for  \code{pcaRes}. The new plot only shows
R2cum but the result is pretty much the same.}}
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcaMethods-package.R
\docType{data}
\name{helix}
\alias{helix}
\title{A helix structured toy data set}
\usage{
data(helix)
}
\description{
Simulated data set looking like a helix
}
\details{
A matrix containing 1000 observations (rows) and three variables
(columns).
}
\references{
Matthias Scholz, Fatma Kaplan, Charles L. Guy, Joachim
Kopka and Joachim Selbig. - Non-linear PCA: a missing data
approach.  \emph{Bioinformatics 2005 21(20):3887-3895}
}
\author{
Henning Redestig
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xval.R
\name{deletediagonals}
\alias{deletediagonals}
\title{Delete diagonals}
\usage{
deletediagonals(x, diagonals = 1)
}
\arguments{
\item{x}{The matrix}

\item{diagonals}{The diagonal to be replaced, i.e. the first,
second and so on when looking at the fat version of the matrix
(transposed or not) counting from the bottom.
Can be a vector to delete more than one diagonal.}
}
\value{
The original matrix with some values missing
}
\description{
Replace a diagonal of elements of a matrix with NA
}
\details{
Used for creating artifical missing values in matrices without
causing any full row or column to be completely missing
}
\author{
Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nipalsPca.R
\name{nipalsPca}
\alias{nipalsPca}
\title{NIPALS PCA}
\usage{
nipalsPca(Matrix, nPcs = 2, varLimit = 1, maxSteps = 5000,
  threshold = 1e-06, ...)
}
\arguments{
\item{Matrix}{Pre-processed (centered, scaled) numerical matrix
samples in rows and variables as columns.}

\item{nPcs}{Number of components that should be extracted.}

\item{varLimit}{Optionally the ratio of variance that should be
explained. \code{nPcs} is ignored if varLimit < 1}

\item{maxSteps}{Defines how many iterations can be done before
algorithm should abort (happens almost exclusively when there were
some wrong in the input data).}

\item{threshold}{The limit condition for judging if the algorithm
has converged or not, specifically if a new iteration is done if
\eqn{(T_{old} - T)^T(T_{old} - T) > \code{limit}}.}

\item{...}{Only used for passing through arguments.}
}
\value{
A \code{pcaRes} object.
}
\description{
PCA by non-linear iterative partial least squares
}
\details{
Can be used for computing PCA on a numeric matrix using either the
NIPALS algorithm which is an iterative approach for estimating the
principal components extracting them one at a time. NIPALS can
handle a small amount of missing values. It is not recommended to
use this function directely but rather to use the pca() wrapper
function.
}
\examples{
data(metaboliteData)
mat <- prep(t(metaboliteData))
pc <- nipalsPca(mat, nPcs=2)
## better use pca()
pc <- pca(t(metaboliteData), method="nipals", nPcs=2)
\dontshow{stopifnot(sum((fitted(pc) - t(metaboliteData))^2, na.rm=TRUE) < 200)}
}
\references{
Wold, H. (1966) Estimation of principal components and
related models by iterative least squares. In Multivariate
Analysis (Ed., P.R. Krishnaiah), Academic Press, NY, 391-420.
}
\seealso{
\code{prcomp}, \code{princomp}, \code{pca}
}
\author{
Henning Redestig
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nlpca.R
\name{nlpca}
\alias{nlpca}
\title{Non-linear PCA}
\usage{
nlpca(Matrix, nPcs = 2, maxSteps = 2 * prod(dim(Matrix)),
  unitsPerLayer = NULL, functionsPerLayer = NULL,
  weightDecay = 0.001, weights = NULL, verbose = interactive(), ...)
}
\arguments{
\item{Matrix}{\code{matrix} --- Preprocessed data with the
variables in columns and observations in rows. The data may
contain missing values, denoted as \code{NA}}

\item{nPcs}{\code{numeric} -- Number of components to
estimate. The preciseness of the missing value estimation depends
on thenumber of components, which should resemble the internal
structure of the data.}

\item{maxSteps}{\code{numeric} -- Number of estimation
steps. Default is based on a generous rule of thumb.}

\item{unitsPerLayer}{The network units, example: c(2,4,6) for two
input units 2feature units (principal components), one hidden
layer fornon-linearity and three output units (original amount
ofvariables).}

\item{functionsPerLayer}{The function to apply at each layer
eg. c("linr", "tanh", "linr")}

\item{weightDecay}{Value between 0 and 1.}

\item{weights}{Starting weights for the network. Defaults to
uniform random values but can be set specifically to make
algorithm deterministic.}

\item{verbose}{\code{boolean} -- nlpca prints the number of steps
and warning messages if set to TRUE. Default is interactive().}

\item{...}{Reserved for future use. Not passed on anywhere.}
}
\value{
Standard PCA result object used by all PCA-basedmethods of
this package. Contains scores, loadings, data meanand more. See
\code{\link{pcaRes}} for details.
}
\description{
Neural network based non-linear PCA
}
\details{
Artificial Neural Network (MLP) for performing non-linear
PCA. Non-linear PCA is conceptually similar to classical PCA but
theoretically quite different. Instead of simply decomposing our
matrix (X) to scores (T) loadings (P) and an error (E) we train a
neural network (our loadings) to find a curve through the
multidimensional space of X that describes a much variance as
possible. Classical ways of interpreting PCA results are thus not
applicable to NLPCA since the loadings are hidden in the network.
However, the scores of components that lead to low
cross-validation errors can still be interpreted via the score
plot.  Unfortunately this method depend on slow iterations which
currently are implemented in R only making this method extremely
slow. Furthermore, the algorithm does not by itself decide when it
has converged but simply does 'maxSteps' iterations.
}
\examples{
## Data set with three variables where data points constitute a helix
data(helix)
helixNA <- helix
## not a single complete observation
helixNA <- t(apply(helix, 1, function(x) { x[sample(1:3, 1)] <- NA; x}))
## 50 steps is not enough, for good estimation use 1000
helixNlPca <- pca(helixNA, nPcs=1, method="nlpca", maxSteps=50)
fittedData <- fitted(helixNlPca, helixNA)
plot(fittedData[which(is.na(helixNA))], helix[which(is.na(helixNA))])
## compared to solution by Nipals PCA which cannot extract non-linear patterns
helixNipPca <- pca(helixNA, nPcs=2)
fittedData <- fitted(helixNipPca)
plot(fittedData[which(is.na(helixNA))], helix[which(is.na(helixNA))])
}
\references{
Matthias Scholz, Fatma Kaplan, Charles L Guy, Joachim
Kopkaand Joachim Selbig. Non-linear PCA: a missing
data approach. \emph{Bioinformatics, 21(20):3887-3895, Oct 2005}
}
\author{
Based on a matlab script by Matthias Scholz and ported to
R by Henning Redestig
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{rediduals-methods}
\alias{rediduals-methods}
\alias{residuals.pcaRes}
\alias{residuals,pcaRes-method}
\alias{resid,pcaRes-method}
\title{Residuals values from a PCA model.}
\usage{
\method{residuals}{pcaRes}(object, data = completeObs(object), ...)

\S4method{residuals}{pcaRes}(object, data = completeObs(object), ...)

\S4method{resid}{pcaRes}(object, data = completeObs(object), ...)
}
\arguments{
\item{object}{\code{pcaRes} the \code{pcaRes} object of interest.}

\item{data}{\code{matrix} The data that was used to calculate the
PCA model (or a different dataset to e.g. adress its proximity to
the model).}

\item{...}{Passed on to \code{\link{predict.pcaRes}}. E.g. setting
the number of used components.}
}
\value{
A \code{matrix} with the residuals
}
\description{
This function extracts the residuals values from a pcaRes object
for the PCA methods SVD, Nipals, PPCA and BPCA
}
\examples{
data(iris)
pcIr <- pca(iris[,1:4])
head(residuals(pcIr, iris[,1:4]))
}
\author{
Henning Redestig
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orth.R
\name{orth}
\alias{orth}
\title{Calculate an orthonormal basis}
\usage{
orth(mat, skipInac = FALSE)
}
\arguments{
\item{mat}{matrix to calculate orthonormal base}

\item{skipInac}{do not include components with precision below
.Machine$double.eps if TRUE}
}
\value{
orthonormal basis for the range of matrix
}
\description{
ONB = orth(mat) is an orthonormal basis for the range of matrix
mat.  That is, ONB' * ONB = I, the columns of ONB span the same
space as the columns of mat, and the number of columns of ONB is
the rank of mat.
}
\author{
Wolfram Stacklies
}
